/***************************************************************************
                          CdboCreateDataCdAction.cpp  -  description
                             -------------------
    begin                : Sat Sep 14 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qtimer.h>
#include <qregexp.h>

#include <kconfig.h>
#include <kprocess.h>
#include <klocale.h>

#include "CdboCreateDataCdAction.h"
#include "CdboCreateDataCdAction.moc"

#include "CdboConstants.h"

CdboCreateDataCdAction::CdboCreateDataCdAction(KLibFactory* factory, QObject* d, const char* name) :
	CdboActionHandler(factory, d, name){

}
//---------------------------------------------------------------------------------------------
CdboCreateDataCdAction::~CdboCreateDataCdAction(){

}
//---------------------------------------------------------------------------------------------
void CdboCreateDataCdAction::initProcess(KProcess *, CdboParamsDict* params){
	readyToLaunch = false;
	this->params = params;

/////////////////////////////////////////////////////////////////////////////////////////////

	bool createImage = false;
	boolParamVal("IS_CREATE_IMAGE", createImage, false);
	
	bool createImageOnly = false;
	boolParamVal("IS_CREATE_IMAGE_ONLY", createImageOnly, false);

	onTheFly = false;
	boolParamVal("IS_ON_THE_FLY_CREATE_DATA", onTheFly, false);

	//isoImageName = paramVal("IMAGE_NAME", false);
	isoImageName = dynamic_cast<QString *>((*params)["IMAGE_NAME"]);
	if(isoImageName == NULL || isoImageName->isNull()){
		isoImageName = new QString("");
		setParamVal("IMAGE_NAME", isoImageName);
	}
	if(isoImageName->isEmpty()){
		if(onTheFly)
			*isoImageName = "-";
		else{
			QString tmpDir = paramVal("TMP_DIRECTORY");
			if(tmpDir.isEmpty())
				return;
			config->setGroup("Default Settings");
			*isoImageName = config->readEntry("Custom ISO Name", "cdboImage_<DATE>.iso");
			if(isoImageName->contains("<DATE>"))
				isoImageName->replace(QRegExp("<DATE>"), timeStamp());
			*isoImageName = tmpDir + *isoImageName;
		}
	}
	setParamVal("SESSION_TYPE", &sessionTypeHolder);
	isCreatingNewCD = "TRUE";
	setParamVal("CREATING_NEW_CD", &isCreatingNewCD);
	
	withCleanUp = true;
	boolParamVal("IS_WITH_CLEAN_UP", withCleanUp, false);


/////////////////////////////////////////////////////////////////////////////////////////////


	config->setGroup("Default Settings");
	if(!onTheFly && config->readBoolEntry("Check Tmp Size", true)){
		if(!createAction("CdboCheckTmpAction")){
			processFailed();
			return;
		}
	}
	config->setGroup("Recording Options");
	if(!createImageOnly && config->readBoolEntry("Check Drive First", false)){
		if(!createAction("CdboCheckDriveAction")){
			processFailed();
			return;
		}
	}
	if(onTheFly){
		if(!createAction("CdboCreateDataOTFAction")){
			processFailed();
			return;
		}
	}
	else{
		if(createImage){
			if(!createAction("CdboMkisofsAction")){
				processFailed();
				return;
			}
		}
		if(!createImageOnly){
			if(!createAction("CdboCdrecordAction")){
				processFailed();
				return;
			}
		}
		else
			copiesDone = copiesWanted;
	}

	
	QTimer::singleShot(1, this, SLOT(begin()));
}
//---------------------------------------------------------------------------------------------
void CdboCreateDataCdAction::processFailed(){
	emitOutput(Cdbo::StatusNotOkMessage, i18n("Failed to create Data CD."));
	CdboAction::processFailed();
}
//---------------------------------------------------------------------------------------------
void CdboCreateDataCdAction::resetSettings(){
	CdboActionHandler::resetSettings();
	if(withCleanUp){
		QString image = paramVal("IMAGE_NAME");
		if(image.isEmpty() || image == "-")
			return;
		removeOneFile(image);
	}
}
