/***************************************************************************
                          CdboEraseAction.cpp  -  description
                             -------------------
    begin                : Fri Aug 16 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qtimer.h>

#include <kconfig.h>
#include <kprocess.h>
#include <klocale.h>
#include <krun.h>

#include "CdboEraseAction.h"
#include "CdboEraseAction.moc"

#include "CdboConstants.h"

CdboEraseAction::CdboEraseAction(KLibFactory* factory, QObject* d, const char* name) :
	CdboActionHandler(factory, d, name){

}
//---------------------------------------------------------------------------------------------
CdboEraseAction::~CdboEraseAction(){

}
//---------------------------------------------------------------------------------------------
void CdboEraseAction::initProcess(KProcess *, CdboParamsDict* params){
	readyToLaunch = false;
	this->params = params;
	
	config->setGroup("Erase Options");
	if(config->readBoolEntry("Check Drive First", false)){
		if(!createAction("CdboCheckDriveAction")){
			processFailed();
			return;
		}
	}
	if(!createAction("CdboEraseProcAction")){
		processFailed();
		return;
	}
	QTimer::singleShot(1, this, SLOT(begin()));
}
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


CdboEraseProcAction::CdboEraseProcAction( QObject* d, const char* name) :
	CdboAction(d, name){
}
//---------------------------------------------------------------------------------------------
CdboEraseProcAction::~CdboEraseProcAction(){
}
//---------------------------------------------------------------------------------------------
void CdboEraseProcAction::initProcess(KProcess *proc, CdboParamsDict* /*params*/){

/////////////////////////////////////////////////////////////////////////////////////////////
	
	QString device = paramVal("TARGET_DEVICE");
	if(device.isEmpty())
		return;

	QString actionType = paramVal("ACTION_TYPE");
	if(actionType.isEmpty())
		return;

	QString speed = paramVal("SPEED");
	if(speed.isEmpty())
		return;

	bool forceBlank = false;
	boolParamVal("FORCE_BLANK", forceBlank, false);
/////////////////////////////////////////////////////////////////////////////////////////////

	emitStatus(i18n("Erasing..."));

	config->setGroup("Wizard Detected");
	QString exe = config->readEntry("Path to cdrecord", "cdrecord");
	KRun::shellQuote(exe);
	*proc << exe;

	config->setGroup("Default Settings");
	if(config->readNumEntry("Level Of Output", Cdbo::StandartOutput) == Cdbo::CustomOutput){
		switch(config->readNumEntry("Custom Level Of Output", 1)){
			case 0: *proc  << "-s"; break;
			case 1: *proc  << "-v"; break;
			case 2: *proc  << "-V"; break;
			case 3: *proc  << "-VV"; break;
		}
		if(config->readBoolEntry("Show Debug Info", false))
			*proc  << "-d";
	}
	else
		*proc  << "-v";

	config->setGroup("Erase Options");
	if(config->readBoolEntry("Dummy Erase", false)){
		emitStatus(i18n("Erasing (Test Mode)..."));
		*proc  << "-dummy";
	}

     if(config->readBoolEntry("Eject After Erase", true))
		*proc  << "-eject";

	if(forceBlank)
		*proc  << "-force";
	*proc  << "blank=" + actionType;
	*proc  << "speed=" + speed;
	*proc  << "dev=" + device;
}
//---------------------------------------------------------------------------------------------
bool CdboEraseProcAction::filterOutput(int &/*type*/, QString &message){
	if(message.find("Re-load disk and hit") != -1){
		requestReload();
		return false;
	}
	return true;
}
//---------------------------------------------------------------------------------------------
void CdboEraseProcAction::processLaunched(){
	emitOutput(Cdbo::StatusOkMessage, i18n("Starting erase process."));
	CdboAction::processLaunched();
}
//---------------------------------------------------------------------------------------------
void CdboEraseProcAction::processFailedToLaunch(){
	emitOutput(Cdbo::ErrorMessage,
		i18n("Erase process failed to start. Make sure you have enough privileges."));
	CdboAction::processFailedToLaunch();
}
//---------------------------------------------------------------------------------------------
void CdboEraseProcAction::processSucceded(){
	emitOutput(Cdbo::StatusOkMessage, i18n("Erase completed successfully."));
	CdboAction::processSucceded();
}
//---------------------------------------------------------------------------------------------
void CdboEraseProcAction::processFailed(){
	emitOutput(Cdbo::StatusNotOkMessage, i18n("Erase failed."));
	CdboAction::processFailed();
}
