/***************************************************************************
                          CdboRipAction.cpp  -  description
                             -------------------
    begin                : Sat Aug 17 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qstringlist.h>

#include <kconfig.h>
#include <klocale.h>
#include <kio/job.h>
#include <kurl.h>

#include "CdboRipAction.h"
#include "CdboRipAction.moc"

#include "CdboConstants.h"

CdboRipAction::CdboRipAction( QObject* d, const char* name) :
	CdboJobBasedAction(d, name){
}
//---------------------------------------------------------------------------------------------
CdboRipAction::~CdboRipAction(){

}
//---------------------------------------------------------------------------------------------
void CdboRipAction::launchJob(CdboParamsDict* /*params*/){

/////////////////////////////////////////////////////////////////////////////////////////////

	QString device = paramVal("SOURCE_DEVICE");
	if(device.isEmpty())
		return;

	QStringList tracks;
	if(!listParamVal("PICKED_TRACKS", tracks))
		return;

	QString ext = paramVal("ACTION_TYPE");
	if(ext.isEmpty())
		return;

	QString outputDir = paramVal("TMP_DIRECTORY");
	if(outputDir.isEmpty())
		return;

/////////////////////////////////////////////////////////////////////////////////////////////

	KURL::List urlList;
	for(int i = 0; i < (int)tracks.count(); i++){
		urlList.append( "audiocd:" + tracks[i].stripWhiteSpace() + "." + ext + "?device=" + device );
		//cerr <<  "audiocd:" + ((*tracks)[i]).stripWhiteSpace() + "." + ext + "?device=" + device << endl;
	}
	emitOutput(Cdbo::StatusOkMessage, i18n("Ripping audio CD."));
	emitOutput(Cdbo::StatusOkMessage, i18n("Outputting files to %1.").arg(outputDir));

	emitStatus(i18n("Ripping audio CD..."));
	totalToProcess =  tracks.count();
	processesDone = 1;

	emitPercentInfo("CdboRipAction", i18n("Ripping track 1 of %1, percent done:").arg(QString::number(totalToProcess)) );
	emitPercent("CdboRipAction", 0);


	curJob = KIO::copy( urlList, "file:" + outputDir, false );

	connect( curJob, SIGNAL(result( KIO::Job * )), this, SLOT( rippResult( KIO::Job *)) );
	connect( curJob, SIGNAL(canceled( KIO::Job * )), this, SLOT( rippCanceled( KIO::Job *)) );
	connect( curJob, SIGNAL(infoMessage( KIO::Job *, const QString &)),
			this, SLOT( rippMessage( KIO::Job *,const QString &)) );
	connect( curJob, SIGNAL(percent( KIO::Job *,unsigned long)),
			this, SLOT( rippPercent( KIO::Job *,unsigned long)) );

	//connect( curJob, SIGNAL(processedFiles( KIO::Job *,unsigned long)),
		//	this, SLOT( rippProcessedFiles( KIO::Job *,unsigned long)) );
	connect( curJob, SIGNAL(copyingDone ( KIO::Job *, const KURL &, const KURL &, bool, bool)),
			this, SLOT( rippedFile( KIO::Job *, const KURL &, const KURL &, bool, bool )) );

	enableTimeout();
}
//=================================================
void CdboRipAction::rippMessage( KIO::Job *,const QString &msg){
	emitOutput(Cdbo::ProcessMessage, msg);
}
//=================================================
void CdboRipAction::rippPercent( KIO::Job *,unsigned long p){
	resetTimeout();
	emitPercent("CdboRipAction", (int)p);
}
//=================================================
void CdboRipAction::rippProcessedFiles( KIO::Job *,unsigned long ){
	resetTimeout();
	processesDone++;
	emitPercentInfo("CdboRipAction", i18n("Ripping track %1 of %2, percent done:").arg(
			QString::number(processesDone)).arg(QString::number(totalToProcess)) );
}
//=================================================
void CdboRipAction::rippedFile ( KIO::Job *, const KURL &, const KURL &, bool, bool ){
	resetTimeout();
	processesDone++;
	emitPercentInfo("CdboRipAction", i18n("Ripping track %1 of %2, percent done:").arg(
			QString::number(processesDone)).arg(QString::number(totalToProcess)) );
}
//=================================================
void CdboRipAction::rippResult( KIO::Job * job){
	if(job->error() && !canceled){
		emitOutput(Cdbo::StatusNotOkMessage, i18n("Operation exited with the following error: "));
		emitOutput(Cdbo::StatusNotOkMessage, job->errorText());
		processFailed();
	}
	else
		processSucceded();
}
//=================================================
void CdboRipAction::rippCanceled( KIO::Job * ){
	CdboAction::processCancelledByUser();
	emitOutput(Cdbo::InfoMessage, i18n("Files ripped up to this point will remain on your disk."));
}
//---------------------------------------------------------------------------------------------
void CdboRipAction::processFailedToLaunch(){
	CdboAction::processFailedToLaunch();
	killCurrentTask();

	emitOutput(Cdbo::StatusNotOkMessage, i18n("Operation timed out."));
	emitOutput(Cdbo::ErrorMessage, i18n("No responce recieved from audiocd slave."));
	emitOutput(Cdbo::InfoMessage, i18n("Try to use another device."));
	emitOutput(Cdbo::InfoMessage, i18n("Files ripped up to this point will remain on your disk."));
}
//---------------------------------------------------------------------------------------------
void CdboRipAction::processSucceded(){
	CdboAction::processSucceded();
	emitOutput(Cdbo::StatusOkMessage, i18n("Operation completed successfully."));
	emitPercent("CdboRipAction", 100);
}
//---------------------------------------------------------------------------------------------
void CdboRipAction::resetSettings(){
	resetTimeout();
	totalToProcess = 0;
	CdboAction::resetSettings();
}
//---------------------------------------------------------------------------------------------
QString CdboRipAction::cancelMessage(){
	return i18n("This will abort the rip process.");
}

