/***************************************************************************
                          CdboCopyCDDialog.cpp  -  description
                             -------------------
    begin                : Sat May 5 2001
    copyright            : (C) 2001 by Alexy Podolsky
    email                :  alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qtooltip.h>
#include <qspinbox.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qwhatsthis.h>

#include <kconfig.h>
#include <kdebug.h>
#include <kmessagebox.h>

#include "CdboCopyCDDialog.h"
#include "CdboCopyCDDialog.moc"

#include "CdboDriveInfoPnl.h"
#include "CdboSpeedPnl.h"
#include "CdboCddaView.h"

#include "CdboEraseCDDialog.h"
#include "CdboPreviewDialog.h"

#include "CdboConstants.h"

#define COPY 0
#define ON_THE_FLY_COPY 1
#define FORCE_COPY 2
#define TAO_COPY 2
#define TEST_COPY 3
#define TEST_ON_THE_FLY_COPY 4
#define TEST_FORCE_COPY 5
#define TEST_TAO_COPY 5

#define AUDIO 0
#define DATA 2
#define VCD 1

//---------------------------------------------------------------------------------------------
CdboCopyCDDialog::CdboCopyCDDialog(QWidget *parent, const char *name ) :
	CdboCopyCDBase(parent,name,true),
	DCOPObject( "CdboDialogBase" ){
	init();

}
//=======================================
void CdboCopyCDDialog::init(){
	CdboDialogBase::init();
	
	prevType = -1;

	///////////////////////////////////////////////////
	targetDevicePnl->setType(CdboDriveInfoPnl::Target);
	targetDevicePnl->setUsrBtnTitle(i18n("&Erase"));
	/////////////////////////////////////////////////
	srcDevicePnl->setType(CdboDriveInfoPnl::Source);
	srcDevicePnl->setUsrBtnTitle(i18n("&Preview"));
	////////////////////////////////////////////////
	dialogCntPnl->setStartBtnTitle(i18n("C&opy"));
	QToolTip::add(dialogCntPnl->getStartBtn(), i18n( "Start copying." ) );
	/////////////////////////////////////////////////

	readOptions();
	targetDevicePnl->closeTray();
	srcDevicePnl->closeTray();
	///////////////////////////////////////////////

	// connecting to the Drive Info Panel
	connect( targetDevicePnl->getDetectBtn(), SIGNAL( clicked() ), this, SLOT( detectSlot() ) );
	connect( targetDevicePnl->getUsrBtn(), SIGNAL( clicked() ), this, SLOT( eraseSlot() ) );
	connect( targetDevicePnl, SIGNAL( selectedChaged() ), this, SLOT( verifyMode() ) );
	

	// connecting to the Src Drive Info Panel
	connect( srcDevicePnl->getDetectBtn(), SIGNAL( clicked() ), this, SLOT( detectSlot() ) );
	connect( srcDevicePnl->getUsrBtn(), SIGNAL( clicked() ), this, SLOT( previewSlot() ) );
	connect( srcDevicePnl, SIGNAL( trayEjected() ), trackView, SLOT( invalidate() ) );
	connect( srcDevicePnl, SIGNAL( selectedChaged() ), trackView, SLOT( invalidate() ) );
	connect( srcDevicePnl, SIGNAL( selectedChaged() ), this, SLOT( verifyMode() ) );

	// connecting to the Dialog Control Panel
	connect( dialogCntPnl->getStartBtn(), SIGNAL( clicked() ), this, SLOT( startBtnSlot() ) );
	connect( dialogCntPnl->getCancelBtn(), SIGNAL( clicked() ), this, SLOT( cancelSlot() ) );
	connect( dialogCntPnl->getSettingsBtn(), SIGNAL( clicked() ), this, SLOT( settingsSlot() ) );
	connect( dialogCntPnl->getQuitBtn(), SIGNAL( clicked() ), this, SLOT( quitSlot() ) );

	connect( typeCmb, SIGNAL( activated(int) ), this, SLOT( typeChangedSlot(int) ) );
	connect( actionCmb, SIGNAL( activated(int) ), this, SLOT( verifyMode(int) ) );
}
//---------------------------------------------------------------------------------------------
CdboCopyCDDialog::~CdboCopyCDDialog(){
}
//---------------------------------------------------------------------------------------------
void CdboCopyCDDialog::saveOptions(){
	if(!config)
		return;
	//////////////////////////////////////////////////////////
	config->setGroup("Copy Options");
	config->writeEntry("Type of CD", typeCmb->currentItem());
	config->writeEntry("Number of Copies", numberSpn->value());
	config->writeEntry("Type of Copy " + QString::number(typeCmb->currentItem()), actionCmb->currentItem());
	config->writeEntry("Remove Tmp File", removeTmpChk->isChecked());

	speedPnl->saveOptions();
 	targetDevicePnl->saveOptions();
 	srcDevicePnl->saveOptions();

 	config->sync();
}
//---------------------------------------------------------------------------------------------
void CdboCopyCDDialog::readOptions(){
	saveOptions();
	CdboDialogBase::readOptions();

	config->setGroup("Default Settings");
	tempDir = config->readEntry("Tmp Dir", "/tmp/");
	minTmpSize = config->readEntry("Minimum Tmp Size", "650");

	config->setGroup("Copy Options");
	removeTmpChk->setChecked(config->readBoolEntry("Remove Tmp File", true));
	numberSpn->setValue( config->readNumEntry("Number of Copies", 1) );
	typeCmb->setCurrentItem(config->readNumEntry("Type of CD", VCD));

	speedPnl->readOptions();
	targetDevicePnl->readOptions();
	srcDevicePnl->readOptions();
	initBtns();
}
//---------------------------------------------------------------------------------------------
void CdboCopyCDDialog::initBtns(){
	if(targetDevicePnl->isReady()){
		blockBtns(false);
		dialogCntPnl->status(i18n("Ready"));
	}
	else{
		blockBtns(true);
		dialogCntPnl->getCancelBtn()->setEnabled(false);
		if(srcDevicePnl->isReady()){
			srcDevicePnl->blockBtns(false);
		}
		dialogCntPnl->status(i18n("No recording device configured."));
	}
	typeChangedSlot(typeCmb->currentItem());
}
//---------------------------------------------------------------------------------------------
void CdboCopyCDDialog::blockBtns(bool yes){
	dialogCntPnl->getStartBtn()->setEnabled(!yes);
	dialogCntPnl->getCancelBtn()->setEnabled(yes);
	trackView->setEnabled(!yes);
	detailsFrm->setEnabled(!yes);
	targetDevicePnl->blockBtns(yes);
	srcDevicePnl->blockBtns(yes);
	if(doingSomething()){
		targetDevicePnl->getDetectBtn()->setEnabled(false);
		srcDevicePnl->getDetectBtn()->setEnabled(false);
		dialogCntPnl->getSettingsBtn()->setEnabled(false);
	}
	else{
		targetDevicePnl->getDetectBtn()->setEnabled(true);
		srcDevicePnl->getDetectBtn()->setEnabled(true);
		dialogCntPnl->getSettingsBtn()->setEnabled(true);
	}
}
//======================================================
void CdboCopyCDDialog::typeChangedSlot(int type){
	//clearOutput();
	switch(type){
		case DATA: dataSlot(); break;
		case AUDIO: musicSlot(); break;
		case VCD: dataSlot(); break;
	}
	restructureCopyOptions();
	prevType = type;
}
//======================================================
void CdboCopyCDDialog::restructureCopyOptions(){
	config->setGroup("Copy Options");
	if(prevType != -1){
		config->writeEntry("Type of Copy " + QString::number(prevType), actionCmb->currentItem());
		config->sync();
	}
	int last = config->readNumEntry("Type of Copy " + QString::number(typeCmb->currentItem(), 0));
	
	actionCmb->clear();
	QWhatsThis::remove(actionCmb);
	QString whatsThis = i18n("Choose how and what type of CD should be created.");
	
	actionCmb->insertItem( i18n( "Copy CD in Safe Mode" ) );
	whatsThis += i18n("<br><br><b>Copy CD in Safe Mode:</b> In this mode process will first read out source CD as an image file(s) to your hard drive "\
	"and then burn it to target CD-R(W). This mode decreases chance of buffer underruns. Working directory must have sufficient space "\
	"available to store image file(s). If not sure, use this mode.");
	
	actionCmb->insertItem( i18n( "Copy CD in Fast Mode" ) );
	whatsThis += i18n("<br><br><b>Copy CD in Fast Mode:</b> Perform CD copy on the fly without creating an image file. "\
	"Success of such copy depends on your machine capabilities and system load. Higher system load espesially on weaker machines "\
	"increases possibility of failure due to buffer underruns. This is however the fastest way to copy CD.");
	
	actionCmb->insertItem( i18n( "Test Safe Mode" ) );
	actionCmb->insertItem( i18n( "Test Fast Mode" ) );

	switch(typeCmb->currentItem()){
		case DATA: 
			actionCmb->insertItem( i18n( "Force Copy" ), FORCE_COPY);
			whatsThis += i18n("<br><br><b>Force Copy:</b> In this mode process will try to read out source Data CD even on some errors. "\
				"Use when Safe and Fast modes fail.");
			actionCmb->insertItem( i18n( "Test Force Mode" ), TEST_FORCE_COPY); 
			break;
		case AUDIO: 
			actionCmb->insertItem( i18n( "Copy CD in Track-at-Once Mode" ), TAO_COPY);
			actionCmb->insertItem( i18n( "Test Track-at-Once Mode" ), TEST_TAO_COPY); 
			whatsThis +=  i18n("<br><br><b>Copy CD in Track-at-Once Mode:</b> Will append tracks to an empty CD-R(W) or an existing Audio CD in Track-at-Once Mode. "\
				"If not finalized, it will be possible to add Tracks to such CD at a later time (provided there is enough space left). "\
				"However, CD Text can not be added and CD layout can not be manipulated. "\
				"To append to an existing Audio CD it must itself be in Track-at-Once Mode and not finalized (fixated).");
			break;
		case VCD: break;
	}
	
	whatsThis += i18n("<br><br><b>Test ... :</b> Same as coresponding real Mode (one that does not start with test), "\
		"but laser will stay cool - nothing will be actually burned. Allows testing without possibility of corrupting recordable media.");
	
	whatsThis += i18n("<br><br><b>Please Note:</b> Some modes are only available if Source device is recognized as SCSI device.");
	QWhatsThis::add(actionCmb, whatsThis);
	actionCmb->setCurrentItem(last);
	verifyMode();
}
//======================================================
void CdboCopyCDDialog::verifyMode(){
	if(!targetDevicePnl->isReady() || !srcDevicePnl->isReady() || doingSomething())
		return;
		
	int action = actionCmb->currentItem();
	if(typeCmb->currentItem() == VCD && action > 1)
		action += 1;
		
	//if on-the-fly -> can not use same device
	if(action == ON_THE_FLY_COPY || action == TEST_ON_THE_FLY_COPY){
		if(srcDevicePnl->getDriveName() == targetDevicePnl->getDriveName()){
			dialogCntPnl->getStartBtn()->setEnabled(false);
			dialogCntPnl->status(i18n("Can not use same device for this mode..."));
			return;
		}
	}
	//must use scsi for some instances:
	if(srcDevicePnl->getScsiDevice().isNull()){
		bool invalid = false;
		//if data type and force copy
		if(typeCmb->currentItem() == DATA && (action == FORCE_COPY || action == TEST_FORCE_COPY))
			invalid = true;
		//if dao
		if(typeCmb->currentItem() == VCD)
			invalid = true;
		//if audio and dao
		if(typeCmb->currentItem() == AUDIO && (action != TAO_COPY && action != TEST_TAO_COPY))
			invalid = true;
			
		if(invalid){
			dialogCntPnl->getStartBtn()->setEnabled(false);
			dialogCntPnl->status(i18n("Source device must be SCSI for this mode..."));
			return;
		}
	}
	
	dialogCntPnl->getStartBtn()->setEnabled(true);
	dialogCntPnl->status(i18n("Ready"));
}
//======================================================
void CdboCopyCDDialog::musicSlot(){
	resetConfig();
	settingTypes = "cdboburnopt cdboaudioopt cdbodefaultopt";
	audioTb->setEnabled(true);
	srcDevicePnl->setUsrBtnTitle(i18n("Scan &Tracks"));
}
//======================================================
void CdboCopyCDDialog::dataSlot(){
	resetConfig();
	//clearOutput();
	settingTypes = "cdboburnopt cdbodefaultopt";
	audioTb->setEnabled(false);
	srcDevicePnl->setUsrBtnTitle(i18n("&Preview"));
	trackView->invalidate();
}
//------------------------------------------------------------------------------------------
void CdboCopyCDDialog::previewSlot(){
	if(typeCmb->currentItem() == DATA || typeCmb->currentItem() == VCD){
		CdboPreviewDialog *pd = new CdboPreviewDialog((QWidget *)parent(), "pd");
		pd->open(srcDevicePnl->getDriveName());
	}
	else
		scanTracks();
}
//------------------------------------------------------------------------------------------
void CdboCopyCDDialog::scanTracks(){
	clearOutput();
	trackView->invalidate();

	if(!createAction("CdboScanAudioTracksAction")){
		return;
	}
	srcDevicePnl->closeTray();

	addActionParam("SOURCE_DEVICE",  srcDevicePnl->driveName());
	addActionParam("TRACKS_LIST", &pickedTracks);

	launchCurrAction();
}
//---------------------------------------------------------------------------------------------
void CdboCopyCDDialog::eraseSlot(){
	saveOptions();
	CdboEraseCDDialog *ed = new CdboEraseCDDialog((QWidget *)parent(),"eraseD");
	hide();
	ed->exec();
	delete ed;
	readOptions();
	initBtns();
	exec();
}
//---------------------------------------------------------------------------------------------
QString* CdboCopyCDDialog::actionType(){

	int action = actionCmb->currentItem();
	if(typeCmb->currentItem() == VCD && action > 1)
		action += 1;
		
	if(action  == TEST_COPY || action == TEST_ON_THE_FLY_COPY 
				|| action == TEST_FORCE_COPY || action == TEST_TAO_COPY)
		mActionType = "dummy";
	else
		mActionType = "burn";

	return &mActionType;
}
//---------------------------------------------------------------------------------------------
void CdboCopyCDDialog::startBtnSlot(){
	srcDevicePnl->closeTray();
	targetDevicePnl->closeTray();
	clearOutput();
	
	numberOfCopies = QString::number(numberSpn->value());
	sourceDeviceName = srcDevicePnl->getDriveName();
	
	if(!srcDevicePnl->getScsiDevice().isNull()){//try to use scsi whenever possible
		isScsi = "TRUE";
		sourceDevice = srcDevicePnl->getScsiDevice();
	}
	else{
		isScsi = "FALSE";
		sourceDevice = srcDevicePnl->getDriveName();
	}
	if(removeTmpChk->isChecked())
		isWithCleanUp = "TRUE";
	else
		isWithCleanUp = "FALSE";

	int action = actionCmb->currentItem();
	if(typeCmb->currentItem() == VCD && action > 1)
		action += 1;
	
	
	isNoError = "FALSE";
	if(action == FORCE_COPY || action == TEST_FORCE_COPY)
		isNoError = "TRUE";

	
	imageName = "";
	isOnTheFly = "FALSE";
	if(action == ON_THE_FLY_COPY || action == TEST_ON_THE_FLY_COPY){
		isOnTheFly = "TRUE";
		if(typeCmb->currentItem() == DATA)
			imageName = srcDevicePnl->getDriveName();

	}
	
	isDao = "FALSE";
	if(typeCmb->currentItem() == VCD)
		isDao = "TRUE";

	pickedTracks = "";
	isAudio = "FALSE";
	
	if(typeCmb->currentItem() == AUDIO){
		isAudio = "TRUE";
		
		QStringList pickedTracksList;
		bool gotTracks = trackView->getPickedTracks(&pickedTracksList);

		if(action == TAO_COPY || action == TEST_TAO_COPY){
			isMultyStep = "TRUE";
			isDao = "FALSE";
			if(gotTracks && pickedTracksList.count() == 0){
				dialogCntPnl->output(Cdbo::ErrorMessage, i18n("No tracks chosen. There is nothing to copy."));
				return;
			}
			if(gotTracks){
				sourceDevice = srcDevicePnl->getDriveName();
				pickedTracks = pickedTracksList.join("\n");
			}
			//if !gotTracks, copy full CD
		}
		else{
			isMultyStep = "FALSE";
			isDao = "TRUE";
			if(gotTracks && pickedTracksList.count() > 0){
				int result = KMessageBox::warningContinueCancel(this,
					i18n("In this mode your track selection is ignored. " \
						"Continue anyway?"), i18n("Ignore track selection?"),
					KStdGuiItem::cont(),"Ignore track selection?");
				switch(result) {
					case KMessageBox::Continue:
						break;
					case KMessageBox::Cancel:
						return;
				}
			}
		}
	}
	isSameDriveCopy = "FALSE";
	if(srcDevicePnl->getDriveName() == targetDevicePnl->getDriveName())
		isSameDriveCopy = "TRUE";

	if(!createAction("CdboCopyCdAction"))
		return;
	
	addActionParam("ACTION_TYPE", actionType());
	addActionParam("SPEED", speedPnl->speed());
	addActionParam("TARGET_DEVICE",  targetDevicePnl->scsiDevice());
	addActionParam("SOURCE_DEVICE",  &sourceDevice);
	addActionParam("SOURCE_DEVICE_NAME",  &sourceDeviceName);
	addActionParam("IS_SCSI_SOURCE", &isScsi);
	addActionParam("SAME_DRIVE_COPY", &isSameDriveCopy);
	
	addActionParam("TMP_DIRECTORY", &tempDir);
	addActionParam("IMAGE_SIZE", &minTmpSize);
	
	addActionParam("NUMBER_OF_COPIES", &numberOfCopies);
	addActionParam("IS_WITH_CLEAN_UP", &isWithCleanUp);
	
	
	addActionParam("IS_NO_ERROR_ENABLED", &isNoError);
	
	addActionParam("IS_ON_THE_FLY", &isOnTheFly);
	addActionParam("IMAGE_NAME", &imageName);//used by cdrecord action for on-the-fly
		
	addActionParam("IS_DAO", &isDao);
	
	addActionParam("IS_AUDIO", &isAudio);
	addActionParam("IS_MULTY_STEP_AUDIO", &isMultyStep);
	addActionParam("PICKED_TRACKS", &pickedTracks);
	
	//debugCommands();
	//debugOutput();
	
	launchCurrAction();
}
//---------------------------------------------------------------------------------------------
void CdboCopyCDDialog::taskDone(bool success){
	if(success){
		if(isCurrAction("CdboScanAudioTracksAction")){
			detailsTab->showAudioTab();
			trackView->load(pickedTracks, false);
		}
	}
	CdboDialogBase::taskDone(success);
}



