/***************************************************************************
                          CdboDataCDDialog.cpp  -  description
                             -------------------
    begin                : Sun May 13 2001
    copyright            : (C) 2001 by Alexy Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qcombobox.h>
#include <qtooltip.h>
#include <qspinbox.h>
#include <qdir.h>
#include <qcheckbox.h>
#include <qwhatsthis.h>

#include <kiconloader.h>
#include <kfiledialog.h>
#include <klocale.h>
#include <kconfig.h>
#include <kurl.h>
#include <kdebug.h>
#include <ksqueezedtextlabel.h>
#include <dcopclient.h>


#include "CdboDataCDDialog.h"
#include "CdboDataCDDialog.moc"
#include "CdboEraseCDDialog.h"
#include "CdboPreviewDialog.h"

#define BURN 0
#define CREATE 1
#define CREATE_ONLY 2

CdboDataCDDialog::CdboDataCDDialog(QWidget *parent, const char *name) :
	CdboDataCDBase(parent,name,true),
	DCOPObject( "CdboDialogBase" ){

	init();
}
//---------------------------------------------------------------------------------------------
CdboDataCDDialog::~CdboDataCDDialog(){
}
//---------------------------------------------------------------------------------------------
void CdboDataCDDialog::init(){
	createImageOnly = false;
	createImage = false;
	topLevelList = false;
	pathspecReady = false;
	hidejoiName = "";
	hidehfsName = "";
	hiderrName = "";
	
	CdboDialogBase::init();

	imageBrowseBtn->setPixmap( SmallIcon("fileopen") );

	///////////////////////////////////////////////////
	targetDevicePnl->setType(CdboDriveInfoPnl::Target);
	targetDevicePnl->setUsrBtnTitle( i18n("&Erase"));
	/////////////////////////////////////////////////
	dialogCntPnl->setStartBtnTitle( i18n("C&reate"));
	QToolTip::add(dialogCntPnl->getStartBtn(), i18n( "Create" ) );
	/////////////////////////////////////////////////

	readOptions();
	targetDevicePnl->closeTray();

	connect( targetDevicePnl->getDetectBtn(), SIGNAL( clicked() ), this, SLOT( detectSlot() ) );
	connect( targetDevicePnl->getUsrBtn(), SIGNAL( clicked() ), this, SLOT( eraseSlot() ) );

	connect( dialogCntPnl->getStartBtn(), SIGNAL( clicked() ), this, SLOT( startBtnSlot() ) );
	connect( dialogCntPnl->getCancelBtn(), SIGNAL( clicked() ), this, SLOT( cancelSlot() ) );
	connect( dialogCntPnl->getSettingsBtn(), SIGNAL( clicked() ), this, SLOT( settingsSlot() ) );
	connect( dialogCntPnl->getQuitBtn(), SIGNAL( clicked() ), this, SLOT( quitSlot() ) );

	connect( imageBrowseBtn, SIGNAL( clicked() ), this, SLOT( isoBrowseSlot() ) );
	connect( isoPreviewBtn, SIGNAL( clicked() ), this, SLOT( isoPreviewSlot() ) );
	
	connect( actionCmb, SIGNAL( activated(int) ), this, SLOT( verifyMode(int) ) );
	
	settingTypes = "cdboisoopt cdboburnopt cdbodefaultopt";

}
//---------------------------------------------------------------------------------------------
void CdboDataCDDialog::saveOptions(){
	if(!config)
		return;

	config->setGroup("Create Options");
	config->writeEntry("Type of Create " + QString::number(prevType), actionCmb->currentItem());
	config->writeEntry("Number of Copies", numberSpn->value());
	config->writeEntry("Remove Image", removeIsoChk->isChecked());

	speedPnl->saveOptions();
	targetDevicePnl->saveOptions();

	config->sync();
}
//---------------------------------------------------------------------------------------------
void CdboDataCDDialog::readOptions(){
	saveOptions();
	CdboDialogBase::readOptions();

	config->setGroup("Create Options");
	numberSpn->setValue(config->readNumEntry("Number of Copies", 0));
	removeIsoChk->setChecked(config->readBoolEntry("Remove Image", false));

	config->setGroup("Default Settings");
	tempDir = config->readEntry("Tmp Dir", "/tmp/");

	//QString s = imageLbl->text();
	//if((s.isNull() || s == "") && createImage){
	if(createImage){
		config->setGroup("ISO Settings");
		imageLbl->setText(tempDir + config->readEntry("Custom ISO Name", "cdboImage_<DATE>.iso"));
		imageLbl->setTooltip(i18n("Use Settings Dialog to change ISO image name and working directory location.<b></b>"));
	}
	speedPnl->readOptions();
	targetDevicePnl->readOptions();
	initBtns();
}
//---------------------------------------------------------------------------------------------
void CdboDataCDDialog::initBtns(){
	if(!createImage){
		if(targetDevicePnl->isReady()){
			restructureActions(BURN);

			blockBtns(false);
			//isoOptionsBtn->setEnabled(false);

			QString s = imageLbl->text();
			if(!s.isNull() && s != ""){
				dialogCntPnl->getStartBtn()->setEnabled(true);
				isoPreviewBtn->setEnabled(true);
			}
			else{
				dialogCntPnl->getStartBtn()->setEnabled(false);
				isoPreviewBtn->setEnabled(false);
			}

			blockActionBtns(false);
			imageBrowseBtn->setEnabled(true);
			dialogCntPnl->status(i18n("Ready"));
		}
		else{
			blockBtns(true);
			restructureActions(CREATE_ONLY);
			dialogCntPnl->getCancelBtn()->setEnabled(false);
			dialogCntPnl->status(i18n("No recording device." ));
		}
	}
	else{
		blockBtns(false);

		if(targetDevicePnl->isReady()){
			blockActionBtns(false);
			dialogCntPnl->status(i18n("Ready"));
			if(createImageOnly)
				restructureActions(CREATE_ONLY);
			else
				restructureActions(CREATE);
		}
		else{
			restructureActions(CREATE_ONLY);
			targetDevicePnl->blockBtns(true);
			speedPnl->setEnabled(false);

			blockActionBtns(true);

			dialogCntPnl->getCancelBtn()->setEnabled(false);
			dialogCntPnl->status(i18n("No recording device."));
		}
		isoPreviewBtn->setEnabled(false);
		imageBrowseBtn->setEnabled(false);
		dialogCntPnl->getStartBtn()->setEnabled(true);
	}
}
//---------------------------------------------------------------------------------------------
void CdboDataCDDialog::blockActionBtns(bool yes){
	numberSpn->setEnabled(!yes);
	numOfCopiesLbl->setEnabled(!yes);
	removeIsoChk->setEnabled(!yes);
	imageBrowseBtn->setEnabled(!yes);
}
//----------------------------------------------------------------------------------------------
void CdboDataCDDialog::blockBtns(bool yes){
	dialogCntPnl->getStartBtn()->setEnabled(!yes);
	dialogCntPnl->getCancelBtn()->setEnabled(yes);
	speedPnl->setEnabled(!yes);
	detailsPnl->setEnabled(!yes);
	targetDevicePnl->blockBtns(yes);
	if(doingSomething()){
		targetDevicePnl->getDetectBtn()->setEnabled(false);
		dialogCntPnl->getSettingsBtn()->setEnabled(false);
	}
	else{
		targetDevicePnl->getDetectBtn()->setEnabled(true);
		dialogCntPnl->getSettingsBtn()->setEnabled(true);
	}
}
//----------------------------------------------------------------------------------------------
void CdboDataCDDialog::restructureActions(int type){
	prevType = type;
	actionCmb->clear();
	QWhatsThis::remove(actionCmb);
	QString whatsThis = i18n("Choose how and what type of CD should be created.");
	switch(type){
		case BURN:{
			actionCmb->insertItem( i18n( "Create CD" ) );
			whatsThis += i18n("<br><br><b>Create CD:</b> Will create Single- or Muti-Session CD from this ISO image. "\
				"In Multi-Session Mode this will become the first session.");
			
			actionCmb->insertItem( i18n( "Append to CD" ) );
			whatsThis += i18n("<br><br><b>Append to CD:</b> Will append this ISO image to an existing Multi-Session CD.");
			
			actionCmb->insertItem( i18n( "Test Create CD" ) );
			actionCmb->insertItem( i18n( "Test Append to CD" ) );
			break;
		}
		case CREATE:{
			actionCmb->insertItem( i18n( "Create CD in Safe Mode" ) );
			whatsThis += i18n("<br><br><b>Create CD in Safe Mode:</b> Will create Single- or Muti-Session CD from selected files. "\
				"Working directory must have sufficient space available to store image file. If not sure, use this mode.");
				
			actionCmb->insertItem( i18n( "Create CD in Fast Mode" ) );
			whatsThis += i18n("<br><br><b>Create CD in Fast Mode:</b> Will create Single- or Muti-Session CD from selected files "\
				"on the fly without creating an image file. Success of such operation depends on your machine capabilities and system load. "\
				"Higher system load espesially on weaker machines increases possibility of failure due to buffer underruns. "\
				"This is however the fastest way to create CD.");
			
			actionCmb->insertItem( i18n( "Test Safe Mode" ) );
			actionCmb->insertItem( i18n( "Test Fast Mode" ) );
			
			actionCmb->insertItem( i18n( "Create Image Only" ) );
			whatsThis += i18n("<br><br><b>Create Image Only:</b> Will create ISO Image and stop.");
			break;
		}
		case CREATE_ONLY:{
			actionCmb->insertItem( i18n( "Create Image Only" ) );
			whatsThis += i18n("<br><br><b>Create Image Only:</b> Will create ISO Image and stop.");
			QWhatsThis::add(actionCmb, whatsThis);
			return;
		}
	}
	config->setGroup("Create Options");
	int index = config->readNumEntry("Type of Create " + QString::number(prevType), 0);
	whatsThis += i18n("<br><br><b>Test ... :</b> Same as coresponding real Mode (one that does not start with test), "\
		"but laser will stay cool - nothing will be actually burned. Allows testing without possibility of corrupting recordable media.");
	QWhatsThis::add(actionCmb, whatsThis);
	actionCmb->setCurrentItem(index);
}
//---------------------------------------------------------------------------------------------
void CdboDataCDDialog::verifyMode(int newMode){
	if(!targetDevicePnl->isReady() || doingSomething())
		return;
	if(createImageOnly){
		removeIsoChk->setEnabled(false);
		return;
	}
		
	if(createImage && (newMode == 1 || newMode == 3 || newMode == 4)){
		removeIsoChk->setEnabled(false);
		return;
	}
		
	removeIsoChk->setEnabled(true);
}
//---------------------------------------------------------------------------------------------
void CdboDataCDDialog::setSourceDirs(QString sourceDirs, QString delim){
	clearOutput();
	QStringList list = QStringList::split(delim , sourceDirs, false);
	createImage = true;
	fileList = list.join("\n");
	readOptions();
}
//---------------------------------------------------------------------------------------------
void CdboDataCDDialog::setISOImage(QString image){
	clearOutput();
	createImage = false;
	topLevelList = false;
	pathspecReady = false;
	imageLbl->setText(image);
	imageLbl->setTooltip(image);
	initBtns();
	if(dialogCntPnl->getStartBtn()->isEnabled()){
		dialogCntPnl->output(Cdbo::InfoMessage,
 			i18n("Ready to create CD from %1").arg(image));
	}
}
//---------------------------------------------------------------------------------------------
void CdboDataCDDialog::setURL(QString url){
	clearOutput();
	QString pointerName;
	KURL fromUrl(url);
	if(fromUrl.protocol() == "newcd"){
		QStringList path = QStringList::split('/', fromUrl.path(), false);
		pointerName = path[0];
		url = fromUrl.prettyURL();
		url = url.section('/', 0, 1);
	}
	else if(url.endsWith("desktop")&& QFile::exists(url)){
		KConfig *deskFile = new KConfig(url);
		deskFile->setGroup("Desktop Entry");
		url = deskFile->readEntry("URL", "");
		delete deskFile;
		if(url.isEmpty())
			return;
		pointerName = url.section('/', 1, 1);
	}
	else{
		dialogCntPnl->output(Cdbo::ErrorMessage,
			i18n("Malformed URL: %1").arg(url));
		return;
	}

	DCOPClient *dcopClient = new DCOPClient();
	if(!dcopClient->attach()){
		dialogCntPnl->output(Cdbo::ErrorMessage,
			i18n("Failed to communicate with cdrwatcher, could not parse URL."));
		return;
	}

	QByteArray data;
	QDataStream arg(data, IO_WriteOnly);
	arg << pointerName;
	QByteArray replyData;
	QCString replyType;
	if (!dcopClient->call( "kded", "cdrwatcher", "getSourceForPointer(QString )", data, replyType, replyData, false) ){
		dialogCntPnl->output(Cdbo::ErrorMessage,
			i18n("Failed to communicate with cdrwatcher, could not parse URL."));
		return;
	}

	QDataStream replyStream(replyData, IO_ReadOnly);
	replyStream >> fileList;

	if(fileList.isEmpty())
		return;

	createImage = true;
	topLevelList = true;
	pathspecReady = false;
	readOptions();

	if(dialogCntPnl->getStartBtn()->isEnabled()){
		QString what = i18n("new CD");
		if(createImageOnly)
			what = i18n("ISO Image");
		dialogCntPnl->output(Cdbo::InfoMessage,
 			i18n("Ready to create %1  from %2").arg(what).arg(url));
	}
}
//----------------------------------------------------------------------------------------------
void CdboDataCDDialog::setImageOnly(bool imageOnly){
	createImageOnly = imageOnly;
	initBtns();
}
//----------------------------------------------------------------------------------------------
void CdboDataCDDialog::setPathspecName(const QString &name){
	pathspecName = name;
	createImage = true;
	topLevelList = false;
	pathspecReady = true;
	readOptions();
}
//----------------------------------------------------------------------------------------------
void CdboDataCDDialog::setHidehfsName(const QString &name){
	hidehfsName = name;
}
//----------------------------------------------------------------------------------------------
void CdboDataCDDialog::setHiderrName(const QString &name){
	hiderrName = name;
}
//----------------------------------------------------------------------------------------------
void CdboDataCDDialog::setHidejoiName(const QString &name){
	hidejoiName = name;
}
//----------------------------------------------------------------------------------------------
void CdboDataCDDialog::isoBrowseSlot(){

	QString buff = imageLbl->text();
	if(!buff.isNull() && buff != "")
		buff = buff.left(buff.findRev("/", -2));
	else{
		config->setGroup("Default Settings");
		buff = config->readEntry("Tmp Dir", QDir::homeDirPath());
	}
	QString fileToOpen = KFileDialog::getOpenFileName( buff,
				i18n("*.iso *.ISO *.raw *.RAW|ISO image files (*.iso, *.raw)\n*|All files"),
				this,
				i18n("Open ISO Image"));
	//if(!fileToOpen.isEmpty()){
	if(QFile::exists(fileToOpen))
		setISOImage(fileToOpen);
}
//------------------------------------------------------------------------------------------
void CdboDataCDDialog::isoPreviewSlot(){
	CdboPreviewDialog *pd = new CdboPreviewDialog(0, "pd");
	pd->open(imageLbl->text(), true);
	//delete pd;
}
//---------------------------------------------------------------------------------------------
void CdboDataCDDialog::eraseSlot(){

	//saveOptions();
	hide();
	CdboEraseCDDialog *ed = new CdboEraseCDDialog((QWidget *)parent(),"eraseD");
	ed->exec();
	delete ed;
	readOptions();
	exec();
}
//---------------------------------------------------------------------------------------------
QString* CdboDataCDDialog::actionType(){
	int pos = actionCmb->currentItem();
	switch(pos){
		case 0:{
			mActionType =   "burn";
			break;
		}
		case 1:{
			if(!createImage)
				mActionType = "append";
			else
				mActionType =   "burn";
			break;
		}
		case 2: {
			mActionType = "dummy";
			break;
		}
		case 3: {
			if(!createImage)
				mActionType = "dummy append";
			else
				mActionType = "dummy";
			break;
		}
		default:
			mActionType = "burn";
	}
	return &mActionType;
}
//---------------------------------------------------------------------------------------------
void CdboDataCDDialog::startBtnSlot(){
	clearOutput();
	config->setGroup("Default Settings");
	if(createImage && !topLevelList && pathspecReady){
		imageSize = config->readEntry("Image Size", "0");
		mappingsReady = "TRUE";
	}
	else{
		imageSize = config->readEntry("Minimum Tmp Size", "650");
		mappingsReady = "FALSE";
	}

	if(createImage && (actionCmb->currentItem() == 1 || actionCmb->currentItem() == 3))
		isOnTheFly = "TRUE";
	else
		isOnTheFly = "FALSE";

	if(topLevelList && !pathspecReady)
		isTopLevelList = "TRUE";
	else
		isTopLevelList = "FALSE";

	numberOfCopies = QString::number(numberSpn->value());

	if(removeIsoChk->isEnabled() && removeIsoChk->isChecked())
		isWithCleanUp = "TRUE";
	else
		isWithCleanUp = "FALSE";

	if(createImage){
		isCreateImage = "TRUE";
		if(createImageOnly || actionCmb->currentItem() == 4)
			isCreateImageOnly = "TRUE";
		else
			isCreateImageOnly = "FALSE";
		isoImageName = "";
	}
	else{
		isoImageName = imageLbl->text();
		isCreateImage = "FALSE";
		isCreateImageOnly = "FALSE";
	}

	if(!createAction("CdboCreateDataCdAction"))
		return;

	targetDevicePnl->closeTray();

	addActionParam("ACTION_TYPE", actionType());
	addActionParam("SPEED", speedPnl->speed());
	addActionParam("TARGET_DEVICE",  targetDevicePnl->scsiDevice());

	addActionParam("IMAGE_NAME", &isoImageName);

	addActionParam("IS_MAPPINGS_READY", &mappingsReady);
	
	if(!pathspecReady)
		addActionParam("FILE_LIST", &fileList);
	else{
		addActionParam("PATH_SPEC_NAME", &pathspecName);
		if(!hidejoiName.isEmpty())
			addActionParam("HIDE_JOI_NAME", &hidejoiName);
		if(!hidehfsName.isEmpty())
			addActionParam("HIDE_HFS_NAME", &hidehfsName);
		if(!hiderrName.isEmpty())
			addActionParam("HIDE_RR_NAME", &hiderrName);
	}
	
	addActionParam("TMP_DIRECTORY", &tempDir);
	addActionParam("IMAGE_SIZE", &imageSize);
			
	addActionParam("IS_WITH_CLEAN_UP", &isWithCleanUp);

	addActionParam("IS_ON_THE_FLY_CREATE_DATA", &isOnTheFly);
	addActionParam("IS_TOP_LEVEL", &isTopLevelList);


	addActionParam("IS_CREATE_IMAGE", &isCreateImage);
	addActionParam("IS_CREATE_IMAGE_ONLY", &isCreateImageOnly);

	addActionParam("NUMBER_OF_COPIES", &numberOfCopies);
	//debugCommands();
	launchCurrAction();
}
//---------------------------------------------------------------------------------------------
void CdboDataCDDialog::taskDone(bool success){
	if(success){
		if(isCurrAction("CdboCreateDataCdAction") && createImage && isOnTheFly != "TRUE" && !removeIsoChk->isChecked()){
			setISOImage(isoImageName);
		}
	}
	CdboDialogBase::taskDone(success);
}

