/***************************************************************************
                          CdboEraseCDBase.cpp  -  description
                             -------------------
    begin                : Sat May 5 2001
    copyright            : (C) 2001 by Alexy Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/
/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include "CdboEraseCDBase.h"
#include "CdboEraseCDBase.moc"

#include <qcheckbox.h>
#include <qcombobox.h>
#include <qframe.h>
#include <qgroupbox.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>


#include "CdboDialogCntPnl.h"
#include "CdboDriveInfoPnl.h"
#include "CdboSpeedPnl.h"

#include <klocale.h>

CdboEraseCDBase::CdboEraseCDBase( QWidget* parent,  const char* name, bool modal, WFlags fl)
	: CdboDialogBase( parent, name, modal, fl ){

	if ( !name )
		setName( "CdboEraseCDBase" );
	eraseMode = true;
	init();
}
//===========================================================
CdboEraseCDBase::CdboEraseCDBase(bool eraseMode, QWidget* parent,  const char* name, bool modal, WFlags fl)
	: CdboDialogBase( parent, name, modal, fl ){

	if ( !name )
		setName( "CdboEraseCDBase" );
	this->eraseMode = eraseMode;
	init();
}
//===========================================================	
void CdboEraseCDBase::init(){

	if(eraseMode)
		setCaption( i18n( "Erase CD-RW - CD Bake Oven" ) );
	else
		setCaption( i18n( "Fixate CD - CD Bake Oven" ) );
	cdboEraseBaseLayout = new QGridLayout( this );
	cdboEraseBaseLayout->setSpacing( 0 );
	cdboEraseBaseLayout->setMargin( 0 );

	dialogCntPnl = new CdboDialogCntPnl( this, "dialogCntPnl" );

	cdboEraseBaseLayout->addMultiCellWidget( dialogCntPnl, 1, 1, 0, 1 );

	baseFrm = new QFrame( this, "baseFrm" );
	baseFrmLayout = new QVBoxLayout( baseFrm );
	baseFrmLayout->setSpacing( 6 );
	baseFrmLayout->setMargin( 11 );

	targetDevicePnl = new CdboDriveInfoPnl( baseFrm, "targetDevicePnl" );
	baseFrmLayout->addWidget( targetDevicePnl );

	cdboEraseBaseLayout->addWidget( baseFrm, 0, 0 );


	detailsFrm = new QFrame( this, "detailsFrm" );

	detailsPnlLayout = new QVBoxLayout( detailsFrm );
	detailsPnlLayout->setSpacing( 6 );
	detailsPnlLayout->setMargin( 11 );

	optPnl = new QGroupBox( detailsFrm, "optPnl" );
	optPnl->setEnabled( TRUE );
	optPnl->setFrameShape( QGroupBox::Box );
	optPnl->setFrameShadow( QGroupBox::Sunken );
	if(eraseMode){
		optPnl->setTitle( i18n( "Blank-Type" ) );
		QWhatsThis::add(  optPnl, i18n( "Type of erase." ) );
	}
	else{
		optPnl->setTitle( i18n( "Fixate-Type" ) );
		QWhatsThis::add(  optPnl, i18n( "Type of fixate." ) );
	}
	optPnl->setColumnLayout(0, Qt::Vertical );
	optPnl->layout()->setSpacing( 0 );
	optPnl->layout()->setMargin( 0 );
	optPnlLayout = new QHBoxLayout( optPnl->layout() );
	optPnlLayout->setAlignment( Qt::AlignTop );
	optPnlLayout->setSpacing( 6 );
	optPnlLayout->setMargin( 11 );

	blankTypeCmb = new QComboBox( FALSE, optPnl, "blankTypeCmb" );


	if(eraseMode){
		blankTypeCmb->insertItem( i18n( "Minimal Blank" ) );
		blankTypeCmb->insertItem( i18n( "Blank Entire CD-RW" ) );
		blankTypeCmb->insertItem( i18n( "Blank a Track" ) );
		blankTypeCmb->insertItem( i18n( "Unreserve a Track" ) );
		blankTypeCmb->insertItem( i18n( "Blank Tail of a Track" ) );
		blankTypeCmb->insertItem( i18n( "Unclose last Session" ) );
		blankTypeCmb->insertItem( i18n( "Blank last Session" ) );
		QToolTip::add(  blankTypeCmb, i18n( "<b><i>Please note:</i></b> Full blank at single speed takes about 74 min. Minimal blank is done in a few minutes." ) );
		QWhatsThis::add(  blankTypeCmb, i18n( "Please note: Full blank at single speed takes about 74 min. Minimal blank is done in a few minutes." ) );
	}
	else{
		blankTypeCmb->insertItem( i18n( "Fixate CD" ) );
		blankTypeCmb->insertItem( i18n( "Test Fixate CD" ) );
		QToolTip::add(  blankTypeCmb, i18n( "Please choose the action to take." ) );
	}


	optPnlLayout->addWidget( blankTypeCmb );

	QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
	optPnlLayout->addItem( spacer );

	
	forceChk = new QCheckBox( optPnl, "forceChk" );
	
	if(eraseMode){
		forceChk->setText( i18n( "&Force blanking" ) );
		QToolTip::add(  forceChk, i18n( "Force to continue on errors. <b></b>This action currently implements some tricks that will allow you to blank bad CD-RW disks." ) );
		QWhatsThis::add(  forceChk, i18n( "This action currently implements some tricks that will allow you to blank bad CD-RW disks." ) );
	}
	else{
		forceChk->setText( i18n( "&Eject at End" ) );
		QToolTip::add(  forceChk, i18n( "Eject CD once it is finalized." ) );
	}
	optPnlLayout->addWidget( forceChk );
	
	detailsPnlLayout->addWidget( optPnl );

	speedPnl = new CdboSpeedPnl( detailsFrm, "speedPnl" );
	detailsPnlLayout->addWidget( speedPnl );

	cdboEraseBaseLayout->addWidget( detailsFrm, 0, 1 );
	cdboEraseBaseLayout->setColStretch(0,3);
	cdboEraseBaseLayout->setColStretch(1,5);
	// tab order	
	setTabOrder( forceChk, blankTypeCmb );
}
CdboEraseCDBase::~CdboEraseCDBase(){
}

