/***************************************************************************
                          CdboPreviewDialog.cpp  -  description
                             -------------------
    begin                : Fri Jun 1 2001
    copyright            : (C) 2001 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qlayout.h>
#include <qlabel.h>

#include <kio/netaccess.h>
#include <klocale.h>
#include <kconfig.h>
#include <kurl.h>
#include <kmessagebox.h>
#include <kstandarddirs.h>
#include <kcursor.h>
#include <kdiroperator.h>
#include <kfileview.h>
#include <kdebug.h>
#include <ktoolbar.h>
#include <kstdaction.h>
#include <kfilepreview.h>


#include "CdboPreviewDialog.h"
#include "CdboPreviewDialog.moc"

#define URL_LBL 101


CdboPreviewDialog::CdboPreviewDialog( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : KDialog( parent, name, modal, fl ){
	if ( !name )
       	setName( "CdboPreviewDialog" );
	resize( 600, 350 );


	config = new KConfig("cdbakeovenrc");
	config->setGroup("Default Settings");

	toolbar = new KToolBar( this, "CdboPreviewDialog::toolbar", true);
	toolbar->setFlat(true);
	
	layout = new QVBoxLayout( this );
	layout->setSpacing( 6 );
	layout->setMargin( 11 );

	layout->addWidget(toolbar, AlignTop);

	dirOp = new KDirOperator( KURL(), this );
	
	
	
	dirOp->setView(KFile::Detail);
	//dirOp->setView(new KFilePreview(dirOp, "dirOpView"));
	
	
	dirOp->readConfig( config, "Preview dirOp" );
	
	connect( dirOp, SIGNAL(finishedLoading ()), this, SLOT(dirOpFinishedLoading() ) );

	

	actionCollection = new KActionCollection(this, "actionCollection");
	quit = KStdAction::quit(this, SLOT(quitSlot()), actionCollection);

	dirOp->actionCollection()->action( "up" )->plug( toolbar );
	dirOp->actionCollection()->action( "back" )->plug( toolbar );
	dirOp->actionCollection()->action( "forward" )->plug( toolbar );
	dirOp->actionCollection()->action( "reload" )->plug( toolbar );
	dirOp->actionCollection()->action( "separator" )->plug( toolbar );
	
	quit->plug( toolbar );
	dirOp->actionCollection()->action( "separator" )->plug( toolbar );

	urlLbl = new QLabel(i18n("Loading..."), toolbar, "urlLbl");
	urlLbl->setFrameShape( QLabel::Box );
	urlLbl->setFrameShadow( QLabel::Raised );
	toolbar->insertWidget(URL_LBL, 0, urlLbl);

	
	
	toolbar->setItemAutoSized (URL_LBL);
	toolbar->setIconText(KToolBar::IconOnly);
	toolbar->setBarPos(KToolBar::Top);
	toolbar->setMovingEnabled(false);
	toolbar->adjustSize();

	
	layout->addWidget( dirOp );
	layout->setStretchFactor(dirOp, 10);
	dirOp->setupMenu(KDirOperator::SortActions|KDirOperator::ViewActions);

}
//===================================================
CdboPreviewDialog::~CdboPreviewDialog(){
	delete config;
}
//===================================================
bool CdboPreviewDialog::open(QString location, bool image){

	setCaption(i18n("%1 - CD Bake Oven").arg(location));
	device = location;

	urlTitle = location;
	if(image){
		urlTitle = location.section("/",-1);
		QString buff = locateLocal("data", "cdbakeoven");
		device = buff + "/sessionLoad";
	}

	config->setGroup("Wizard Detected");
	mountPoint = config->readEntry("Mount point for " + device, QString::null);
	if(mountPoint.isNull()){
		KMessageBox::sorry(this,
				i18n("Mount point for this action is not configured yet. "\
					"Please use CD Bake Oven Configuration Wizard to detect mount point for this device."));
		needToUmount = false;
		quitSlot();
		return false;
	}
	if(mountPoint.endsWith("*supermount")){
		needToUmount = false;
		removeMountPoint = false;
		mountPoint.truncate(mountPoint.length() - 11);
		dirOp->setURL(KURL(mountPoint), true);
		dirOp->clearHistory();
		exec();
		return true;
	}
	if(image){
		KIO::SimpleJob * job = KIO::symlink( location, KURL(device), true);
		job->setName(device.utf8());
		connect(job, SIGNAL(result ( KIO::Job *)),
			this, SLOT(slotSymlinkResult(KIO::Job *) ));
	}
	else
		doOpen(device, false);

	return true;
}
///=========================================================
void CdboPreviewDialog::slotSymlinkResult( KIO::Job * j){
	if(j->error()){
		KMessageBox::error(this,
				i18n("Failed to prepare for mount."));
		needToUmount = false;
		removeMountPoint = false;
		quitSlot();
		return;
	}

	doOpen(device, true);
}
//===================================================
bool CdboPreviewDialog::doOpen(QString device, bool image){

	//check if already mounted - does not work :(
	/*QString usedMountPoint = KIO::findDeviceMountPoint(device);
	if(!mountPoint.isNull()){
	cerr << "f(!mountPoint.isNull())\n";
		needToUmount = false;
		removeMountPoint = false;
		top = new FolderItem(filesView, usedMountPoint, false, showHidden);
		top->setText(0, device);
		exec();
		return true;
	}*/
	if(image)
		removeMountPoint = true;
	else
		removeMountPoint = false;

	doMount(device);
	return true;
}
///=========================================================
void CdboPreviewDialog::doMount(QString device){

	KIO::SimpleJob * job = KIO::mount(true, 0, device, mountPoint, false);
	needToUmount = true;
	job->setName(mountPoint.utf8());
	connect(job, SIGNAL(result ( KIO::Job *)),
		this, SLOT(slotMountResult(KIO::Job *) ));

}
///=========================================================
void CdboPreviewDialog::slotMountResult( KIO::Job * j){
	if(j->error()){
		if(device.endsWith("sessionLoad")){
			KMessageBox::error(this,
				i18n("Failed to mount. "\
				"The image may be corrupt, or you do not have "\
				"enough privileges to use the 'mount' command."));
		}
		else{
			KMessageBox::error(this,
				i18n("Failed to mount %1. Disk in this device may be empty or corrupt, or you do not have "\
				"enough privileges to use the 'mount' command.").arg(device) );
		}
		needToUmount = false;
		quitSlot();
		return;
	}

	needToUmount = true;
	dirOp->setURL(KURL(mountPoint), true);
	dirOp->clearHistory();
  
	exec();
}
//============================================================
void CdboPreviewDialog::quitSlot(){
	dirOp->writeConfig( config, "Preview dirOp" );
	if(!needToUmount) {
		if(removeMountPoint)
			KIO::NetAccess::del(device);
		accept();
		return;
	}

	KIO::SimpleJob * job = KIO::unmount(mountPoint, false);
	connect(job, SIGNAL(result ( KIO::Job *)),
		this, SLOT(slotUMountResult(KIO::Job *) ));

}
//---------------------------------------------------------------------------------------------
void CdboPreviewDialog::slotUMountResult(KIO::Job *){
	if(removeMountPoint)
		KIO::NetAccess::del(device);
	accept();
}
//---------------------------------------------------------------------------------------------
void CdboPreviewDialog::keyPressEvent(QKeyEvent *e){
	if(e->key() == Key_Escape){
		quitSlot();
	}
}
//---------------------------------------------------------------------------------------------
void CdboPreviewDialog::closeEvent(QCloseEvent *e){
	e->ignore();
	quitSlot();
}
//---------------------------------------------------------------------------------------------
void CdboPreviewDialog::dirOpFinishedLoading(){
	QString curUrl = urlTitle + ":" + dirOp->url().path().remove(0, mountPoint.length());
	urlLbl->setText(curUrl);

	
	(mountPoint + "/" == dirOp->url().path())?
		dirOp->actionCollection()->action( "up" )->setEnabled( false ):
		dirOp->actionCollection()->action( "up" )->setEnabled( true );
}

