/***************************************************************************
                          CdboBurnOpt.cpp  -  description
                             -------------------
    begin                : Sat May 5 2001
    copyright            : (C) 2001 by Alexy Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qspinbox.h>
#include <qlabel.h>
#include <qcheckbox.h>
#include <qbuttongroup.h>
#include <qlineedit.h>

#include <kaboutdata.h>
#include <kcmodule.h>
#include <klocale.h>
#include <kglobal.h>
#include <dcopclient.h>
#include <kapplication.h>

#include "CdboBurnOpt.h"
#include "CdboBurnOpt.moc"

CdboBurnOpt::CdboBurnOpt( QWidget* parent,  const char* name, WFlags )
	:CdboBurnOptBase( parent, name){

	config = new KConfig("cdbakeovenrc");
	load();
	connect( useCustOptChk, SIGNAL( clicked() ), this, SLOT( useCustOptSlot() ) );
	connect( useCdrdaoCustOptChk, SIGNAL( clicked() ), this, SLOT( useCustCdrdaoOptSlot() ) );

}

CdboBurnOpt::~CdboBurnOpt(){
}
void CdboBurnOpt::load(){
	////////////////////////////////////////////////////////////
	config->setGroup("Recording Options");
	ejectChk->setChecked(config->readBoolEntry("Eject Media At End", true));
	checkDriveChk->setChecked(config->readBoolEntry("Check Drive First", false));
	overburnChk->setChecked(config->readBoolEntry("Overburn CD", false));
	mcnTxt->setText(config->readEntry("Media Catalog Number", ""));

	///////// Custom cdrecord Opt //////////////////////////////////
	useCustOptChk->setChecked(config->readBoolEntry("Use Custom Options", false));
	useCustOptSlot();

	burnProofChk->setChecked(config->readBoolEntry("Burn Proof", false));
	daoChk->setChecked(config->readBoolEntry("Disk At Once", false));
	fifoSizeSpn->setValue(config->readNumEntry("FIFO Size", 4));
	useInfChk->setChecked(config->readBoolEntry("Use Inf Files", false));
	ignoreInfSlot();
     defPregapSpn->setValue( config->readNumEntry("Default Pregap", 0) );
     scsiTimeOutSpn->setValue( config->readNumEntry("SCSI Time Out", 45) );
	driverNameTxt->setText(config->readEntry("Driver Name", ""));
	driverOptTxt->setText(config->readEntry("Driver Options", ""));

	///////// Custom cdrdao Opt //////////////////////////////////
	useCdrdaoCustOptChk->setChecked(config->readBoolEntry("Use Custom cdrdao Options", false));
	useCustCdrdaoOptSlot();
	forceCdrdaoChk->setChecked(config->readBoolEntry("Force cdrdao", false));
	reloadChk->setChecked(config->readBoolEntry("Reload cdrdao", false));
	numBuffersSpn->setValue( config->readNumEntry("Number of Buffers for cdrdao", 32) );


	///////////////Erase opt///////////////////////
     config->setGroup("Erase Options");
	dummyEraseChk->setChecked(config->readBoolEntry("Dummy Erase", false));
	ejectEraseChk->setChecked(config->readBoolEntry("Eject After Erase", true));
	checkDriveEraseChk->setChecked(config->readBoolEntry("Check Drive First", false));

}
//-----------------------------------------------------------------------------------------------
void CdboBurnOpt::save(){
	////////////////////////////////////////////////////////////
	config->setGroup("Recording Options");
     config->writeEntry("Eject Media At End", ejectChk->isChecked());
	config->writeEntry("Check Drive First", checkDriveChk->isChecked());
	config->writeEntry("Overburn CD", overburnChk->isChecked());
	
	QString buff = mcnTxt->text().remove(";").remove("`");
	mcnTxt->setText(buff);
	config->writeEntry("Media Catalog Number", buff);


	///////// Custom cdrecord Opt //////////////////////////////////
	config->writeEntry("Use Custom Options", useCustOptChk->isChecked());
	config->writeEntry("Disk At Once", daoChk->isChecked());
	config->writeEntry("Burn Proof", burnProofChk->isChecked());
	config->writeEntry("FIFO Size", fifoSizeSpn->value());
	config->writeEntry("Use Inf Files", useInfChk->isChecked());
	config->writeEntry("Default Pregap", defPregapSpn->value());
	config->writeEntry("SCSI Time Out", scsiTimeOutSpn->value());
	
	buff = driverNameTxt->text().remove(";").remove("`");
	driverNameTxt->setText(buff);
	config->writeEntry("Driver Name", buff);
	
	buff = driverOptTxt->text().remove(";").remove("`");
	driverOptTxt->setText(buff);
	config->writeEntry("Driver Options", buff);

	///////// Custom cdrdao Opt //////////////////////////////////
	config->writeEntry("Use Custom cdrdao Options", useCdrdaoCustOptChk->isChecked());
	config->writeEntry("Force cdrdao", forceCdrdaoChk->isChecked());
	config->writeEntry("Reload cdrdao", reloadChk->isChecked());
	config->writeEntry("Number of Buffers for cdrdao", numBuffersSpn->value());

	///////////////////////////////////////////////////
     config->setGroup("Erase Options");
     config->writeEntry("Dummy Erase", dummyEraseChk->isChecked());
	config->writeEntry("Eject After Erase", ejectEraseChk->isChecked());
	config->writeEntry("Check Drive First", checkDriveEraseChk->isChecked());
	config->sync();

	QByteArray data;
	if ( !kapp->dcopClient()->isAttached() )
		kapp->dcopClient()->attach();
	kapp->dcopClient()->send( "cdbakeoven", "CdboPluginManager", "updateSettings()", data );
	kapp->dcopClient()->send( "*", "CdboDialogBase", "readOptions()", data );
}
//----------------------------------------------------------------------------------------------
void CdboBurnOpt::defaults(){
	config->deleteGroup("Recording Options");
	config->deleteGroup("Erase Options");
	config->sync();
	load();
}
//----------------------------------------------------------------------------------------------------------
void CdboBurnOpt::ignoreInfSlot(){
   defPregapSpn->setEnabled(useInfChk->isChecked());
   defPregapLbl->setEnabled(useInfChk->isChecked());
}
//------------------------------------------------------------------------------------------
void CdboBurnOpt::useCustOptSlot(){
	custOptPnl->setEnabled(useCustOptChk->isChecked());
}
//------------------------------------------------------------------------------------------
void CdboBurnOpt::useCustCdrdaoOptSlot(){
	custCdrdaoOptPnl->setEnabled(useCdrdaoCustOptChk->isChecked());
}
//=========================================
void CdboBurnOpt::slotChanged(){
   emit changed(true);
}
//---------------------------------------------------------------------------------------------
int CdboBurnOpt::buttons() {
	return KCModule::Default|KCModule::Apply|KCModule::Cancel|KCModule::Ok|KCModule::Help;
}
//---------------------------------------------------------------------------------------------
QString CdboBurnOpt::quickHelp() const{
	return i18n("Recording Options.");
}
//---------------------------------------------------------------------------------------------
const KAboutData* CdboBurnOpt::aboutData() const
{

	KAboutData *myAboutData = new KAboutData(I18N_NOOP("cdbakeoven"),
					I18N_NOOP("CD Bake Oven recording options."),
                  0, 0, KAboutData::License_GPL,
                  "(c) 2001 - 2002 Alex Podolsky");

    myAboutData->addAuthor("Alex Podolsky", 0, "alexpod@users.sf.net");

    return myAboutData;
}
//---------------------------------------------------------------------------------------------
extern "C"{
    KCModule *create_cdboburnopt(QWidget *parent, const char *name){
        KGlobal::locale()->insertCatalogue("cdbakeoven");
        return new CdboBurnOpt(parent, name);
    }

}




