/***************************************************************************
                          CdboWizard.cpp  -  description
                             -------------------
    begin                : Fri Jul 14 2002
    copyright            : (C) 2001 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qpixmap.h>
#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qradiobutton.h>
#include <qdir.h>
#include <qfileinfo.h>
#include <qlineedit.h>
#include <qbuttongroup.h>
#include <qevent.h>
#include <qtimer.h>
#include <qcombobox.h>
#include <qpopupmenu.h>

#include <kstandarddirs.h>
#include <kprocess.h>
#include <klocale.h>
#include <kglobal.h>
#include <kfiledialog.h>
#include <kconfig.h>
#include <kmessagebox.h>
#include <kiconloader.h>
#include <kapplication.h>
#include <klistview.h>
#include <kurl.h>
#include <dcopclient.h>
#include <kdebug.h>

#include "CdboWizard.h"
#include "CdboWizard.moc"

#include "CdboCdromDevices.h"
#include "CdboToolsList.h"
#include "CdboTmpDirConfig.h"

#define DEFAULT_MOUNT_BASE_DIR "/mnt/"
#define DEFAULT_MOUNT_NAME "cdrom"


CdboWizard::CdboWizard( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : CdboWizardBase( parent, name, modal, fl ),
	DCOPObject( "CdboWizard" ){

		QTimer::singleShot(5, this, SLOT(initUserInfo()));

		if(!KStandardDirs::exists (locate("config", "cdbakeovenrc")))
			notFirstTimePnl->hide();

		config = new KConfig("cdbakeovenrc");
		initPix();
		load();
		
		currPage = 0;
		setNextEnabled(page(3), false);
		
		outputMenu = new QPopupMenu();
		outputMenu -> insertItem(BarIcon("txt"), i18n("Save this Information to a File"),this ,SLOT(dumpOutputSlot()));
		connect( permOutputView, SIGNAL(rightButtonPressed(QListViewItem*, const QPoint&, int)),
					 this, SLOT(popupOutputMenuSlot(QListViewItem*, const QPoint&, int)));
		permOutputView->setSorting(-1);
		deepestPage = 0;

		QTimer::singleShot(1, this, SLOT(exec()));

}
//==========================================
CdboWizard::~CdboWizard(){
	delete sidePix;
}
//============================================
void CdboWizard::initPix(){
	sidePix = new QPixmap();
	sidePix->load(locate("data", "cdbakeoven/toolbar/wizard_side.png"));

	introSidelbl->setPixmap( *sidePix );
	introSidelbl->setScaledContents( true );

	devicesSidelbl->setPixmap( *sidePix );
	devicesSidelbl->setScaledContents( true );

	toolsSidelbl->setPixmap( *sidePix );
	toolsSidelbl->setScaledContents( true );

	permSidelbl->setPixmap( *sidePix );
	permSidelbl->setScaledContents( true );

	doneSidelbl->setPixmap( *sidePix );
	doneSidelbl->setScaledContents( true );
}
//============================================
void CdboWizard::load(){
	config->setGroup("Wizard");
	if(config->readBoolEntry("Reset Settings", true))
		yesResetRBtn->setChecked(true);
	else
		noResetRBtn->setChecked(true);
}
//============================================
void CdboWizard::save(){
	QString buff;
	config->setGroup("Wizard");
	if(yesResetRBtn->isChecked()){
		QString conf = locateLocal("config", "cdbakeovenrc");
		QFile configFile(conf);
		if(configFile.exists())
			configFile.remove();
		config->setGroup("Wizard");
		config->writeEntry("Reset Settings", true);
	}
	else
		config->writeEntry("Reset Settings", false);

	if(deepestPage > 1){
		config->setGroup("Wizard Detected");
		//saving mount points
		if(mountPointsList.count() > 0){
			for(unsigned int i = 0; i < mountPointsList.count(); i++){
				config->writeEntry("Mount point for " + mountPointsList[i].section("=", 0, 0),
					mountPointsList[i].section("=", 1, 1));
			}
		}
		tmpDirConfig->saveOptions();
		toolsList->saveOptions();
	}

	config->sync();
	if(deepestPage > 0){
		devicesPnl->save();
	}
	QByteArray data;
	if ( !kapp->dcopClient()->isAttached() )
		kapp->dcopClient()->attach();
	kapp->dcopClient()->send( "cdbakeoven", "CdboPluginManager", "updateSettings()", data );
	kapp->dcopClient()->send( "*", "CdboDialogBase", "readOptions()", data );
}
//============================================
void CdboWizard::next(){
	if(currPage == 0){
		if(yesResetRBtn->isChecked())
			devicesPnl->defaults();
		devicesPnl->rescanSlot();
	}
	if(currPage == 1){
		if(!devicesPnl->hasDetectedScsi()){
			int result = KMessageBox::warningContinueCancel(this,
				i18n("There were no detected SCSI devices. Only drives recognized by "\
					"your system as SCSI devices can be used to burn CDs."),
				i18n("Warning"));
				switch(result){
					case KMessageBox::Cancel:
						return;
					case KMessageBox::Continue:
						break;
				}
		}
		QTimer::singleShot(5, this, SLOT(checkDevices()));
		if(yesResetRBtn->isChecked())
			tmpDirConfig->defaults();
		
		tmpDirConfig->readOptions();
		QTimer::singleShot(5, tmpDirConfig, SLOT(checkTmpSize()));
		
		if(yesResetRBtn->isChecked())
			QTimer::singleShot(5, toolsList, SLOT(scanTools()));
		else
			QTimer::singleShot(5, toolsList, SLOT(loadSavedTools()));
	}
	if(currPage == 2){
		if(!tmpDirConfig->bigEnouphTmp()){
			int result = KMessageBox::warningContinueCancel(this,
				i18n("Working directory has less space than the minimum required. Click 'Cancel' to stay on this page "\
				"and choose another directory."),
				i18n("Warning"));
			switch(result){
				case KMessageBox::Cancel:
					return;
				case KMessageBox::Continue:
					break;
			}
		}
		permOutputView->clear();
		initReqiredActions();
	}
	if(currPage == 3){
		setFinishEnabled(page(4), true);
	}
	currPage++;
	if(deepestPage < currPage)
		deepestPage = currPage;
	QWizard::next();
}
//============================================
void CdboWizard::back(){
	if(currPage == 4){
		setBackEnabled(page(3), true);
	}
	currPage--;
	QWizard::back();
}
//============================================
void CdboWizard::accept (){
	save();
	QWizard::accept ();
	kapp->exit(0);
}
//============================================
void CdboWizard::lunchSettingsSlot(){
	save();
	QString error;
	KApplication::kApplication()->startServiceByDesktopPath(
			locate("data", "cdbakeoven/cdbosettings.desktop"),QString(""), &error);
	if(!error.isEmpty())
		kdDebug() << "Failed to lunch  CD Bake Oven Settings Dialog::::" << error << endl;
	QWizard::accept ();
	kapp->exit(0);
}
//============================================
void CdboWizard::reject (){
	int result = KMessageBox::warningYesNoCancel(this,
			i18n("Are you sure you want to quit this Wizard?\n"\
				"If not, press 'Cancel'.\n"\
				"To quit and save all the configuration done up to this point, press 'Quit and Save'.\n"\
				"To quit and revert all settings to those before this Wizard started, press 'Quit and Revert'."),
			i18n("Really Quit?"), i18n("Quit and &Save"), i18n("Quit and &Revert"), "Confirm Wizard Exit");
	switch(result){
		case KMessageBox::Yes:
			accept ();
			return;
       	case KMessageBox::No:
			config->setGroup("Wizard");
			config->writeEntry("Wizard Used", true);
			QWizard::reject ();
			kapp->exit(0);
			return;
		case KMessageBox::Cancel:
			return;
	}
}
//============================================
void CdboWizard::closeEvent(QCloseEvent *e){
	e->ignore();
	reject ();
}
//============================================
void CdboWizard::initReqiredActions(){

	QString buff = "";
	output(i18n("Configuration report:\n "));
	toolsToFixList.clear();
	toolsList->initToolsToFixList(toolsToFixList);

//////////////////check if group configuration is required ////////////////////////////////////////////////////////////////
	if(devicesToFixList.count() != 0 || toolsToFixList.count() != 0){
		QString groups = userGroups.join(" ");
		QFileInfo info;
		bool inforceGroup = false;
		if(toolsToFixList.count() > 0){
			for(unsigned int i = 0; i < toolsToFixList.count(); i++){
				info.setFile(toolsToFixList[i]);
				if(userGroupsNames.findIndex(info.group()) == -1)
					inforceGroup = true;
			}
		}
		if(devicesToFixList.count() > 0){
			for(unsigned int i = 0; i < devicesToFixList.count(); i++){
				info.setFile(devicesToFixList[i]);
				if(userGroupsNames.findIndex(info.group()) == -1)
					inforceGroup = true;
			}
		}
		if(!inforceGroup){
			permGrpChk->setEnabled(false);
			permGrpChk->setChecked(false);

			permGroupCmb->setEnabled(false);
			output(i18n("Will reuse existing group(s)."));
		}
		else{
			permGrpChk->setEnabled(true);
			permGrpChk->setChecked(true);
			output(i18n("Changes to group settings created"));
			permGroupCmb->setEnabled(true);
		}
	}
	else{
		permGrpChk->setEnabled(false);
		permGrpChk->setChecked(false);

		permGroupCmb->setEnabled(false);
		output(i18n("No group configuration required."));
	}

//////////////////check tools ////////////////////////////////////////////////////////////////
	if(toolsToFixList.count() == 0){
		permToolsChk->setEnabled(false);
		permToolsChk->setChecked(false);
		output(i18n("All tools seem to be properly configured."));
	}
	else{
		permToolsChk->setEnabled(true);
		permToolsChk->setChecked(true);
		unsigned int i = 0;
		buff += "(" + toolsToFixList[i] + ")";
		if(toolsToFixList.count() > 1){
			for(i++; i < toolsToFixList.count(); i++)
				buff += ",(" + toolsToFixList[i] + ")";
		}
		buff = i18n("Tools %1 require 'execute' permission.").arg(buff);
		output(buff);
		buff = "";
	}

//////////////////check fstab ////////////////////////////////////////////////////////////////
	if(missingFstabList.count() == 0){
		permFstabChk->setEnabled(false);
		permFstabChk->setChecked(false);
		output(i18n("All required fstab entries seem to be in place."));
	}
	else{
		permFstabChk->setEnabled(true);
		permFstabChk->setChecked(true);
		unsigned int i = 0;
		if(missingFstabList[i].find("cdboTmpIsoLoopWhole") != -1)
			buff += "(loop device)";
		else{
			if(missingFstabList[i].endsWith("*****"))
				buff += "(" + (missingFstabList[i].left(missingFstabList[i].length() - 5)) + ")";
			else if(missingFstabList[i].find("::"))
				buff += "(" + missingFstabList[i].section("::", 0, 0) + ")";
			else
				buff += "(" + missingFstabList[i] + ")";
		}

		if(missingFstabList.count() > 1){
			for(i++; i < missingFstabList.count(); i++){
				if(missingFstabList[i].find("cdboTmpIsoLoopWhole") != -1)
					buff += ",(loop device)";
				else{
					if(missingFstabList[i].endsWith("*****"))
						buff += ",(" + (missingFstabList[i].left(missingFstabList[i].length() - 5)) + ")";
					else if(missingFstabList[i].find("::"))
						buff += ",(" + missingFstabList[i].section("::", 0, 0) + ")";
					else
						buff += ",(" + missingFstabList[i] + ")";
				}
			}
		}
		buff = i18n("Devices %1 require new entries in fstab.").arg(buff);
		//buff = "Devices " + buff + " require mount point configuration.";
		output(buff);
		buff = "";

	}

//////////////////check devices ////////////////////////////////////////////////////////////////
	if(devicesToFixList.count() == 0){
		permDevicesChk->setEnabled(false);
		permDevicesChk->setChecked(false);
		output(i18n("Permissions seem to be properly configured for all devices"));
	}
	else{
		permDevicesChk->setEnabled(true);
		permDevicesChk->setChecked(true);
		unsigned int i = 0;
		bool ld = false;
		if(devicesToFixList[i].find("loop") != 0){
			buff += "(loop device(s))";
			ld = true;
		}
		else
			buff += "(" + devicesToFixList[i] + ")";

		if(devicesToFixList.count() > 1){
			for(i++; i < devicesToFixList.count(); i++){
				if(devicesToFixList[i].find("loop") != 0){
					if(ld)
						continue;
					buff += ",(loop device)";
					ld = true;
					continue;
				}
				buff += ",(" + devicesToFixList[i] + ")";
			}
		}
		buff = i18n("Devices %1 require 'read' permission.").arg(buff);
		output(buff);
		buff = "";
	}

////////////////// final check if everything is file :) ////////////////////////
	if(devicesToFixList.count() == 0 && toolsToFixList.count() == 0 && missingFstabList.count() == 0){
		permApplyBtn->setEnabled(false);
		setNextEnabled(page(3), true);
		output(i18n(""));
		output(i18n(" \n \nNo system changes required."));
		return;
	}
	else{
		permApplyBtn->setEnabled(true);
		setNextEnabled(page(3), false);
		output(i18n(""));
		output(i18n(" \n \nReady to apply required system changes. Use 'Apply' to start."));
	}

}

//============================================
void CdboWizard::kcmshellDone(KProcess *proc){
	delete proc;
}
//////////////////////////////////////////////////////////
///////////////// USER INFO ///////////////////////////////
//////////////////////////////////////////////////////////
//============================================
void CdboWizard::initUserInfo(){
	cdboProc = new KProcess();
	*cdboProc << "id";

	connect(cdboProc, SIGNAL(processExited(KProcess *)),
      			this, SLOT(kcmshellDone(KProcess *)));
 	connect(cdboProc, SIGNAL(receivedStdout (KProcess *, char *, int)),
		this, SLOT(receivedUserInfoMessageSlot(KProcess *, char *, int)));

	cdboProc->start(KProcess::NotifyOnExit, KProcess::AllOutput);
}
//============================================
void CdboWizard::receivedUserInfoMessageSlot(KProcess *, char *buffer, int buflen){
	if(buflen <= 0) return;

	QString buff(buffer);
	buff = buff.left(buflen);

	QStringList comOutput(QStringList::split(" ",buff));
	buff = comOutput[0].mid(4, comOutput[0].length() - 4);
	buff.truncate(buff.length() - 1);

	userId = buff.left(buff.find("("));

	userName = buff.right(buff.length() - buff.find("(") - 1);

	userGroups = QStringList::split(",", comOutput[2]);
	userGroups[0] = userGroups[0].mid(7, userGroups[0].length() - 7);

	toolsInfoLbl->setText(toolsInfoLbl->text().arg(userName));

	userGroups[userGroups.count() - 1].truncate(userGroups[userGroups.count() - 1].find(")") + 1);
	permGroupCmb->insertStringList(userGroups);
	QTimer::singleShot(5, this, SLOT(pickActiveGroup()));
	QTimer::singleShot(5, toolsList, SLOT(scanTools()));
}
//============================================
void CdboWizard::initUserGroupsNames(){
	QString buff = "";
	userGroupsNames.clear();
	for(unsigned int i = 0; i < userGroups.count(); i++){
		buff = userGroups[i].right(userGroups[i].length() - userGroups[i].find("(") - 1);
		buff.truncate(buff.find(")"));
		userGroupsNames.append(buff);
	}
}
//============================================
void CdboWizard::pickActiveGroup(){

	initUserGroupsNames();//looks like an OK place to call this

///broke into separate loops becouse want to keep order
	for(unsigned int i = 0; i < userGroups.count(); i++){
		if(userGroups[i].find(i18n("cdwriter")) != -1){
			//pickedActiveGroup = userGroups[i];
			permGroupCmb->setCurrentItem(i);
			return;
		}
	}
	/*for(unsigned int i = 0; i < userGroups.count(); i++){
		if(userGroups[i].find(i18n("cdr")) != -1){
			//pickedActiveGroup = userGroups[i];
			permGroupCmb->setCurrentItem(i);
			return;
		}
	}*/
	for(unsigned int i = 0; i < userGroups.count(); i++){
		if(userGroups[i].find(i18n("cdrecording")) != -1){
			//pickedActiveGroup = userGroups[i];
			permGroupCmb->setCurrentItem(i);
			return;
		}
	}
	/*for(unsigned int i = 0; i < userGroups.count(); i++){
		if(userGroups[i].find(i18n("disk")) != -1){
			//pickedActiveGroup = userGroups[i];
			permGroupCmb->setCurrentItem(i);
			return;
		}
	}*/
	permGroupCmb->insertItem(i18n("cdwriter (new group to be created)"), 0);
	//pickedActiveGroup = userGroups[0];
}
///////////////////////////////////////////////////////////////////
/////////// DEVICES ///////////////////////////////
///////////////////////////////////////////////////////////////////
//============================================
void CdboWizard::checkDevices(){
	devicesToFixList.clear();
	missingFstabList.clear();
	QStringList devList;
	devicesPnl->getAllDevicesList(&devList);
	/*if(!devicesPnl->getAllDevicesList(&devList)){
		///more logic here
		return;
	}*/
	QString buff;
	QFileInfo info;
	for(unsigned int i = 0; i < devList.count(); i++){
		if(missingFstabList.findIndex(devList[i]) == -1)//for fstab check
			missingFstabList.append(devList[i]);
		info.setFile(devList[i]);
		if(!info.isReadable()){
			if(devicesToFixList.findIndex(buff) == -1)
				devicesToFixList.append(devList[i]);
		}
	}
	for(int i = 0 ; i <= 7; i++){
		buff = "/dev/loop" + QString::number(i);
		info.setFile(buff);
		if(!info.isReadable()){
			if(devicesToFixList.findIndex(buff) == -1)
				devicesToFixList.append(buff);
		}
	}
	QString cdboTmpIsoLoopWholePath = locateLocal("data", "cdbakeoven/cdboTmpIsoLoopWhole/");
	cdboTmpIsoLoopWholePath.truncate(cdboTmpIsoLoopWholePath.length() - 1);
	missingFstabList.append(cdboTmpIsoLoopWholePath);

	checkFstab();
}
//============================================
void CdboWizard::checkFstab(){
	mountPointsList.clear();
	QString line, found;
	QStringList leftOvers, sec;
	bool remove = false;
	unsigned int modifyOnly = 0;
	QFile fstab("/etc/fstab");
	if(fstab.open(IO_ReadOnly)) {
		QTextStream stream( &fstab );
		while ( !stream.eof() ) {
			line = stream.readLine();
			if(line.startsWith("#"))
				continue;
			for(unsigned int i = 0; i < missingFstabList.count(); i++){
				if(line.find(missingFstabList[i]) != -1){
					sec = QStringList::split(" ", line);//Mandrake is using spaces
					if(sec[0].find("\t") != -1)
						sec = QStringList::split("\t", line);//SuSE is using tabs

					//collecting mount points
					if(sec[2].find("supermount") == -1){
						if(sec[1].find(missingFstabList[i]) != -1)//loop
							mountPointsList.append(sec[0] + "=" + sec[1]);
						else
							mountPointsList.append(missingFstabList[i] + "=" + sec[1]);
					}
					else
						mountPointsList.append(missingFstabList[i] + "=" + sec[1] + "*supermount");

					if(sec[2].find("supermount") == -1 && sec[3].find("user") == -1){
						missingFstabList[i] = missingFstabList[i] + "*****";
						modifyOnly++;
					}
					else{
						found = missingFstabList[i];
						remove = true;
					}
					break;
				}
			}
			if(remove){
				missingFstabList.remove(found);
				remove = false;
			}
			else
				leftOvers.append(line);
		}
		fstab.close();
	}
	else
		return;

	if(missingFstabList.count() == 0)
		return;

	//Now, lets try a bit harder
	QFileInfo info;
	QString buff;
	int pos;
	for(unsigned int i = 0; i < leftOvers.count(); i++){
		sec = QStringList::split(" ", leftOvers[i]);//Mandrake is using spaces
		if(sec[0].find("\t") != -1)
			sec = QStringList::split("\t", leftOvers[i]);//SuSE is using tabs

		info.setFile(sec[0]);

		if(info.isSymLink()){
			found = info.readLink();
			pos = missingFstabList.findIndex(found);
			if(pos != -1){

				if(sec[2].find("supermount") == -1)//collecting mount points
					mountPointsList.append(missingFstabList[pos] + "=" + sec[1]);
				else
					mountPointsList.append(missingFstabList[pos] + "=" + sec[1] + "*supermount");

				if(sec[2].find("supermount") == -1 && sec[3].find("user") == -1){
					missingFstabList[pos] = sec[0] + "*****";
					modifyOnly++;
				}
				else
					missingFstabList.remove(found);
			}
		}
	}
	//assign new mount points for the ones that do not have it yet
	if(missingFstabList.count() > modifyOnly){
		QString newMountPoint;
		QDir dir("/mnt/");
		QStringList ex = dir.entryList();
		int c = 1;
		for(unsigned int i = 0; i < missingFstabList.count(); i++){
			if(missingFstabList[i].endsWith("*****"))
				continue;
			else if(missingFstabList[i].find("cdboTmpIsoLoopWhole") != -1){

				buff = missingFstabList[i].left(missingFstabList[i].findRev("/")) + "/sessionLoad";
				mountPointsList.append(buff + "=" + missingFstabList[i]);

				missingFstabList[i] = buff + "::" + missingFstabList[i];
			}
			else{
				while(true){//create mount points
					newMountPoint = DEFAULT_MOUNT_NAME + QString::number(c);
					if(ex.findIndex(newMountPoint) != -1){
						c++;
						continue;
					}
					else{
						newMountPoint = DEFAULT_MOUNT_BASE_DIR + newMountPoint;
						c++;
						break;
					}
				}
				mountPointsList.append(missingFstabList[i] + "=" + newMountPoint);
				missingFstabList[i] = missingFstabList[i] + "::" + newMountPoint;
			}
		}
	}
}
//==============================================================
void CdboWizard::popupOutputMenuSlot(QListViewItem *, const QPoint &point, int){
	if(permOutputView->childCount() == 0)
		return;
	outputMenu->popup( point);
}
//=====================================
void CdboWizard::dumpOutputSlot(){

	if(!permOutputView->firstChild())
		return;
    QString fileToOpen = KFileDialog::getSaveFileName(QDir::homeDirPath(),
				i18n("*|All files"),
				this,
				i18n("Save Output As..."));
	if(!fileToOpen || fileToOpen == "")
		return;
	QFile file(fileToOpen);

	if(file.exists())
		file.remove();
	if(!file.open(IO_ReadWrite))
		return;

	QTextStream stream(&file);

	stream << i18n("CDBO Configuration Tool Output:");
	stream << "\n\n";

	for(QListViewItem *i = permOutputView->firstChild(); i; i = i->nextSibling()){
		stream << i->text(0);
		stream << "\n";
	}

	QDate date = QDate::currentDate();
	stream << "\n\nCD Bake Oven. ";
	stream << date.toString();
	stream << ".";
	file.close();
}
//==============================================================
void CdboWizard::permGrpChkSlot(){
	bool yes = permGrpChk->isChecked();
	if(toolsToFixList.count() != 0){
		permToolsChk->setEnabled(yes);
		permToolsChk->setChecked(yes);
	}

	if(devicesToFixList.count() != 0){
		permDevicesChk->setEnabled(yes);
		permDevicesChk->setChecked(yes);
	}

	permFstabChk->setEnabled(yes);
	permFstabChk->setChecked(yes);

	//permUsrChk->setEnabled(yes);
	//permUsrChk->setChecked(yes);

	permApplyBtn->setEnabled(yes);

}
//////////////////////////////////////////////////////////
///////////////// OUTPUT ///////////////////////////////
//////////////////////////////////////////////////////////
//----------------------------------------------------------------------------------------------
void CdboWizard::output(QString message){
	QStringList list = QStringList::split("\n", message);

	QListViewItem *item;
	QString buff;
	for(int  i = 0; i < (int)list.count(); i++){
		buff = list[i].simplifyWhiteSpace();
		item = new QListViewItem(permOutputView, permOutputView->lastChild());

		item->setText(0," " + buff);
		permOutputView->ensureItemVisible(item);
	}
}
//////////////////////////////////////////////////////////
///////////////// CONFIG TOOL COM///////////////////////////////
//////////////////////////////////////////////////////////
//============================================
void CdboWizard::permApplySlot(){
	QString error;
	KApplication::kApplication()->startServiceByDesktopPath(
			locate("data", "cdbakeoven/cdbosetuptool.desktop"),QString(""), &error);
	if(!error.isEmpty())
		kdDebug() << "Failed to lunch  cdbosetuptool" << error << endl;
}
//============================================
void CdboWizard::setupToolAcknowledgedInstance(){

	QTimer::singleShot(5, this, SLOT(initJobInfo()));
	output(i18n(" \n \nSetup tool is up."));

	permGrpChk->setEnabled(false);
	permToolsChk->setEnabled(false);
	permFstabChk->setEnabled(false);
	permDevicesChk->setEnabled(false);
	permApplyBtn->setEnabled(false);
	permGroupCmb->setEnabled(false);
	setBackEnabled(page(3), false);
}
//===============================
void CdboWizard::initJobInfo(){

/////////////////////// base info /////////////////////////////////
	QByteArray data;
	QDataStream arg(data, IO_WriteOnly);
	QString buff = "USER::" + userName;
	arg << buff;
	buff = "USER_ID::" + userId;
	arg << buff;

/////////////////////// group info /////////////////////////////////
	if(permGrpChk->isChecked()){
		buff = "INFORCE_GROUP::TRUE";
		arg << buff;
		buff = permGroupCmb->currentText();
		if(buff.find("(new group to be created)") == -1){
			buff = buff.right(buff.length() - buff.find("(") - 1);
			buff.truncate(buff.length() - 1);
			buff = "GROUP_TO_USE::" + buff;
			arg << buff;
			buff = "NEW_GROUP::FALSE";
			arg << buff;
		}
		else{
			buff = buff.left(buff.find(" "));
			buff = "GROUP_TO_USE::" + buff;
			arg << buff;
			buff = "NEW_GROUP::TRUE";
			arg << buff;
		}
	}
	else{
		buff = "INFORCE_GROUP::FALSE";
		arg << buff;
	}

/////////////////////// tools info /////////////////////////////////
	if(permDevicesChk->isChecked()){
		buff = "NUMBER_DEVICES_TO_FIX::" + QString::number(devicesToFixList.count());
		arg << buff;
		for(unsigned int i = 0; i < devicesToFixList.count(); i++)
				arg << devicesToFixList[i];
	}
	else
		arg << "NUMBER_DEVICES_TO_FIX::0";
/////////////////////// fstab info /////////////////////////////////
	if(permFstabChk->isChecked()){
		buff = "NUMBER_FSTAB_TO_FIX::" + QString::number(missingFstabList.count());
		arg << buff;
		bool hasModifyOnly = false;
		for(unsigned int i = 0; i < missingFstabList.count(); i++){
			arg << missingFstabList[i];
			if(missingFstabList[i].endsWith("*****"))
				hasModifyOnly = true;
		}
		if(hasModifyOnly){
			buff = "HAS_FSTAB_MODIFY_ONLY_TRUE";
			arg << buff;
		}
		else{
			buff = "HAS_FSTAB_MODIFY_ONLY_FALSE";
			arg << buff;
		}
	}
	else
		arg << "NUMBER_FSTAB_TO_FIX::0";

/////////////////////// tools info /////////////////////////////////
	if(permToolsChk->isChecked()){
		buff = "NUMBER_TOOLS_TO_FIX::" + QString::number(toolsToFixList.count());
		arg << buff;
		for(unsigned int i = 0; i < toolsToFixList.count(); i++)
				arg << toolsToFixList[i];
	}
	else
		arg << "NUMBER_TOOLS_TO_FIX::0";

	if ( !kapp->dcopClient()->isAttached() )
		kapp->dcopClient()->attach();
	if(!kapp->dcopClient()->send( "cdbosetuptool*", "CdboSetupTool", "initJobInfo(QString)", data ))
		KMessageBox::error(this,
			i18n("CdboWizard Failed to communicate!"));

	//////////////////////////////////////////////////////////////////////////////////////////////////

}
//============================================
void CdboWizard::toolDone(QString success, QString extra){

	QTimer::singleShot(1, this, SLOT(checkDevices()));
	QTimer::singleShot(1, toolsList, SLOT(rescanTools()));


	if(success.find("SUCCESS") != -1){
		setNextEnabled(page(3), true);
		output(i18n(" \n \n Configuration completed successfully. Click next to complete this Wizard."));

		if(extra.find("RESTART_REQ_TRUE") != -1){
			/*message = i18n("Group configuration for %1 (current user) was modified. "\
                               "Re-login is required for those changes to take effect. There is no need to reboot your system.").
                          arg(userName);*/
			KMessageBox::sorry(this, i18n("Group configuration for %1 (current user) was modified. "\
                               "Re-login is required for those changes to take effect. There is no need to reboot your system.").
                          arg(userName), i18n("Relogin for changes") );

		}

	}
	else{
		output(i18n(" \n \n Configuration tool failed."));
		setBackEnabled(page(3), true);
	}
}
//============================================
void CdboWizard::help(){
	kapp->invokeHelp("", "cdbakeoven");
}

