/***************************************************************************
                          cdbosetuptool.cpp  -  description
                             -------------------
    begin                : Tue Jul 16 10:22:34 EDT 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sf.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qtimer.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <qdir.h>

#include <klocale.h>
#include <kglobal.h>
#include <dcopclient.h>
#include <kapplication.h>
#include <kmessagebox.h>
#include <kprocess.h>

#include "cdbosetuptool.h"
#include "cdbosetuptool.moc"


#define GROUP_FILE "/etc/group"
#define FSTAB "/etc/fstab"

CdboSetupTool::CdboSetupTool(QWidget *parent, const char *name) : QWidget(parent, name),
		DCOPObject( "CdboSetupTool" )
{
	kapp->dcopClient()->setAcceptCalls (true) ;
	QTimer::singleShot(1, this, SLOT(verifyInstance()));
	restartRequired = false;

}
//======================================================
CdboSetupTool::~CdboSetupTool(){
}
//======================================================
void CdboSetupTool::verifyInstance(){
	cdboProc = new KShellProcess();
	*cdboProc << "id -un";

	connect(cdboProc, SIGNAL(processExited(KProcess *)),
      			this, SLOT(kcmshellDone(KProcess *)));
 	connect(cdboProc, SIGNAL(receivedStdout (KProcess *, char *, int)),
		this, SLOT(receivedVerifyInstanceMessageSlot(KProcess *, char *, int)));
	connect(cdboProc, SIGNAL(receivedStderr (KProcess *, char *, int)),
		this, SLOT(receivedVerifyInstanceMessageSlot(KProcess *, char *, int)));

	cdboProc->start(KProcess::NotifyOnExit, KProcess::AllOutput);
}
//======================================================
void CdboSetupTool::receivedVerifyInstanceMessageSlot(KProcess *, char *buffer, int buflen){

	if(buflen <= 0) return;

	QString buff(buffer);
	buff = buff.left(buflen);
	if(buff.stripWhiteSpace() != "root"){
		KMessageBox::error(this,
			i18n("This tool requires superuser privileges."));
		done(false);
	}
	else
		acknowledgeInstance();
}
//================================================================
bool CdboSetupTool::process(const QCString &fun, const QByteArray &data, QCString& replyType, QByteArray &replyData){
	if(fun == "done(bool)"){
		QDataStream arg(data, IO_ReadOnly);
		QString message;
		arg >> message;
		if(message.find("SUCCESS") != -1){
			done(true);
		}
		else{
			done(false);
		}
	}
	else if(fun == "initJobInfo(QString)"){
		 output(i18n("Starting to apply configuration..."));

		groupFileBackedup = false;
		fstabBackedup = false;

		QDataStream arg(data, IO_ReadOnly);
		QString buff = "", message;
		arg >> buff;
		userName = buff.section("::", 1); //"USER::"
		buff = "";

		arg >> buff;
		userId = buff.section("::", 1); //"USER_ID::"
		buff = "";
		//message = " \n userName =  " + userName + "\n userId = " + userId;

/////////////////////// group info /////////////////////////////////

		arg >> buff;
		buff = buff.section("::", 1); //"INFORCE_GROUP::";

		if(buff.stripWhiteSpace() == "TRUE"){
			inforceGroup = true;
			//message += " \n inforceGroup =  TRUE";
			buff = "";
			arg >> buff;
			groupToUse = buff.section("::", 1); //"GROUP_TO_USE::"
			buff = "";
			//message += " \n groupToUse =  " + groupToUse;
			arg >> buff;
			buff = buff.section("::", 1); //NEW_GROUP::"
			if(buff.stripWhiteSpace() == "TRUE"){
				newGroup = true;
			//	message += " \n newGroup =  TRUE";
			}
			else{
				newGroup = false;
			//	message += " \n newGroup =  FALSE";
			}
			buff = "";
		}
		else{
			inforceGroup = false;
		//	message += " \n inforceGroup =  FALSE";
		}
		buff = "";
/////////////////////// tools info /////////////////////////////////
		arg >> buff;
		buff = buff.section("::", 1); //"NUMBER_DEVICES_TO_FIX::"
		bool ok;
		int c = buff.toInt(&ok);
		if(ok && c > 0){
			for(int i = 0; i < c; i++){
				arg >> buff;
				devicesToFixList.append(buff);
			//	message += " \n deviceToFix =  " + buff;
				buff = "";
			}
		}
		//else
		//	message += " \n deviceToFix =  0";
		buff = "";

/////////////////////// fstab info /////////////////////////////////
		arg >> buff;
		buff = buff.section("::", 1); //"NUMBER_FSTAB_TO_FIX::"
		c = buff.toInt(&ok);
		if(ok && c > 0){
			for(int i = 0; i < c; i++){
				arg >> buff;
				missingFstabList.append(buff);
		//		message += " \n missingFstab =  " + buff;
				buff = "";
			}
		}
		//else
		//	message += " \n missingFstab =  0";

		buff = "";
		arg >> buff;
		if(buff == "HAS_FSTAB_MODIFY_ONLY_TRUE"){
			hasFstabModifyOnly = true;
		//	message += " \n hasFstabModifyOnly =  TRUE";
		}
		else{
			hasFstabModifyOnly = false;
		//	message += " \n hasFstabModifyOnly =  FALSE";
		}
		buff = "";

/////////////////////// devices info /////////////////////////////////
		arg >> buff;
		buff = buff.section("::", 1); //"NUMBER_TOOLS_TO_FIX::"
		c = buff.toInt(&ok);
		if(ok && c > 0){
			for(int i = 0; i < c; i++){
				arg >> buff;
				toolsToFixList.append(buff);
		//		message += " \n toolsToFix =  " + buff;
				buff = "";
			}
		}
		//else
		//	message += " \n toolsToFix =  0";
		//buff = "";

		//output(message);
		QTimer::singleShot(1, this, SLOT(begin()));
		//done();
	}
	else
		return DCOPObject::process( fun, data, replyType, replyData );
	return true;
}
//======================================================
void CdboSetupTool::acknowledgeInstance(){
	QByteArray data;
	if ( !kapp->dcopClient()->isAttached() )
		kapp->dcopClient()->attach();
	if(!kapp->dcopClient()->send( "*", "CdboWizard",
				"setupToolAcknowledgedInstance()", data )){
		KMessageBox::error(this,
			i18n("Internal error occurred. CdboSetupTool failed to communicate with main application!"));
		done(false);
		return;
	}
}
//======================================================
void CdboSetupTool::output(QString mess){
	QByteArray data;
	QDataStream arg(data, IO_WriteOnly);
	arg << mess;
	if ( !kapp->dcopClient()->isAttached() )
		kapp->dcopClient()->attach();

	if(!kapp->dcopClient()->send( "*", "CdboWizard",
				"output(QString)", data )){
		KMessageBox::error(this,
			i18n("Internal error occurred. CdboSetupTool failed to communicate with main application!"));
		done(false);
		return;
	}
}
//======================================================
void CdboSetupTool::done(bool success){
	//KMessageBox::information(this,
			//i18n("Debugging message", "done called"));

	QByteArray data;
	QDataStream arg(data, IO_WriteOnly);
	QString buff;
	if(success){
		buff = "SUCCESS";
		arg << buff;
	}
	else{
		buff = "FAILURE";
		arg << buff;
	}
	if(!restartRequired){
		buff = "RESTART_REQ_FALSE";
		arg << buff;
	}
	else{
		buff = "RESTART_REQ_TRUE";
		arg << buff;
	}
	if ( !kapp->dcopClient()->isAttached() )
		kapp->dcopClient()->attach();

	if(!kapp->dcopClient()->send( "*", "CdboWizard",
				"toolDone(QString, QString)", data )){
		KMessageBox::error(this,
			i18n("Internal error occurred. CdboSetupTool failed to communicate to main application!"));
	}


	kapp->exit();
}
//============================================
void CdboSetupTool::kcmshellDone(KProcess *proc){
	delete proc;
}
//================================================================
//void CdboSetupTool::initJobInfo(QString){
//}
//================================================================
void CdboSetupTool::begin(){
	QString buff;
/////////////////////// group info /////////////////////////////////
	if(newGroup && inforceGroup){
		if(!groupFileBackedup){
			buff = i18n("Backing up group configuration file...");
			output(buff);
			if(!backupGroupFile()){
				buff = i18n("Failed to back up group configuration file.");
				output(buff);
				done(false);
				return;
			}
			return;
		}
		buff = i18n("Creating group '%1'...").arg(groupToUse);
		output(buff);
		if(!createNewGroup()){
			buff = i18n("Failed to create group %1.").arg(groupToUse);
			output(buff);
			done(false);
			return;
		}
	}
/////////////////////// tools info /////////////////////////////////
	if(toolsToFixList.count() > 0){
		if(!makeToolUsable()){
			buff = i18n("Failed to setup permissions for %1.").arg(toolsToFixList[0]);
			output(buff);
			done(false);
			return;
		}
		return;
	}
/////////////////////// fstab info /////////////////////////////////
	if(missingFstabList.count() > 0){
		if(!fstabBackedup){
			buff = i18n("Backing up fstab...");
			output(buff);
			if(!backupFstab()){
				buff = i18n("Failed to back up fstab.");
				output(buff);
				done(false);
				return;
			}
			return;
		}
		if(!addFstabEntries()){
			buff = i18n("Failed to add required entries to fstab.");
			output(buff);
			done(false);
			return;
		}
	}
/////////////////////// devices info /////////////////////////////////
	if(devicesToFixList.count() > 0){
		if(!makeDeviceReadable()){
			buff = i18n("Failed to setup permissions for %1.").arg(devicesToFixList[0]);
			output(buff);
			done(false);
			return;
		}
		return;
	}
	done(true);
}
//================================================================
bool CdboSetupTool::createNewGroup(){

	QString line, buff;
	bool groupExists = false;
	int newGroupId = 0;
	QFile oldGroupFile(QString(GROUP_FILE) + ".cdbo.bak"), groupFile(GROUP_FILE);

	if(oldGroupFile.open(IO_ReadOnly) && groupFile.open(IO_WriteOnly)) {
		QTextStream inStream( &oldGroupFile ), outStream( &groupFile );
		while ( !inStream.eof() ) {
			line = inStream.readLine();

			if(groupExists){
				outStream << line << "\n";
				continue;
			}

			buff = line.section(":",0, 0);
			if(buff == groupToUse){
				groupExists = true;
				buff = i18n("Group '%1' already exists; will only assign user '%2' to it.").
                       arg(groupToUse).
                       arg(userName);
				output(buff);
				buff = line.section(":",3);
				if(!buff.isEmpty()){//try to prevent adding same user twice
					QStringList sec = QStringList::split(",", line);
					bool found = false;
					for(unsigned int i = 0; i < sec.count(); i++){
						if(sec[i] == userName){
							buff = i18n("User '%1' is already part of group '%2'.")
                                   .arg(userName)
                                   .arg(groupToUse);
							output(buff);
							found = true;
						}
					}
					if(found)
						continue;
				}

				line = line + "," + userName;
				outStream << line << "\n";
				continue;
			}
			buff = line.section(":",2, 2);
			if(buff.toInt() > newGroupId && line.section(":",0, 0) != "nogroup")
				newGroupId = buff.toInt();

			outStream << line << "\n";
		}
		oldGroupFile.close();
		if(!groupExists){
			line = groupToUse + ":*:" + QString::number(newGroupId + 1) + ":" + userName;
			outStream << line << "\n";
		}
		groupFile.close();
	}
	else{
		KMessageBox::error(this,
			i18n("Failed to open group configuration file."));
		return false;
	}

	newGroup = false;
	restartRequired = true;
	if(!groupExists){
		buff = i18n("Done. Group '%1' created successfully.").arg(groupToUse);
		output(buff);
	}
	else{
		buff = i18n("Done. Group configuration completed successfully.");
		output(buff);
	}
	return true;
}
//================================================================
bool CdboSetupTool::backupGroupFile(){
	QFile file(QString(GROUP_FILE) + ".cdbo.bak");
	if(file.exists())
		file.remove();

	cdboProc = new KShellProcess();
	*cdboProc << "cp " + QString(GROUP_FILE) + " " + QString(GROUP_FILE) + ".cdbo.bak";

	connect(cdboProc, SIGNAL(processExited(KProcess *)),
			this, SLOT(backupGroupDone(KProcess *)));
	return cdboProc->start(KProcess::NotifyOnExit, KProcess::NoCommunication);
}
//============================================
void CdboSetupTool::backupGroupDone(KProcess *proc){
	delete proc;
	groupFileBackedup = true;
	QString buff = i18n("Done. Old version saved as: %1.cdbo.bak").arg(GROUP_FILE);
	output(buff);
	QTimer::singleShot(1, this, SLOT(begin()));
}
//================================================================
bool CdboSetupTool::makeToolUsable(){

	QString file = toolsToFixList[0];
	QFileInfo info(file);
	if(info.isSymLink()){
		file = info.readLink();
		if(file.left(1) != "/")
			file = info.dirPath(true) + "/" + file;
	}
	if(inforceGroup){
		if(info.group() != groupToUse){
			return applyGroupToUse(file);
		}
	}
	QString buff = i18n("Adding execute permission to %1.").arg(toolsToFixList[0]);
	output(buff);

	cdboProc = new KShellProcess();
	*cdboProc << "chmod g+x " + file;

	connect(cdboProc, SIGNAL(processExited(KProcess *)),
			this, SLOT(makeToolUsableDone(KProcess *)));
	return cdboProc->start(KProcess::NotifyOnExit, KProcess::NoCommunication);
}
//============================================
void CdboSetupTool::makeToolUsableDone(KProcess *proc){
	delete proc;
	toolsToFixList.remove(toolsToFixList[0]);
	QString buff = "Done.";
	output(i18n(buff.utf8()));
	QTimer::singleShot(1, this, SLOT(begin()));
}
//============================================
bool CdboSetupTool::addFstabEntries(){

	QString line, buff;
	QFile fstab(FSTAB);
	QString separator = " ";
	if(hasFstabModifyOnly){
		QFile oldFstab(QString(FSTAB) + ".cdbo.bak");
		QStringList sec;
		QString found;
		bool remove = false;
		if(oldFstab.open(IO_ReadOnly) && fstab.open(IO_WriteOnly)) {
			QTextStream inStream( &oldFstab ), outStream( &fstab );
			while ( !inStream.eof() ) {
				line = inStream.readLine();
				for(unsigned int i = 0; i < missingFstabList.count(); i++){
					if(missingFstabList[i].endsWith("*****")){
						if(line.startsWith(missingFstabList[i].left(missingFstabList[i].length() - 5))){
							sec = QStringList::split(" ", line);//Mandrake is using spaces
							if(sec[0].find("\t") != -1){
								sec = QStringList::split("\t", line);//SuSE is using tabs
								separator = "\t";
							}
							sec[3] = "user," +sec[3];
							line = sec.join(separator);
							found = missingFstabList[i];
							remove = true;

							buff = i18n("Fixed fstab entry for '%1'.").arg(sec[0]);
							output(buff);
							break;
						}
					}
				}
				if(remove){
					missingFstabList.remove(found);
					remove = false;
				}
				outStream << line + "\n";
			}
			oldFstab.close();
			fstab.close();
		}
		else{
			KMessageBox::error(this,
				i18n("Failed to open fstab."));
				return false;
		}
	}
	else{//determine separator - not really needed, but for the sake of consistency.
		if(fstab.open(IO_ReadOnly)) {
			QTextStream inStream( &fstab );
			line = inStream.readLine();
			if(line.find("\t") != -1)
				separator = "\t";
			fstab.close();
		}
		else{
			KMessageBox::error(this,
				i18n("Failed to open fstab."));
				return false;
		}
	}
	if(missingFstabList.count() == 0){
		buff = i18n("Done. Required entries were modified in fstab successfully.");
		output(buff);
		return true;
	}
	QDir dir;
	QString newMountPoint, device;
	if(fstab.open(IO_WriteOnly | IO_Append)){
		QTextStream stream( &fstab );
		for(unsigned int i = 0; i < missingFstabList.count(); i++){
			device = missingFstabList[i].section("::", 0, 0);
			newMountPoint = missingFstabList[i].section("::", 1, 1);
			if(newMountPoint.find("cdboTmpIsoLoopWhole") != -1){
				line = i18n("#Following line was added for ISO image mounting by cdbakeoven for user '%1'.").arg(userName);
				stream << line + "\n";

				line = device + separator + newMountPoint + separator + "iso9660" + separator + "ro,user,noauto,unhide,loop";
				stream << line + "\n";

				buff = i18n("Added new fstab entry for 'loop device'.");
				output(buff);
			}
			else{
				line = device + separator + newMountPoint + separator + "auto" + separator + "ro,noauto,user,exec 0 0";
				stream << line + "\n";
				buff = i18n("Added new fstab entry for '%1'.").arg(device);
				output(buff);
			}
			if(!dir.exists(newMountPoint)){
				dir.mkdir(newMountPoint);
				buff = i18n("Created new mount point for device '%1' at '%2'.")
                       .arg(device)
                       .arg(newMountPoint);
				output(buff);
			}
		}
		fstab.close();
	}
	else{
		KMessageBox::error(this,
			i18n("Failed to open fstab."));
		return false;
	}
	missingFstabList.clear();
	buff = i18n("Done. Required entries were added to fstab successfully.");
	output(buff);

	return true;
}
//================================================================
bool CdboSetupTool::backupFstab(){

	QFile file(QString(FSTAB) + ".cdbo.bak");
	if(file.exists())
		file.remove();

	cdboProc = new KShellProcess();
	*cdboProc << "cp " + QString(FSTAB) + " " + QString(FSTAB) + ".cdbo.bak";

	connect(cdboProc, SIGNAL(processExited(KProcess *)),
			this, SLOT(backupFstabDone(KProcess *)));
	return cdboProc->start(KProcess::NotifyOnExit, KProcess::NoCommunication);
}
//============================================
void CdboSetupTool::backupFstabDone(KProcess *proc){
	delete proc;
	fstabBackedup = true;
	QString buff = i18n("Done. Old version saved as: %1.cdbo.bak").arg(FSTAB);
	output(buff);
	QTimer::singleShot(1, this, SLOT(begin()));
}
//================================================================
bool CdboSetupTool::makeDeviceReadable(){
	QString file = devicesToFixList[0];
	QFileInfo info(file);
	if(info.isSymLink()){
		file = info.readLink();
		if(file.left(1) != "/")
			file = info.dirPath(true) + "/" + file;
	}
	if(inforceGroup){
		if(info.group() != groupToUse){
			return applyGroupToUse(file);
		}
	}
	QString buff = i18n("Adding read permission to %1.").arg(devicesToFixList[0]);
	output(buff);

	cdboProc = new KShellProcess();
	*cdboProc << "chmod g+r " + file;
	connect(cdboProc, SIGNAL(processExited(KProcess *)),
			this, SLOT(makeDeviceReadableDone(KProcess *)));
	return cdboProc->start(KProcess::NotifyOnExit, KProcess::NoCommunication);
}
//============================================
void CdboSetupTool::makeDeviceReadableDone(KProcess *proc){
	delete proc;
	devicesToFixList.remove(devicesToFixList[0]);
	QString buff = i18n("Done.");
	output(buff);
	QTimer::singleShot(1, this, SLOT(begin()));
}
//================================================================
bool CdboSetupTool::applyGroupToUse(QString file){
	QString buff = i18n("Changing group for %1.").arg(file);
	output(buff);

	cdboProc = new KShellProcess();

	*cdboProc << "chgrp " + groupToUse + " " + file;
	connect(cdboProc, SIGNAL(processExited(KProcess *)),
			this, SLOT(applyGroupToUseDone(KProcess *)));
	return cdboProc->start(KProcess::NotifyOnExit, KProcess::NoCommunication);
}
//============================================
void CdboSetupTool::applyGroupToUseDone(KProcess *proc){
	delete proc;
	QString buff = i18n("Done.");
	output(buff);
	QTimer::singleShot(1, this, SLOT(begin()));
}


