/***************************************************************************
                          CdboPluginManager.h  -  description
                             -------------------
    begin                : Sat Nov 27 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/  

#ifndef CDBOPLUGINMANAGER_H
#define CDBOPLUGINMANAGER_H

#include <qptrlist.h>
#include <qdict.h>
#include <kparts/part.h>
#include <dcopobject.h>

class KConfig;
class CdboShell;
class CdboMainView;

class CdboPluginManagerHelper;


class CdboPluginManager : public QObject, public virtual DCOPObject{
	
	Q_OBJECT
	
public:
	~CdboPluginManager();
	
	static CdboPluginManager* self();
	
	void setParent(CdboShell *newParent){ parent = newParent; }
	void setMainView(CdboMainView* view) { mainView = view; }

	void handleURL(const KURL&  url );
	QString prettySupportedExt(){ return m_prettySupportedExt; }
	QString supportedExt(){ return m_supportedExt; }
	void registerExt(QObject *receiver, const QStringList &extList, const QStringList &prettyExtList);
	void loadPart(const QString& partLibName, const QString& partName, const KURL& url, bool readWrite = false, const QString& icon = "", const QString& type = "");

	QWidget* partsHolder();
	void addView(KParts::Part* newPart);
	void createHelper(const QString& name, const QString& partLibName, const QString& partName,
							const QString& icon = "", const QString& shortcut = "", const QString& comment = "");
	void setLastHelper(CdboPluginManagerHelper* helper){ lastHelper = helper; }
	bool process(const QCString &fun, const QByteArray &data, QCString&, QByteArray &);
	
public slots:
	void createTop();
	void loadViewPlugins();
	void loadLastViews();
	bool queryClose();
	void openURL(const KURL&  url );
	void activateHelper(const QString &type);
	void activateHelper(const QString &type, const KURL& url);
    void updateSettings();
	
signals:
	void loadURL(const KURL& url);

private:
	CdboPluginManager(QObject* parent = 0, const char* name = 0);
	void loadPlugin(KConfig * pointer);
		
	static CdboPluginManager* s_self;
	
	CdboPluginManagerHelper* lastHelper;
	QPtrList<CdboPluginManagerHelper> helpers;
	QDict<QObject> extPluginMap;
	
	QString m_prettySupportedExt;
	QString m_supportedExt;
	
	CdboShell *parent;
	CdboMainView* mainView;
};


/////////////////////////////////////////


class CdboPluginManagerHelper : public QObject{
	Q_OBJECT
public:
	CdboPluginManagerHelper(QObject *manager, const char *name=0);
	~CdboPluginManagerHelper();

	KParts::Part * createPart();

public slots:
	void loadView();
	void loadView(const KURL& url);

public:
	QString partLibName;
	QString partName;
	QString comment;
	QString icon;
	bool readWrite;
};

#endif
