/***************************************************************************
                          CdboShell.cpp  -  description
                             -------------------
    begin                : Sat Oct 19 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/
#include <qtimer.h>

#include <kparts/partmanager.h>
#include <kpopupmenu.h>
#include <kkeydialog.h>
#include <kconfig.h>
#include <kurl.h>
#include <klocale.h>
#include <kedittoolbar.h>
//#include <kiconloader.h>
#include <kaction.h>
#include <kstdaction.h>
#include <kstatusbar.h>
#include <kdebug.h>
#include <kapplication.h>
#include <kfiledialog.h>
#include <kprogress.h>

#include "CdboShell.h"
#include "CdboShell.moc"

#include "CdboMainView.h"
#include "CdboPluginManager.h"

CdboShell::CdboShell( const char *name)
	: KParts::MainWindow(name), DCOPObject( "cdbakeoven" ){

	config = new KConfig("cdbakeovenrc");
	setXMLFile("cdbakeovenui.rc", true);

	setStandardToolBarMenuEnabled(true);

	initStatusBar();

	m_partManager = new KParts::PartManager( this );
	connect( m_partManager, SIGNAL( activePartChanged( KParts::Part * ) ),
		this, SLOT( createGUI( KParts::Part * ) ) );

	mainView = new CdboMainView(this, "CdboMainView");
	connect( m_partManager, SIGNAL( activePartChanged( KParts::Part * ) ),
		mainView, SLOT( activeChanged( KParts::Part * ) ) );

	CdboPluginManager::self()->setParent(this);
	CdboPluginManager::self()->setMainView(mainView);
	QTimer::singleShot(1, CdboPluginManager::self(), SLOT(createTop()));

	qApp->processEvents();
	setupActions();
	setCentralWidget(mainView);

	createGUI(0l);
	plugNewViewAction();

	QTimer::singleShot(1, CdboPluginManager::self(), SLOT(loadViewPlugins()));
	qApp->processEvents();
	QTimer::singleShot(1, CdboPluginManager::self(), SLOT(loadLastViews()));
	qApp->processEvents();
	readOptions();
	updateProgress(80);
}
//================================
CdboShell::~CdboShell(){
	delete config;
	delete m_partManager;
}
//================================
void CdboShell::setupActions(){

	KStdAction::open(this, SLOT(slotFileOpen()), actionCollection());
	closeAction = KStdAction::close(mainView, SLOT(slotCloseView()), actionCollection());
	new KAction(i18n("Clo&se All"), KShortcut::null(),
		mainView, SLOT(queryClose()), actionCollection(), "close_all_action");

	KStdAction::quit(this, SLOT(slotFileQuit()), actionCollection());
	KStdAction::keyBindings(this, SLOT(optionsConfigureKeys()), actionCollection());
	KStdAction::configureToolbars(this, SLOT(optionsConfigureToolbars()), actionCollection());

	fileOpenRecent = KStdAction::openRecent(this, SLOT(slotFileOpenRecent(const KURL&)), actionCollection());

	newViewMenuAction = new KActionMenu( i18n( "&New" ), "filenew", this );
	newViewMenuAction->setDelayed( false );

	showStatusbarAction = KStdAction::showStatusbar(this, SLOT(optionsShowStatusbar()), actionCollection());
}
//------------------------------------------------------------------------------------
void CdboShell::initStatusBar(){

 	status = new QLabel(statusBar(), "status");
	status->setMaximumHeight(18);
	statusBar()->addWidget(status,12, true);

	progress = new KProgress(100, statusBar(), "progress");
	progress->setMaximumHeight(18);
  	statusBar()->addWidget(progress, 3, true);

	info = new QLabel(statusBar(), "info");
	info->setMaximumHeight(18);
	//info->setAlignment(AlignHCenter | AlignVCenter | ExpandTabs);
  	statusBar()->addWidget(info, 3, true);

  	QLabel *spacer2 = new QLabel(statusBar(), "spacer2");
	spacer2->setMaximumHeight(18);
	statusBar()->addWidget(spacer2,1, true);

}
//================================
void CdboShell::plugNewViewAction() {
	QPtrList<KAction> lst;
	lst.append( newViewMenuAction );
	plugActionList( "new_view_menu", lst );
}
//================================
void CdboShell::unplugNewViewAction() {
	unplugActionList( "new_view_menu" );
}
//================================
void CdboShell::replugNewViewAction() {
	unplugNewViewAction();
	plugNewViewAction();
}
//================================
void CdboShell::plugIntoNewViewAction(KAction* action) {
	action->plug( newViewMenuAction->popupMenu() );
}
//================================
void CdboShell::unplugFromNewViewAction(KAction* action) {
	action->plug( newViewMenuAction->popupMenu() );
}
//=======================================
bool CdboShell::queryExit(){
	saveOptions();
	return true;
}
//=======================================
bool CdboShell::queryClose(){
	return CdboPluginManager::self()->queryClose();
}
//================================
void CdboShell::saveOptions(){
	saveMainWindowSettings(config, "General Options");
	fileOpenRecent->saveEntries(config);
	mainView->saveOptions();
	config->sync();
}
//================================
void CdboShell::readOptions(){
	applyMainWindowSettings(config, "General Options");
	fileOpenRecent->loadEntries(config);
}
//================================
void CdboShell::slotFileQuit(){
	close();
}
//================================
void CdboShell::load(const KURL& url){
	CdboPluginManager::self()->handleURL( url );
	fileOpenRecent->addURL( url );
}
//================================
void CdboShell::optionsShowStatusbar(){
    if (showStatusbarAction->isChecked())
        statusBar()->show();
    else
        statusBar()->hide();
}
//================================
void CdboShell::slotFileOpen(){
	slotSetStatusBarText(i18n("Opening file..."));
	KURL url=KFileDialog::getOpenURL(QDir::homeDirPath(),
		i18n("%1|Supported Files (%2)\n*|All files").arg(CdboPluginManager::self()->supportedExt()).arg(CdboPluginManager::self()->prettySupportedExt()),
		this, i18n("Open ISO Image or CD List File"));
	if(!url.isEmpty()){
		load( url );
	}
}
//================================
void CdboShell::slotFileOpenRecent(const KURL &url){
	load( url );
}
//================================
void CdboShell::slotSetStatusBarText(const QString &text){
	status->setText(text);
}
//================================
void CdboShell::newType(const QString &type){
  	info->setText(i18n("Mode: %1").arg(type));
}
//================================
void CdboShell::slotSetCaption(const QString &newCaption, bool modified){
	setCaption(kapp->makeStdCaption(newCaption, true, modified));
}
//================================
void CdboShell::updateProgress(int value){
	if(value == 100){
		progress->hide();
		return;
	}
	if(value == 0)
		progress->show();

	progress->setValue(value);
}
//================================
void CdboShell::optionsConfigureKeys(){
    KKeyDialog::configureKeys(actionCollection(), "cdbakeovenui.rc");
}
//================================
void CdboShell::optionsConfigureToolbars(){
	saveMainWindowSettings (config, "General Options");
	KEditToolbar dlg(factory());
	connect(&dlg,SIGNAL(newToolbarConfig()),this,SLOT(slotNewToolbarConfig()));
	dlg.exec();
}
//================================
void CdboShell::slotNewToolbarConfig(){
	replugNewViewAction();
	applyMainWindowSettings(config, "General Options");
}


