/***************************************************************************
                          CdboSrcVeiwPart.h  -  description
                             -------------------
    begin                : Sat Oct 27 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/ 

#include <qtimer.h>

#include <kaboutdata.h>
#include <kaction.h>
#include <kinstance.h>
#include <klocale.h>
#include <kxmlguifactory.h>
#include <kglobal.h>
#include <kiconloader.h>

#include "CdboSrcVeiwPart.h"
#include "CdboSrcVeiwPart.moc"

#include "CdboFileListView.h"
#include "CdboVersion.h"

K_EXPORT_COMPONENT_FACTORY( libcdbosrcview, CdboSrcVeiwFactory );

//===================================
CdboSrcVeiwPart::CdboSrcVeiwPart(QWidget *parentWidget, const char *widgetName,
                            QObject *parent, const char *name, const QStringList& )
	: KParts::ReadOnlyPart( parent, name ){
	
	KGlobal::locale()->insertCatalogue("cdbakeoven");
	
	setInstance( CdboSrcVeiwFactory::instance() );

	topView = new CdboFileListView(parentWidget, widgetName);
	setWidget(topView);
	topView->readOptions();
	setupActions();
	
	setXMLFile( "cdbosrcviewpartui.rc" );
	
	QTimer::singleShot(1, this, SLOT(loadPath()));
	
	connect(topView, SIGNAL(addToCD(KURL::List&)), 
		this, SLOT(emitAddToCD(KURL::List&)));
}
//===================================
CdboSrcVeiwPart::~CdboSrcVeiwPart(){
	topView->saveOptions();
}
//===================================
void CdboSrcVeiwPart::loadPath(){
	topView->clearHistory();

	topView->addBranch("/", i18n("Root Directory"), SmallIcon("tux"));
	topView->addBranch(QDir::homeDirPath(), i18n("Home Directory"), SmallIcon("folder_home"), true);
}
//===================================
void CdboSrcVeiwPart::setupActions(){
	topView->enableBookmarkAction();
	topView->enableAddToCDAction();
	actionCollection()->addDocCollection(topView->getActionCollection());
	
}
//===================================
KAboutData* CdboSrcVeiwPart::createAboutData(){
	return new KAboutData( "cdbosrcviewpart", I18N_NOOP("CD Bake Oven"),
		CDBO_VERSION, I18N_NOOP("Thank you for using CD Bake Oven - CD Burning facility for KDE"),
		KAboutData::License_GPL,
		I18N_NOOP("(c) 2001-2002, Alexy Podolsky"));
}

