/***************************************************************************
                          CDRWatcherModule.cpp  -  description
                             -------------------
    begin                : Tue Sep 24 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sf.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qfileinfo.h>
#include <qdir.h>
#include <qregexp.h>

#include <kdatastream.h> 
#include <kstandarddirs.h>
#include <kio/netaccess.h>
#include <kglobalsettings.h>
#include <kdebug.h>
#include <klocale.h>
#include <kconfig.h>
#include <kdirnotify_stub.h>
#include <kmessagebox.h>


#include "cdrwatcher.moc"
#include "cdrwatcher.h"

#include "CdboCDRom.h"
#include "CdboConstants.h"

//===========================================
CDRWatcherModule::CDRWatcherModule(const QCString &obj)
    : KDEDModule(obj){
	 
	connectedToDirWatcher = false;
	cdromDict.setAutoDelete(true);
	reloadCDRs();
}
//===========================================
CDRWatcherModule::~CDRWatcherModule(){
	cdromDict.clear();
}
//===========================================
void CDRWatcherModule::reloadCDRs(){

	CdboCDRom *cdrom;
	KConfig *config = new KConfig("cdbakeovenrc");
	config->setGroup("Target Devices");
	QStringList targetsList(config->readListEntry("Targets List"));
	cdromDict.clear();
	QString device;

	for (uint i = 0; i < targetsList.count(); i++ ) {
		device = config->readEntry(targetsList[i] + " device");
		cdrom = new CdboCDRom(device, this, targetsList[i].latin1());
		
		//cdrom->setDevName(config->readEntry(targetsList[i], i18n("UnknownCD-R(W)")));
		
		connect( cdrom, SIGNAL(stateChanged(CdboCDRom&) ),
    		this, SLOT(cdromStateChanged(CdboCDRom&)) );
			
		cdrom->startMonitoringState();
		cdromDict.insert(targetsList[i], cdrom);
	}
	delete config;
}
//===========================================
void CDRWatcherModule::cdromStateChanged(CdboCDRom &cdrom){

	Cdbo::MediaStatus currMediaStatus = cdrom.currentMediaStatus();

	switch(currMediaStatus) {
		case Cdbo::MediaTrayIsOpen:
		case Cdbo::MediaUnknown:
		case Cdbo::MediaAudio:
				debug("CDRWatcherModule::cdromStateChanged>> cdrom name: <" + cdrom.devPath() + 
				">\ncurrMediaStatus: " + QString::number(currMediaStatus));
				removeLink(cdrom);
			break;

		case Cdbo::MediaUnmounted:
		case Cdbo::MediaMounted:
				debug("CDRWatcherModule::cdromStateChanged>> cdrom name: <" + cdrom.devPath() + 
					">\ncurrMediaStatus: " + QString::number(currMediaStatus));
				createLink(cdrom);
			break;

	}
}
//===========================================
void CDRWatcherModule::createLink(CdboCDRom &cdrom){
//debug("CDRWatcherModule::createLink>>should create "
//					+ KGlobalSettings::desktopPath() + QString(cdrom.name())  + ".desktop");
	QString name = cdrom.name();
	name.replace(QRegExp("/"), "-");
	QString slaveSource = "newcd:/" + name + "/";
	createListPointer(name);
	createDesktopLink(name, slaveSource);
}
//===========================================
void CDRWatcherModule::removeLink(CdboCDRom &cdrom){
//debug("CDRWatcherModule::removeLink>> should remove "
//					+ KGlobalSettings::desktopPath() + QString(cdrom.name()) + ".desktop");
	
	QString fileName = cdrom.name();
	fileName = KGlobalSettings::desktopPath() + fileName.replace(QRegExp("/"), "-") + ".desktop";
	debug("fileName = " + fileName);
	if(!QFile::exists(fileName))
		return;
		
		
	KConfig *deskFile = new KConfig(fileName);
	deskFile->setGroup("Desktop Entry");
	QString pointerName = deskFile->readEntry("URL", "");
	pointerName = pointerName.section('/', 1, 1);
	
	debug("pointerName = " + pointerName);
	
	QString src = getSourceForPointer(pointerName);
	
	debug("src = " + src);
	
	QDir thisDir(src);
	if(thisDir.exists()){
	debug("thisDir.exists()");
		
		thisDir.setFilter (QDir::All|QDir::Hidden|QDir::System);
		QStringList filesList = thisDir.entryList();
		filesList.remove(".");
		filesList.remove("..");
		
		bool cleanup = false;
		if(filesList.count() > 0){
			debug("filesList.count()= " + QString::number(filesList.count()));
			int result = KMessageBox::questionYesNo(0,
				i18n("There are files associated with this CD-R(W). Should those files be preserved for the next time?"),
				i18n("Preserve the files?"));
				switch(result){
					case KMessageBox::Yes:
						cleanup = false;
						break;
					case KMessageBox::No:
						cleanup = true;
						break;
				}
				debug("done with mess");
		
		}
		else{
			cleanup = true;
			debug("filesList.count() < 0");
		}
		if(cleanup){
			//KIO::NetAccess::del(KURL(src));
			KIO::NetAccess::del(KURL("newcd:/" + pointerName));
			KIO::NetAccess::del(locateLocal("data", "cdbakeoven/newCDsHome/") + pointerName + ".desktop");
			KDirNotify_stub allDirNotify("*", "KDirNotify*");
			allDirNotify.FilesAdded( KURL("newcd:/" + pointerName) );
		}
	}
	
	KIO::NetAccess::del(KURL(fileName));
}
//=============================================
void CDRWatcherModule::createDesktopLink(QString name, QString slaveSource){
	QString fileName = KGlobalSettings::desktopPath() + name + ".desktop";
	
	
	if(QFile::exists(fileName))
		KIO::NetAccess::del(KURL(fileName));
	
	QString line, buff;
	QFile desktopFile(fileName), templateFile(locate("data", "cdbakeoven/templates/cdr_template.desktop"));

	if(templateFile.open(IO_ReadOnly) && desktopFile.open(IO_WriteOnly)) {
		QTextStream inStream( &templateFile ), outStream( &desktopFile );
		while ( !inStream.eof() )
			outStream << inStream.readLine() << "\n";
			
		templateFile.close();
		outStream << "URL=" + slaveSource + "\n";
		outStream << "Name=" + name + "\n";
		desktopFile.close();
	}
}
//=====================================
void CDRWatcherModule::createListPointer(QString name, QString sourcePath, QString newCDsHome, QString type){

	if(newCDsHome.isEmpty())
		newCDsHome = locateLocal("data", "cdbakeoven/newCDsHome/");
	if(newCDsHome.endsWith("/"))
		newCDsHome = newCDsHome + "/";
	
	//KIO::NetAccess::del(KURL(newCDsHome + name + ".desktop"));
	KConfig *pointer = new KConfig(newCDsHome + name + ".desktop");
	if(type.isEmpty())
		type = "data";
	pointer->writeEntry("Type", type);
	
	pointer->writeEntry("Name",name);
	pointer->writeEntry("Size", 0);
	
	if(sourcePath.isEmpty())
		sourcePath = locateLocal("data", "cdbakeoven/newCDsHome/" + name + "/");
	pointer->writeEntry("Source", sourcePath);
	
	KDirNotify_stub allDirNotify("*", "KDirNotify*");
	allDirNotify.FilesAdded( KURL("newcd:/"/* + name*/) );

	pointer->sync();
	delete pointer;
}
//=====================================
QString CDRWatcherModule::getSourceForPointer(QString pointerName){
	pointerName = locateLocal("data", "cdbakeoven/newCDsHome/") + pointerName + ".desktop";

	QFileInfo d(pointerName);
	if(!d.exists ())
		return QString::null;

	KConfig *listFile = new KConfig(pointerName);
	return listFile->readEntry("Source", QString::null);
}
//===========================================
extern "C" {
	KDEDModule *create_cdrwatcher(const QCString &obj){
		return new CDRWatcherModule(obj);
	}
};



//=========================================
void CDRWatcherModule::debug(QString mess, QString fname){
	return;
	if(fname.isEmpty())
		fname = "/home/alexpod/NewCDProtocol";
	QFile file(fname);

	
	if(!file.open(IO_ReadWrite))
		return;

	file.at( file.size() ); 
	QTextStream stream(&file);
	
	QDate today = QDate::currentDate();
	QTime now = QTime::currentTime();
	stream <<  "[" + QString::number(today.month()) +
		":" + QString::number(today.day()) +
		":" + QString::number(now.hour()) +
		":" + QString::number(now.minute()) +
		":" + QString::number(now.second()) + "] ";
		
		
	stream << mess;
	stream << "\n";
	
	
		
	file.close();
}



