/***************************************************************************
                          kio_newcd.cpp  -  description
                             -------------------
    begin                : Sun Sep 1 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <sys/stat.h>
#include <stdlib.h>

#include <qfile.h>

#include <kurl.h>
#include <kdebug.h>
#include <klocale.h>
#include <kinstance.h>
#include <kmimemagic.h>

#include "kio_newcd.h"
#include "newcd_engine.h"

using namespace KIO;

extern "C" { int kdemain(int argc, char **argv); }

int kdemain( int argc, char **argv ){
	KInstance instance( "kio_newcd" );

	if(argc != 4){
		fprintf(stderr, "Usage: newcd protocol domain-socket1 domain-socket2\n");
		exit(-1);
	}
	
	NewCDProtocol slave(argv[2], argv[3]);
	slave.dispatchLoop();
	return 0;
}

NewCDProtocol::NewCDProtocol( const QCString &pool, const QCString &app ) :
	SlaveBase( "newcd", pool, app ){

}
//======================================
NewCDProtocol::~NewCDProtocol(){

}
//=========================================
void NewCDProtocol::listDir( const KURL & url ){
NewCDProtocolEngine::self()->debug("NewCDProtocol::listDir>> got url: " + url.prettyURL());
	UDSEntryList entryList;
	QStringList path = QStringList::split('/', url.path(), false);
	QString listFileName = path[0];


	if (path.count() == 0)
		listRoot();
	
	else if(path.count() == 1){
		if(!NewCDProtocolEngine::self()->load(entryList, listFileName, "")){
			error(KIO::ERR_MALFORMED_URL, url.prettyURL());
			return;
		}
		unsigned int i = 0;
		while(i < entryList.count()){
			listEntry(entryList[i],false);
			i++;
		}
		if(i > 0)
			listEntry(entryList[i], true);
	}
	else{
		path.remove(listFileName);
		if(!NewCDProtocolEngine::self()->load(entryList, listFileName, "/" + path.join("/") + "/")){
			error(KIO::ERR_MALFORMED_URL, url.prettyURL());
			return;
		}
		unsigned int i = 0;
		while(i < entryList.count()){
			listEntry(entryList[i],false);
			i++;
		}
		if(i > 0)
			listEntry(entryList[i], true);
	}
	
	listEntry(KIO::UDSEntry(), true);
	finished();
}
//=========================================
void NewCDProtocol::listRoot(){
	UDSEntryList rootList;
	if(!NewCDProtocolEngine::self()->populateRootList(rootList)){
		error(KIO::ERR_INTERNAL, i18n("No CD Lists Found"));
		return;
	}
	unsigned int i = 0;
	while(i < rootList.count()){
		listEntry(rootList[i],false);
		i++;
	}
	if(i > 0)
		listEntry(rootList[i], true);
	finished();
}
//=========================================
void NewCDProtocol::stat( const KURL & url ){
NewCDProtocolEngine::self()->debug("NewCDProtocol::stat>> got url: " + url.prettyURL());
	UDSEntry entry;

	QStringList path = QStringList::split('/', url.path(), false);
	QString listFileName = path[0];
	path.remove(".");

	if(path.count() == 0)
		NewCDProtocolEngine::self()->createDirEntry(entry, "My CDs", "newcd:/", "inode/new-cd-list");


		
	else if(path.count() == 1){
		if(!NewCDProtocolEngine::self()->statEntry(entry, listFileName, "")){
			error(KIO::ERR_MALFORMED_URL, url.prettyURL());
			return;
		}
	}
	else{
		
		path.remove(listFileName);
		QString intPath = "/" + path.join("/");
		if((url.path()).endsWith("/"))
			intPath = intPath + "/";
		if(!NewCDProtocolEngine::self()->statEntry(entry, listFileName, intPath) ){
			QString redirectUrl;
			if(!NewCDProtocolEngine::self()->getRedirect(listFileName, intPath, redirectUrl)){
				error(KIO::ERR_MALFORMED_URL, url.prettyURL());
				return;
			}
			redirection(KURL("file:/" + redirectUrl));
			finished();
			return;
		}
	}
	statEntry(entry);

	finished();
}
//=========================================
void NewCDProtocol::mkdir( const KURL& url, int){
//NewCDProtocolEngine::self()->debug("NewCDProtocol::mkdir>> got url: " + url.prettyURL());


	QStringList path = QStringList::split('/', url.path(), false);
	path.remove(".");
	QString listFileName = path[0];
	path.remove(listFileName);
	
	if(NewCDProtocolEngine::self()->exists(listFileName, path.join("/"))){
		error(KIO::ERR_DIR_ALREADY_EXIST, url.prettyURL());
		return;
	}
	
	
	if(!NewCDProtocolEngine::self()->mkdir(listFileName, path.join("/"))){
		error( KIO::ERR_COULD_NOT_MKDIR, url.prettyURL() );
		return;
	}
	finished();
}
//=========================================
void NewCDProtocol::del( const KURL& url, bool isfile){
NewCDProtocolEngine::self()->debug("NewCDProtocol::del>> got url: " + url.prettyURL());
	QStringList path = QStringList::split('/', url.path(), false);

	path.remove(".");
	QString listFileName = path[0];
	path.remove(listFileName);
	
	if(path.count() < 1 && listFileName == i18n("New Data CD")){//try to have at least one
		error(KIO::ERR_CANNOT_DELETE, url.prettyURL());
		return;
	}
	
	if(!NewCDProtocolEngine::self()->exists(listFileName, path.join("/"))){
		error(KIO::ERR_DOES_NOT_EXIST, url.prettyURL());
		return;
	}
	
	
	if(!NewCDProtocolEngine::self()->remove(listFileName, path.join("/"), isfile)){
		error( KIO::ERR_CANNOT_DELETE, url.prettyURL() );
		return;
	}
	finished();

}
//=========================================
void NewCDProtocol::put( const KURL& url, int, bool, bool){
	NewCDProtocolEngine::self()->debug("NewCDProtocol::put>> got url: " + url.prettyURL());
	
	QStringList path = QStringList::split('/', url.path(), false);
	QString redirectUrl;
	QString listFileName = path[0];

	path.remove(listFileName);
	path.remove(".");

	if(path.count() < 1){
		error(KIO::ERR_COULD_NOT_ACCEPT, url.prettyURL());
		return;
	}
	
	bool hasMore = true;
	uint fileSize = 0, size;
	do{
		QByteArray buffer;
		dataReq();
	
		readData(buffer); 
		size = buffer.size();
		if(size == 0)
			hasMore = false;
		else
			fileSize += size;
			
			
		NewCDProtocolEngine::self()->debug("NewCDProtocol::put>> got size: " +QString::number(fileSize));
		buffer.resize(0);
	}
	while(hasMore);
	
	if(!NewCDProtocolEngine::self()->canFit(listFileName, fileSize)){
		error(KIO::ERR_DISK_FULL, url.prettyURL());
		return;
	}
	
	if(!NewCDProtocolEngine::self()->getRedirect(listFileName, path.join("/"), redirectUrl)){
		error(KIO::ERR_MALFORMED_URL, url.prettyURL());
		return;
	}
	redirection(KURL("file:/" + redirectUrl));
		
	
	finished();
}
//=========================================
/*void NewCDProtocol::copy( const KURL &src, const KURL &dest, int permissions, bool overwrite ){
NewCDProtocolEngine::self()->debug("NewCDProtocol::copy>> got src: " + src.path());
NewCDProtocolEngine::self()->debug("NewCDProtocol::copy>> got dest: " + dest.path());
	
	finished();
}
//=========================================
void NewCDProtocol::rename(const KURL &src, const KURL &dest, bool overwrite){
NewCDProtocolEngine::self()->debug("NewCDProtocol::rename>> got src: " + src.path());
NewCDProtocolEngine::self()->debug("NewCDProtocol::rename>> got dest: " + dest.path());
	
	finished();

}*/

//=========================================
void NewCDProtocol::get( const KURL & url){
NewCDProtocolEngine::self()->debug("NewCDProtocol::get>> got url: " + url.prettyURL());

	QStringList path = QStringList::split('/', url.path(), false);
	QString redirectUrl;
	QString listFileName = path[0];

	path.remove(listFileName);
	path.remove(".");

	if(path.count() < 1){
		error(KIO::ERR_MALFORMED_URL, url.prettyURL());
		return;
	}
	if(!NewCDProtocolEngine::self()->getRedirect(listFileName, path.join("/"), redirectUrl)){
		error(KIO::ERR_MALFORMED_URL, url.prettyURL());
		return;
	}
	redirection(KURL("file:/" + redirectUrl));
	finished();
}


