/***************************************************************************
                          CdboAudioListView.h  -  description
                             -------------------
    begin                : Sun Nov 18 2001
    copyright            : (C) 2001 by Alex Podolsky
    email                : alexpod@users.sf.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/ 

#ifndef CDBOAUDIOLISTVIEW_H
#define CDBOAUDIOLISTVIEW_H

#include <qwidget.h>
#include <qevent.h>
#include <qdict.h>
#include <qptrlist.h>

#include <kio/job.h>
#include <klistview.h>

class QPopupMenu;
class QListViewItem;

class KActionCollection;
class KAction;
class KActionMenu;

class KConfig;

class CdboAudioEstimate;
class CdboAudioItem;

class CdboAudioListView : public KListView  {
   Q_OBJECT
public:
	CdboAudioListView( QWidget *parent=0L, const char *name=0L);
	~CdboAudioListView();

	KActionCollection* getActionCollection() { return actionCollection; }
	void plugMenuAction(KAction* action);
	
	KConfig* getConfig(){ return config; }
	
	
	static void getMetaInfo(const QString &path, QString &title, QString &artist, QString &length);

	void setEstimate(CdboAudioEstimate *e);
	inline QStringList* getFileList(){ return &fList;}
	bool canAccept(QString path, bool &stopHere);
	void disablePreviewAction(bool doIt);

public slots:
	void moveDown();
	void moveUp();
	void reload();
	void slotProperties();
	void remove();
	void removeAll();
	void cancelOperation();
	void updateSettings();
	void previewSlot();

signals:
	void modified();
	void processStarted();
	void processDone();
	void addItems(KURL::List &urls);
	void selectedURL(const KURL &);
	void selectedURLs(KURL::List &urls);
	void haveStatus(const QString &);
	
	void limitReached();


protected slots:
	void openWithSlot();
	void trackDoubleClicked ( QListViewItem *);
	
	void popupMenuSlot(QListViewItem*, const QPoint&, int);
	void audioPickerDone(KIO::Job *);

	void slotEntries( KIO::Job*, const KIO::UDSEntryList &);
	void checkActions();

protected:
	virtual bool acceptDrag (QDropEvent* event) const;	
	
	bool isValidType(QString path);
	void correctPositions();
	void setupActions();
	
	bool pickAudioFromDir(QString path);
	
	bool tryToAdd(QString path);
	bool addFileItem(QString path, QString artist, QString trackName, QString length);
	
	KActionMenu* actionMenu;

private:
	QStringList fList;
	KActionCollection *actionCollection;
	KConfig* config;

	CdboAudioEstimate *estimate;
	QPtrList <KIO::Job>jobs;
	
	bool firstMenuInsert;
	bool previewEnabled;
};

#endif
