/***************************************************************************
                          CdboComOutDialog.cpp  -  description
                             -------------------
    begin                : Fri Jun 1 2001
    copyright            : (C) 2001 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qtimer.h>
#include <qapplication.h>
#include <qframe.h>
#include <qheader.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qpopupmenu.h>
#include <qtextview.h>
#include <qstringlist.h>
#include <qtextstream.h>
#include <qdir.h>
#include <qfile.h>
#include <qdatetime.h>

#include <klistview.h>
#include <kfiledialog.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kprocess.h>
#include <kmessagebox.h>
#include <kcursor.h>
#include <krun.h>

#include "CdboComOutDialog.h"
#include "CdboComOutDialog.moc"

#define SCAN_SCSI 0
#define SCAN_CD 1
#define UNLOCK_CD 2

CdboComOutDialog::CdboComOutDialog( QWidget* parent,  const char* name, bool modal)
    : QDialog( parent, name, modal ){
	if ( !name )
		setName( "CdboComOutDialog" );
	resize( 501, 421 );

	layout = new QVBoxLayout( this );
	layout->setSpacing( 6 );
	layout->setMargin( 11 );

	outputView = new KListView( this, "outputView" );
	//outputView->setRootIsDecorated( true );
	outputView->header()->hide();

	outputView->setLineWidth( 2 );
	outputView->addColumn( i18n( "Name" ) );
	layout->addWidget( outputView );

	line = new QFrame( this, "line" );
	line->setFrameStyle( QFrame::HLine | QFrame::Sunken );
	layout->addWidget( line );

	closeBtn = new QPushButton( this, "closeBtn" );
	closeBtn->setText( i18n( "&Close" ) );
	QToolTip::add(  closeBtn, i18n( "Close this dialog." ) );
	layout->addWidget( closeBtn );

	menu = new QPopupMenu();
	menu -> insertItem( BarIcon("reload"), i18n("Reload Info"),   this ,SLOT(slotReload()));
	menu -> insertSeparator();
	menu -> insertItem(BarIcon("txt"), i18n("Save this Information to a File"),this ,SLOT(dumpSlot()));

	connect( outputView, SIGNAL(rightButtonPressed(QListViewItem*, const QPoint&, int)),
					this, SLOT(popupMenuSlot(QListViewItem*, const QPoint&, int)));

	connect( closeBtn, SIGNAL( clicked() ), this, SLOT( quitSlot() ) );
	outputView->setSorting( -1 );
	doIt = true;
	lastOperation = -1;
	lastDevice = QString::null;
	visible = false;
}
//===================================================
CdboComOutDialog::~CdboComOutDialog(){
}
//==============================================================
void CdboComOutDialog::popupMenuSlot(QListViewItem *, const QPoint &point, int){
	menu->popup( point);
}
//===========================================================
void CdboComOutDialog::dumpSlot(){
	if(!outputView->firstChild())
		return;
    QString fileToOpen = KFileDialog::getSaveFileName(QDir::homeDirPath(),
				i18n("*|All files"),
				this,
				i18n("Save Process Output As..."));
	if(!fileToOpen || fileToOpen == "")
		return;
	QFile file(fileToOpen);

	if(file.exists())
		file.remove();
	if(!file.open(IO_ReadWrite))
		return;

	QTextStream stream(&file);
	for(QListViewItem *i = outputView->firstChild(); i; i = i->nextSibling()){
		stream << i->text(0);
		stream << "\n";
	}
	QDate date = QDate::currentDate();
	stream << "\n\nCD Bake Oven. ";
	stream << date.toString();
	stream << ".";
	file.close();
}
//============================================================
void CdboComOutDialog::quitSlot(){
	accept();
}
//============================================================
void CdboComOutDialog::slotReload(){
	switch(lastOperation){
		case SCAN_SCSI: outputView->clear(); getScsiDetailes(lastDevice); break;
		case SCAN_CD: outputView->clear(); getCDDetailes(lastDevice); break;
		case UNLOCK_CD: outputView->clear(); unlockCD(lastDevice); break;
	}
}
//===================================================
void CdboComOutDialog::getScsiDetailes(QString dev){
	if(dev.isNull()){
		quitSlot();
		return;
	}
	lastDevice = dev;
	lastOperation = SCAN_SCSI;
	doIt = true;
	cdboProc = new KShellProcess();
	KConfig *config = new KConfig("cdbakeovenrc");
	config->setGroup("Wizard Detected");
	QString exe = config->readEntry("Path to cdrecord", "cdrecord");
	KRun::shellQuote(exe);
	*cdboProc << exe << "-prcap" << "dev=" + dev;

	connect(cdboProc, SIGNAL(processExited(KProcess *)),
      			this, SLOT(processDoneSlot(KProcess *)));
 	connect(cdboProc, SIGNAL(receivedStdout (KProcess *, char *, int)),
		this, SLOT(receivedMessageSlot(KProcess *, char *, int)));
	connect(cdboProc, SIGNAL(receivedStderr (KProcess *, char *, int)),
		this, SLOT(receivedMessageSlot(KProcess *, char *, int)));

	if(!cdboProc->start(KProcess::NotifyOnExit, KProcess::AllOutput)){
		KMessageBox::error(this, i18n("Failed to get detailed info on '%1'. " \
			"Please make sure you have enough privileges.").arg(dev));
		quitSlot();
	}
	else{
		if(!visible){
			QTimer::singleShot(1, this, SLOT(exec()));
			visible = true;
		}
		setCursor(KCursor::waitCursor());
	}
}
//===================================================
void CdboComOutDialog::getCDDetailes(QString dev){
	if(dev.isNull()){
		quitSlot();
		return;
	}
	lastDevice = dev;
	lastOperation = SCAN_CD;
	doIt = false;
	cdboProc = new KShellProcess();
	KConfig *config = new KConfig("cdbakeovenrc");
	config->setGroup("Wizard Detected");
	QString exe = config->readEntry("Path to cdrdao", "cdrdao");
	KRun::shellQuote(exe);
	*cdboProc << exe << "disk-info" << "--device " + dev;

	config->setGroup("cdrdao Driver Mappings");
	*cdboProc << "--driver" <<  config->readEntry("Driver for " + dev, "generic-mmc");

	connect(cdboProc, SIGNAL(processExited(KProcess *)),
      			this, SLOT(processDoneSlot(KProcess *)));
 	connect(cdboProc, SIGNAL(receivedStdout (KProcess *, char *, int)),
		this, SLOT(receivedMessageSlot(KProcess *, char *, int)));
	connect(cdboProc, SIGNAL(receivedStderr (KProcess *, char *, int)),
		this, SLOT(receivedMessageSlot(KProcess *, char *, int)));

	if(!cdboProc->start(KProcess::NotifyOnExit, KProcess::AllOutput)){
		KMessageBox::error(this, i18n("Failed to get detailed info for CD in '%1'. " \
			"Please make sure you have enough privileges.").arg(dev));
		quitSlot();
	}
	else{
		if(!visible){
			QTimer::singleShot(1, this, SLOT(exec()));
			visible = true;
		}
		setCursor(KCursor::waitCursor());
	}
}
//===================================================
void CdboComOutDialog::unlockCD(QString dev){
	if(dev.isNull()){
		quitSlot();
		return;
	}
	lastDevice = dev;
	lastOperation = UNLOCK_CD;
	doIt = false;
	cdboProc = new KShellProcess();
	KConfig *config = new KConfig("cdbakeovenrc");
	QString exe = config->readEntry("Path to cdrdao", "cdrdao");
	KRun::shellQuote(exe);
	*cdboProc << exe << "unlock" << "--device " + dev;

	config->setGroup("cdrdao Driver Mappings");
	*cdboProc << "--driver" <<  config->readEntry("Driver for " + dev, "generic-mmc");

	connect(cdboProc, SIGNAL(processExited(KProcess *)),
      			this, SLOT(processDoneSlot(KProcess *)));
 	connect(cdboProc, SIGNAL(receivedStdout (KProcess *, char *, int)),
		this, SLOT(receivedMessageSlot(KProcess *, char *, int)));
	connect(cdboProc, SIGNAL(receivedStderr (KProcess *, char *, int)),
		this, SLOT(receivedMessageSlot(KProcess *, char *, int)));

	if(!cdboProc->start(KProcess::NotifyOnExit, KProcess::AllOutput)){
		KMessageBox::error(this, i18n("Failed to unlock '%1'. " \
			"Please make sure you have enough privileges.").arg(dev));
		quitSlot();
	}
	else{
		if(!visible){
			QTimer::singleShot(1, this, SLOT(exec()));
			visible = true;
		}
		setCursor(KCursor::waitCursor());
	}
}
//---------------------------------------------------------------------------------------------
void CdboComOutDialog::receivedMessageSlot(KProcess *, char *buffer, int buflen){
	if(buflen > 0){
		QString buff(buffer);
		buff = buff.left(buflen);
		//cerr << buff.latin1();
		append(buff);
	}
}
//---------------------------------------------------------------------------------------------
void CdboComOutDialog::processDoneSlot(KProcess *proc){
	delete proc;
	cdboProc = 0;
	setCursor(KCursor::arrowCursor());
	if(lastOperation == UNLOCK_CD){
		if(proc->normalExit() && proc->exitStatus() == 0)
			append(i18n("Done, try to eject it now."));
		else
			append(i18n("Done."));
	}
}
//----------------------------------------------------------------------------------------------
void CdboComOutDialog::append(QString message){
	QStringList list = QStringList::split("\n", message, true);

	QListViewItem *item, *par;
	QString buff;
	for(int  i = 0; i < (int)list.count(); i++){
		buff = list[i].simplifyWhiteSpace();
		//cerr << "++++++++" << buff.latin1() << "++++++\n";
		if(buff.startsWith("WARNING: Unit not ready, still trying...")){
			item = new QListViewItem(outputView, outputView->lastChild());
			item->setText(0, i18n(" ERROR: Device is not ready."));
			item = new QListViewItem(outputView, outputView->lastChild());
			item->setText(0, i18n(" Either there is no disk in this device or wrong driver is chosen for it."));
			if(cdboProc){
				cdboProc->kill();
				delete cdboProc;
			}
			setCursor(KCursor::arrowCursor());
			return;
		}
		if(!doIt && buff.startsWith("Using")){
			doIt = true;
		}

		if(!doIt)
			continue;
		par = outputView->lastChild();
		if(par && par->text(0) == " " && buff.isEmpty())
			continue;
		item = new QListViewItem(outputView, outputView->lastChild());
		item->setText(0," " + buff);
	}

}
