/***************************************************************************
                          CdboComOutListView.cpp  -  description
                             -------------------
    begin                : Thu May 31 2001
    copyright            : (C) 2001 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qheader.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qscrollbar.h>
#include <qtextstream.h>
#include <qfile.h>
#include <qpixmap.h>
#include <qpainter.h>
#include <qregexp.h>
#include <qpopupmenu.h>
#include <qdatetime.h>

#include <kglobal.h>
#include <kiconloader.h>
#include <kfiledialog.h>
#include <kconfig.h>
#include <klocale.h>
#include <kdebug.h>


#include "CdboComOutListView.h"
#include "CdboComOutListView.moc"
#include "CdboConstants.h"

#define TYPE_COL 0
#define PROGRESS_COL 1

#define TOTAL_PROGRESS_OUT 100
#define PROGRESS_OUT 101




class CdboProgressItem : public KListViewItem{

public:
	CdboProgressItem(QListView* parent, QListViewItem* after): KListViewItem(parent, after){}
	CdboProgressItem(QListViewItem* parent, QListViewItem *after ): KListViewItem(parent, after){}
	~CdboProgressItem(){}
	int percent;
};

//---------------------------------------------------------------------------------------------
CdboComOutListView::CdboComOutListView( QWidget* parent,  const char* name)
    : KListView( parent, name)
{
	if ( !name )
		setName( "CdboComOutListView" );


	addColumn( i18n( "Type" ) );
	addColumn( i18n( "Message" ) );
	setShowSortIndicator( false );
	setFullWidth(true);
	setColumnWidth(0, 80);
	setColumnWidth(1, 530);

	setSorting(-1);

	QToolTip::add(  this, i18n( "Process output monitor" ) );
	QWhatsThis::add(  this, i18n( "Process output monitor" ) );

	menu = new QPopupMenu();
	menu -> insertItem(BarIcon("txt"), i18n("Save Process Output to a Log File."),this ,SLOT(dumpSlot()));
	menu -> insertItem(BarIcon("filesaveas"), i18n("Save Log to New Location."),this ,SLOT(dumpAsSlot()));

	connect( this, SIGNAL(rightButtonPressed(QListViewItem*, const QPoint&, int)),
					 this, SLOT(popupMenuSlot(QListViewItem*, const QPoint&, int)));
	config = 0;
	clearOutput();
}
//---------------------------------------------------------------------------------------------
CdboComOutListView::~CdboComOutListView(){
}
//---------------------------------------------------------------------------------------------
void CdboComOutListView::readOptions(){
	KConfig *config = new KConfig("cdbakeovenrc");
	config->setGroup("Default Settings");
	lastLogFile = config->readEntry("Last Log File", "");
	
	delete config;
}
//---------------------------------------------------------------------------------------------
void CdboComOutListView::saveOptions(){
	KConfig *config = new KConfig("cdbakeovenrc");
	config->setGroup("Default Settings");
	config->writeEntry("Last Log File", lastLogFile);
	config->sync();
	delete config;
}
//---------------------------------------------------------------------------------------------
void CdboComOutListView::clearOutput() {
	clear();
	progressDict.clear();
	resetConfig();
	buttomItem = 0;
	prevItem = 0;
	config->setGroup("Default Settings");
	if(config->readNumEntry("Level Of Output", Cdbo::StandartOutput) == Cdbo::QuiteOutput)
		minimumOut = true;
	else
		minimumOut = false;
}
//---------------------------------------------------------------------------------------------
void CdboComOutListView::resetConfig(){
	if(config)
		delete config;
	config = new KConfig("cdbakeovenrc");
}
//---------------------------------------------------------------------------------------------
void CdboComOutListView::output(int type, QString message){
	if(minimumOut)
		processMinimumOutMess(type, message.stripWhiteSpace());
	else
		processFullOutMess(type, message.stripWhiteSpace());
	emit outputDirty();
}
//============================================
void CdboComOutListView::initByType(int type, QString *mtype, QString *pixmap){
	switch(type){
		case Cdbo::ErrorMessage:{
			*mtype = i18n("Error: ");
			*pixmap = "error";
			break;
		}
		case Cdbo::ProcessMessage:{
			*mtype = i18n("Process: ");
			*pixmap = "process";
			break;
		}
		case Cdbo::InfoMessage:{
			*mtype = i18n("Info: ");
			*pixmap = "info";
			break;
		}
		case Cdbo::StatusOkMessage:{
			*mtype = i18n("Status: ");
			*pixmap = "ok";
			break;
		}
		case Cdbo::StatusNotOkMessage:{
			*mtype = i18n("Status: ");
			*pixmap = "remove";
			break;
		}
		case TOTAL_PROGRESS_OUT:{
			*mtype = i18n("Total Progress: ");
			*pixmap = "color_line";
			break;
		}
		case PROGRESS_OUT:{
			*mtype = i18n("Progress: ");
			*pixmap = "color_line";
			break;
		}
	}
}
//=======================================
void CdboComOutListView::processMinimumOutMess(int type, QString message){
	if(type == Cdbo::ProcessMessage ||
			type == Cdbo::InfoMessage ||
			type == Cdbo::OverwritePrevious){
		return;
	}
	processFullOutMess(type, message);

}
//=======================================
void CdboComOutListView::processFullOutMess(int type, QString message){
	QListViewItem * item;
	
	if(type == Cdbo::OverwritePrevious && prevItem)
		item = prevItem;
	else{
		QString mtype, pixmap, buff;
		initByType(type,&mtype,&pixmap);

		if(buttomItem)
			item = new QListViewItem(this, buttomItem);
		else
			item = new QListViewItem(this);
		buttomItem = item;
			
		item->setPixmap( TYPE_COL, UserIcon(pixmap, KIcon::SizeSmall ));
		item->setText( TYPE_COL, mtype);
	}
	prevItem = item;
	
	item->setText( PROGRESS_COL, message);
	if(verticalScrollBar()->value() == verticalScrollBar()->maxValue() && item == buttomItem)
		ensureItemVisible(item);
}
//=======================================
void CdboComOutListView::percent(QString procId, int percentVal, QString optMess){

	CdboProgressItem* progressItem = progressDict[procId];
	if(!progressItem){

		CdboProgressItem* progressInfoItem = progressDict[procId + "_percentInfo"];
		if(!progressInfoItem){
			progressItem = new CdboProgressItem(this, buttomItem);
			buttomItem = progressItem;
		}
		else{
			progressItem = new CdboProgressItem(this, progressInfoItem);
			if(progressInfoItem == buttomItem)
				buttomItem = progressItem;
		}
		QString mtype, pixmap;
		initByType(PROGRESS_OUT,&mtype,&pixmap);
		progressItem->setPixmap( TYPE_COL, SmallIcon(pixmap));
		progressItem->setText( TYPE_COL, mtype);

		progressDict.insert(procId,  progressItem);
	}
	progressItem->percent = percentVal;
	if(!optMess.isNull())
		updateProgress(percentVal, progressItem, optMess);
	else
		updateProgress(percentVal, progressItem);

}
//=======================================
void CdboComOutListView::percentInfo(QString procId, QString message){
	CdboProgressItem* progressInfoItem = progressDict[procId + "_percentInfo"];
	if(!progressInfoItem){
		CdboProgressItem* progressItem = progressDict[procId];
		if(!progressItem){
			progressInfoItem = new CdboProgressItem(this, buttomItem);
			buttomItem = progressInfoItem;
		}
		else
			progressInfoItem = new CdboProgressItem(this, progressItem->itemAbove());
		
		progressInfoItem->percent = 0;
		QString mtype, pixmap;
		initByType(Cdbo::StatusOkMessage, &mtype, &pixmap);
		progressInfoItem->setPixmap( TYPE_COL, UserIcon(pixmap, KIcon::SizeSmall ));
		progressInfoItem->setText( TYPE_COL, mtype);

		progressDict.insert(procId + "_percentInfo",  progressInfoItem);
	}

	progressInfoItem->setText( PROGRESS_COL, message);

	if(verticalScrollBar()->value() == verticalScrollBar()->maxValue() && progressInfoItem == buttomItem)
		ensureItemVisible(progressInfoItem);
}
//=================================================
void CdboComOutListView::updateProgress(int percentDone, QListViewItem *thisItem, QString optMess){

	int h = fontMetrics().lineSpacing();
	if(h <= 0)
		return;

	int w = columnWidth(PROGRESS_COL)/2;
	if(w <= 0)
		return;
	if(w > 200)
		w = 200;

	QPixmap *pix = new QPixmap( w, h );
	if( pix == 0 )
		return;
	config->setGroup("Default Settings");
	pix->fill(config->readColorEntry("Process Left Color", &Qt::white));//out back color

	QPainter painter(pix);
	painter.setPen(black);

	painter.drawRect(0,0,w,h);


	painter.setBrush(config->readColorEntry("Process Done Color", &Qt::darkGreen));//out done color
	painter.setPen(config->readColorEntry("Process Left Color", &Qt::white));//out left color

	painter.drawRect(1,1,(int)(((float)pix->width()-2)/ 100 * (percentDone)),pix->height()-2);
	thisItem->setPixmap ( PROGRESS_COL, *pix );
	thisItem->setText( PROGRESS_COL, QString::number(percentDone) + "%" + optMess);

	painter.end();
	delete pix;

	if(verticalScrollBar()->value() == verticalScrollBar()->maxValue() && thisItem == buttomItem)
		ensureItemVisible(thisItem);
}
//===============================================
void CdboComOutListView::dumpAsSlot(){
	QString prev = lastLogFile;
	lastLogFile = "";
	if(!dumpSlot())
		lastLogFile = prev;
}
//===============================================
bool CdboComOutListView::dumpSlot(){
	if(!firstChild())
		return false;
	if(lastLogFile.isEmpty()){
		lastLogFile = QDir::homeDirPath();
		lastLogFile = KFileDialog::getSaveFileName(lastLogFile,
			"*|All files", this, i18n("Save Process Output As..."));

		if(!lastLogFile || lastLogFile == "")
			return false;
	}
	QFile file(lastLogFile);

	if(file.exists())
		file.remove();
	if(!file.open(IO_ReadWrite))
		return false;

	QTextStream stream(&file);
	for(QListViewItem *i = firstChild(); i; i = i->nextSibling()){
		stream << i->text(PROGRESS_COL);
		stream << "\n";
	}
	QDate date = QDate::currentDate();
	stream << "\n\nCD Bake Oven. ";
	stream << date.toString();
	stream << ".";
	file.close();
	return true;
}
//==============================================================
void CdboComOutListView::popupMenuSlot(QListViewItem *, const QPoint &point, int){
	if(childCount() == 0)
		return;
	menu->popup( point);
}

