/***************************************************************************
                          CdboDataFolder.cpp  -  description
                             -------------------
    begin                : Sun Dec 2 2001
    copyright            : (C) 2001 by Alex Podolsky
    email                : alexpod@users.sf.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

// QT includes
#include <qapplication.h>
#include <qdir.h>
#include <qfileinfo.h>
#include <qpixmap.h>
#include <qdatetime.h>
#include <qprogressdialog.h>

// KDE includes
#include <kurl.h>
#include <kiconloader.h>
#include <klocale.h>
#include <kmimetype.h>
#include <kmessagebox.h>
#include <kconfig.h>
#include <krun.h>
#include <kdebug.h>

// app includes
#include "CdboDataFolder.h"
#include "CdboDataFoldersView.h"

//==============================================
CdboDataFolder::CdboDataFolder(CdboDataFoldersView * parent, QString name):KListViewItem(parent){
	top = true;
	setText( 0, name );
	size = 0;
	immutable = false;
	init();
}
//==============================================
CdboDataFolder::CdboDataFolder(CdboDataFolder * parent, QString name, bool immutable):
			KListViewItem(parent){
	top = false;
	setText( 0, name );
	if(immutable)
		setPixmap( 0, SmallIcon("folder_green") );
	else
		setPixmap( 0, SmallIcon("folder_red") );
	size = 0;
	this->immutable = immutable;
	init();
}
//======================================
CdboDataFolder::CdboDataFolder(CdboDataFolder *fakeFolderItem, CdboDataFolder *parent, QProgressDialog *progress):
				KListViewItem(parent){
	top = false;
	setText( 0, fakeFolderItem->getName() );
	setPixmap( 0, SmallIcon("folder_red") );
	CdboFileEntry *oldEntry = fakeFolderItem->getLastCdboFileEntry();
	CdboFileEntry *newEntry;
	int s = 0;
	while (oldEntry != 0){
		newEntry = new CdboFileEntry;
		newEntry->path = oldEntry->path;
		newEntry->name = oldEntry->name;
		newEntry->size = oldEntry->size;
		newEntry->immutable = false;
		newEntry->visibility = oldEntry->visibility;
	
		entries.append(newEntry);
		s += oldEntry->size;
		
		oldEntry = fakeFolderItem->getPrevCdboFileEntry();
		parentView()->totalFiles++;
	}
	progress->setProgress(progress->progress() + s);
	
	size = fakeFolderItem->getSize();
	immutable = false;
	init();
}
//======================================
CdboDataFolder::CdboDataFolder(CdboDataFolder *fakeFolderItem, CdboDataFoldersView *parent, QProgressDialog *progress):
				KListViewItem(parent){
	top = false;
	setText( 0, fakeFolderItem->getName() );
	setPixmap( 0, SmallIcon("folder_red") );
	CdboFileEntry *oldEntry = fakeFolderItem->getLastCdboFileEntry();
	
	CdboFileEntry *newEntry;
	int s = 0;
	while (oldEntry != 0){
		newEntry = new CdboFileEntry;
		newEntry->path = oldEntry->path;
		newEntry->name = oldEntry->name;
		newEntry->size = oldEntry->size;
		newEntry->visibility = oldEntry->visibility;
	
		newEntry->immutable = false;
		entries.append(newEntry);
		s += oldEntry->size;
		
		oldEntry = fakeFolderItem->getPrevCdboFileEntry();
		parentView()->totalFiles++;
	}
	progress->setProgress(progress->progress() + s);
	qApp->processEvents();
	size = fakeFolderItem->getSize();
	immutable = false;
	init();
}
//=======================================
void CdboDataFolder::init(){
	entries.setAutoDelete(true);
	parentView()->totalFolders++;
	holder = 0;
}
//====================================
CdboDataFolder::~CdboDataFolder(){
	if(parentView()){
		parentView()->totalFolders--;
		parentView()->totalFiles -= entries.count();
	}
	if(holder)
		holder->itemGone();
}
//=======================================
CdboDataFolderPtr CdboDataFolder::getHolder(){
	if(!holder){
		holder = new CdboDataFolderHolder(listView());
		holder->setItem(this);
	}
	return holder;
}
//=======================================
void CdboDataFolder::setOpen( bool open ){
	if(!top){
		if(immutable){
			if ( open ){
				setPixmap( 0, SmallIcon("folder_green_open") );
			}
			else{
				setPixmap( 0, SmallIcon("folder_green") );
			}
		}
		else{
			if ( open ){
				setPixmap( 0, SmallIcon("folder_red_open") );
			}
			else{
				setPixmap( 0, SmallIcon("folder_red") );
			}
		}
	}
	QListViewItem::setOpen( open );
	opened = open;
}
//=======================================
QPixmap CdboDataFolder::getIcon() {
	if(top)
		return DesktopIcon("cdrom_unmount");
		
	if(immutable)
		return DesktopIcon("folder_green");
		
	return DesktopIcon("folder_red");
}
//=======================================
QPixmap CdboDataFolder::getSmallIcon() {
	if(top)
		return SmallIcon("cdrom_unmount");
		
	if(immutable)
		return SmallIcon("folder_green");
		
	return SmallIcon("folder_red");
}
//=======================================
QString CdboDataFolder::getPathToTop(){
	return parentView()->getPathToTop(this);
}
//=======================================
void CdboDataFolder::setup(){
	setExpandable( true );
	QListViewItem::setup();
}
//===========================================================
bool CdboDataFolder::readPath( QString path, bool includeHidden ){
	QDir thisDir( path );
	QStringList filesList ;
  	
	thisDir.setSorting( QDir::Name);
		
	if(includeHidden)
		thisDir.setFilter (QDir::Hidden|QDir::Files);
	else
		thisDir.setFilter ( QDir::Files);
	
	filesList = thisDir.entryList();		

	for ( int i = 0; i < (int)filesList.count(); ++i ){
		if(!addEntry(path + "/" + filesList	[i])){
			return false;
		}
	}
	return true;
}
//=======================================
bool CdboDataFolder::addEntry(QString path){
	QFileInfo d( path );
	
	CdboFileEntry *newEntry = new CdboFileEntry;	
	newEntry->path = path;
	newEntry->name = d.fileName();
	newEntry->size = d.size();
	newEntry->immutable = false;
	newEntry->visibility = RR_Visible + Joi_Visible + HFS_Visible;
	
	addToSize((unsigned long)newEntry->size);

	entries.append(newEntry);
	parentView()->totalFiles++;
	return true;
}
//=======================================
void CdboDataFolder::addEntry(QString path, QString name, unsigned long size, bool immutable){
	
	CdboFileEntry *newEntry = new CdboFileEntry;	
	newEntry->path = path;
	newEntry->name = name;
	newEntry->size = (int)size;
	newEntry->immutable = immutable;
	newEntry->visibility = RR_Visible + Joi_Visible + HFS_Visible;
	
	addToSize(newEntry->size);
		
	entries.append(newEntry);
	parentView()->totalFiles++;
}
//=======================================
void CdboDataFolder::addEntry(CdboFileEntry *entry){
	
	CdboFileEntry *newEntry = new CdboFileEntry;	
	newEntry->path = entry->path;
	newEntry->name = entry->name;
	newEntry->size = entry->size;
	newEntry->immutable = false;
	newEntry->visibility = entry->visibility;
	
	addToSize((unsigned long)newEntry->size);
	entries.append(newEntry);
	parentView()->totalFiles++;
}
//=======================================
bool CdboDataFolder::removeEntry(QString name){
	CdboFileEntry *thisEntry;
	int i;
	for (i = 0, thisEntry=entries.first(); thisEntry != 0; thisEntry=entries.next(), i++ ){
		if(thisEntry->name == name){
			if(thisEntry->immutable == true)
				return false;
			subtractFromSize(thisEntry->size);
			
			entries.remove(i);
			parentView()->totalFiles--;
			return true;
		}
	}
	return true;
}
//=======================================
bool CdboDataFolder::removeEntry(CdboFileEntry *thisEntry){
	if(thisEntry->immutable == true)
		return false;
	subtractFromSize(thisEntry->size);
	entries.remove(thisEntry);
	parentView()->totalFiles--;
	return true;
}
//=======================================
bool CdboDataFolder::containsEntry(QString name){
	CdboFileEntry *thisEntry;
	if(entries.count() > 0){
		for (thisEntry=entries.first(); thisEntry != 0; thisEntry=entries.next()){
			if(thisEntry->name == name)
				return true;
		}
	}
	return hasChild(name);
}
//======================================
CdboFileEntry* CdboDataFolder::getEntry(QString name){

	CdboFileEntry *thisEntry;	
	for (thisEntry=entries.first(); thisEntry != 0; thisEntry=entries.next()){
		if(thisEntry->name == name)
			break;
	}
	return thisEntry;
}
//==========================================
void CdboDataFolder::generateMappings(QProgressDialog *progress, QTextStream &pathSpec, 
			QTextStream &hideHFS, QTextStream &hideJoi, QTextStream &hideRR){
	
	QString pathToTop = getPathToTop();
	int s = 0;
	
	for (CdboFileEntry *thisEntry=entries.first(); thisEntry != 0; thisEntry=entries.next()){
		if(!thisEntry->immutable){
			pathSpec << pathToTop + thisEntry->name + "=" + thisEntry->path;
			pathSpec << "\n";
			if(thisEntry->visibility < RR_Visible + Joi_Visible + HFS_Visible){
				hideHFS << thisEntry->path;
				hideHFS << "\n";
			}
			if(thisEntry->visibility < RR_Visible + Joi_Visible){
				hideJoi << thisEntry->path;
				hideJoi << "\n";
			}
			if(thisEntry->visibility < RR_Visible){
				hideRR << thisEntry->path;
				hideRR << "\n";
			}
		}
		s += thisEntry->size;
	}
	progress->setProgress(progress->progress() + s / 1000);
	qApp->processEvents();
}
//======================================
void CdboDataFolder::persist(QProgressDialog *progress, KConfig *file){
	QString pathToTop = getPathToTop();
	if(top)
		file->setGroup("top");
	else
		file->setGroup(pathToTop);
	
	file->writeEntry("Name", text(0));
	if(immutable)
			file->writeEntry("Immutable","true");
		else
			file->writeEntry("Immutable","false");
	
	QStringList list;
	for (CdboDataFolder *i = (CdboDataFolder *)firstChild(); ;i=(CdboDataFolder *)i->nextSibling()){
		if(!i)
			break;
		list.append(pathToTop + i->getName() + "/");
	}
	
	file->writeEntry("Children", list.join(";"));
	list.clear();
	
	CdboFileEntry *thisEntry;
	QStringList list2;
	int s = 0;	
	for (thisEntry=entries.first(); thisEntry != 0; thisEntry=entries.next()){
		
		list.append(thisEntry->path);
		list.append(thisEntry->name);
		list.append(QString::number(thisEntry->size));
		if(thisEntry->immutable)
			list.append("true");
		else
			list.append("false");
		list.append(QString::number(thisEntry->visibility));
		list2.append(list.join("|"));
		list.clear();
		s += thisEntry->size;
	}
	file->writeEntry("Entries", list2.join(";"));
	progress->setProgress(progress->progress() + s / 1000);
	
	//progress->setLabelText("Processing " + pathToTop);
	qApp->processEvents();
}//======================================
void CdboDataFolder::load(QProgressDialog *progress, QStringList *list){
	if(list->count() == 0)
		return;

	QStringList list2;	
	CdboFileEntry *newEntry;
	int s = 0;
	for(int i = 0; i < (int)list->count(); i++){
		list2 = QStringList::split("|", (*list)[i], false);
		newEntry = new CdboFileEntry;	
		newEntry->path = list2[0];
		newEntry->name = list2[1];
		newEntry->size = list2[2].toInt();
		if(list2[3] == "true")
			newEntry->immutable = true;
		else
			newEntry->immutable = false;
		newEntry->visibility = list2[4].toInt();
		
		addToSize(newEntry->size);	
		entries.append(newEntry);
		s += newEntry->size;
		parentView()->totalFiles++;
	}
	progress->setProgress(progress->progress() + s);
	qApp->processEvents();
}
//======================================
void CdboDataFolder::load(QStringList *list){
	if(list->count() == 0)
		return;

	QStringList list2;	
	CdboFileEntry *newEntry;
	int s = 0;
	for(int i = 0; i < (int)list->count(); i++){
		list2 = QStringList::split("|", (*list)[i], false);
		newEntry = new CdboFileEntry;	
		newEntry->path = list2[0];
		newEntry->name = list2[1];
		newEntry->size = list2[2].toInt();
		if(list2[3] == "true")
			newEntry->immutable = true;
		else
			newEntry->immutable = false;
		newEntry->visibility = list2[4].toInt();
		
		addToSize(newEntry->size);	
		entries.append(newEntry);
		s += newEntry->size;
		parentView()->totalFiles++;
	}
}
//============================================
void CdboDataFolder::addToSize(unsigned long size) {
	this->size += size;
	if(!top)
		((CdboDataFolder *)parent())->addToSize(size);
}
//============================================
void CdboDataFolder::subtractFromSize(unsigned long size) {
	this->size -= size;
	if(!top)
		((CdboDataFolder *)parent())->subtractFromSize(size);
}
//=======================================
bool CdboDataFolder::hasChild(QString name){
	if(childCount() == 0)
		return false;
	for (CdboDataFolder *i = (CdboDataFolder *)firstChild(); ; i=(CdboDataFolder *)i->nextSibling()){
		if(!i)
			break;
		if(i->getName() == name)
			return true;
	}
	return false;
}
//=======================================
CdboDataFolder *CdboDataFolder::getChild(QString name){
	if(childCount() == 0)
		return 0;
	for (CdboDataFolder *i = (CdboDataFolder *)firstChild(); ; i=(CdboDataFolder *)i->nextSibling()){
		if(!i)
			break;
		if(i->getName() == name)
			return i;
	}
	return 0;
}
//===================================================
void CdboDataFolder::paintCell(QPainter *p, const QColorGroup &cg, int col, int width, int align){
	KConfig* config = parentView()->config;
	config->setGroup("Default Settings");
	if(config->readBoolEntry("Disable Data Colors", false)){
		 QListViewItem::paintCell(p, cg, col, width, align);
		 return;
	}
	QColor color = 
		(isImmutable() == true)? config->readColorEntry("Immutable Folder Color", &(cg.base())):
				config->readColorEntry("Regular Folder Color", &(cg.base()));
    QColorGroup mycg(cg);
    mycg.setBrush(QColorGroup::Base, color);

    QListViewItem::paintCell(p, mycg, col, width, align);
}
//===================================================
void CdboDataFolder::okRename(int col){
	QString tName = getName();
	QListViewItem::okRename(col);

	if(tName == text(col))
		return;

	if(text(col).isEmpty()){
		KMessageBox::sorry(listView(),
			i18n("Name can not be empty. Please choose another name."));
		setText(col, tName);
		startRename(col);
		return;

	}
	if(text(col).find('/') != -1){
		KMessageBox::sorry(listView(),
			i18n("Entered name contains invalid character. Please choose another name."));
		setText(col, tName);
		startRename(col);
		return;
	}
	if(this == parentView()->top){
		KConfig* config = parentView()->config;
	
		config->setGroup("ISO Settings");

		if(text(col).find(".iso", -4, false) == -1)
			setText(col, text(col).stripWhiteSpace() + ".iso");
		config->writeEntry("Custom ISO Name", text(col));
		config->sync();
		parentView()->currentModifiedSlot();
		return;
	}
	QString nName = text(col);
	setText(col, tName);
	if(((CdboDataFolder*)parent())->containsEntry(nName)){
		KMessageBox::sorry(listView(),
			i18n("Item with the same name already exists in this folder. "\
				"Please choose another name."));
		//setText(col, tName);
		startRename(0);
		return;
	}
	setText(col, nName);
	parentView()->currentModifiedSlot();
}
