/***************************************************************************
                          CdboDialogCntPnl.cpp  -  description
                             -------------------
    begin                : Thu May 31 2001
    copyright            : (C) 2001 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qframe.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qdatetime.h>

#include <kstdguiitem.h>
#include <kguiitem.h>
#include <kglobal.h>
#include <kapplication.h>
#include <klocale.h>


#include "CdboDialogCntPnl.h"
#include "CdboDialogCntPnl.moc"

CdboDialogCntPnl::CdboDialogCntPnl( QWidget* parent,  const char* name, WFlags fl )
	: QWidget( parent, name, fl )
{
	if ( !name )
		setName( "CdboDialogCntPnl" );

	mainFrmLayout = new QGridLayout( this );
	mainFrmLayout->setSpacing( 5 );
	mainFrmLayout->setMargin( 9 );
	mainFrmLayout->setResizeMode(QLayout::Minimum);

	mainPnl = new QGroupBox( this, "mainPnl" );
	mainPnl->setTitle( i18n( "Process Output" ) );
	mainPnl->setColumnLayout(0, Qt::Vertical );
	mainPnl->layout()->setSpacing( 0 );
	mainPnl->layout()->setMargin( 0 );
	mainPnlLayout = new QVBoxLayout( mainPnl->layout() );
	mainPnlLayout->setAlignment( Qt::AlignTop );
	mainPnlLayout->setSpacing( 0 );
	mainPnlLayout->setMargin( 6 );

	outputView = new CdboComOutListView( mainPnl, "outputView" );
	outputView->setFrameShape( QListView::Box );
	outputView->setFrameShadow( QListView::Plain );

	mainPnlLayout->addWidget( outputView );

	statusPanel = new QFrame( mainPnl, "statusPanel" );
	statusPanel->setFrameShape( QFrame::StyledPanel );
	statusPanel->setFrameShadow( QFrame::Raised );
	statusPanelLayout = new QHBoxLayout( statusPanel );
	statusPanelLayout->setSpacing( 1 );
	statusPanelLayout->setMargin( 1 );

	statusLbl = new QLabel( statusPanel, "statusLbl" );
	statusLbl->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7,
		(QSizePolicy::SizeType)5, 0, 0,
		statusLbl->sizePolicy().hasHeightForWidth() ) );

	statusLbl->setFrameShape( QLabel::Panel );
	statusLbl->setFrameShadow( QLabel::Sunken );
	statusLbl->setText( i18n( "No recording device." ) );
	//statusLbl->setAlignment( int( QLabel::AlignCenter ) );
	statusPanelLayout->addWidget( statusLbl );

	elapsedTimeLbl = new QLabel( statusPanel, "elapsedTimeLbl" );
	elapsedTimeLbl->setFrameShape( QLabel::Panel );
	elapsedTimeLbl->setFrameShadow( QLabel::Sunken );
	elapsedTimeLbl->setText( i18n( "Elapsed Time:" ) );
	elapsedTimeLbl->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );
	statusPanelLayout->addWidget( elapsedTimeLbl );

	eTimeLbl = new QLabel( statusPanel, "eTimeLbl" );
	eTimeLbl->setFrameShape( QLabel::Panel );
	eTimeLbl->setFrameShadow( QLabel::Sunken );
	eTimeLbl->setText( i18n( " 00 min" ) );
	eTimeLbl->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignLeft ) );
	statusPanelLayout->addWidget( eTimeLbl );

	mainPnlLayout->addWidget( statusPanel );

	mainFrmLayout->addMultiCellWidget( mainPnl, 0, 1, 0, 0 );

	controlPanel = new QFrame( this, "controlPanel" );
	controlPanel->setFrameShape( QFrame::Box );
	controlPanel->setFrameShadow( QFrame::Sunken );
	QWhatsThis::add(  controlPanel, i18n( "Control panel." ) );
	controlPanelLayout = new QVBoxLayout( controlPanel );
	controlPanelLayout->setSpacing( 1 );
	controlPanelLayout->setMargin( 7 );

	startBtn = new KPushButton( KGuiItem(i18n("Start"), "launch"), controlPanel, "startBtn" );
	startBtn->setMinimumSize( QSize( 120, 40 ) );
	startBtn->setMaximumSize( QSize( 120, 40 ) );
	controlPanelLayout->addWidget( startBtn );

	cancelBtn = new KPushButton( KGuiItem(i18n("&Abort"), "cancel"), controlPanel, "cancelBtn" );
	cancelBtn->setEnabled( FALSE );
	QToolTip::add(  cancelBtn, i18n( "Interrupt running process." ) );
	cancelBtn->setMinimumSize( QSize( 120, 40 ) );
	cancelBtn->setMaximumSize( QSize( 120, 40 ) );
	controlPanelLayout->addWidget( cancelBtn );

	settingsBtn = new KPushButton( KGuiItem(i18n("&Settings"), "configure"), controlPanel, "settingsBtn" );
	settingsBtn->setMinimumSize( QSize( 120, 40 ) );
	settingsBtn->setMaximumSize( QSize( 120, 40 ) );
	QToolTip::add(  settingsBtn, i18n( "Launch Settings Dialog" ) );
	controlPanelLayout->addWidget( settingsBtn );

	dumpBtn = new KPushButton( KGuiItem(i18n("Log O&utput"), "color_line"), controlPanel, "dumpBtn" );
	dumpBtn->setMinimumSize( QSize( 120, 40 ) );
	dumpBtn->setMaximumSize( QSize( 120, 40 ) );
	dumpBtn->setEnabled(false);
	controlPanelLayout->addWidget( dumpBtn );

	helpBtn = new KPushButton( KStdGuiItem::help(), controlPanel, "helpBtn" );
	helpBtn->setMinimumSize( QSize( 120, 40 ) );
	helpBtn->setMaximumSize( QSize( 120, 40 ) );
	controlPanelLayout->addWidget( helpBtn );

	quitBtn = new KPushButton( KStdGuiItem::close(), controlPanel, "quitBtn" );
	quitBtn->setMinimumSize( QSize( 120, 40 ) );
	quitBtn->setMaximumSize( QSize( 120, 40 ) );
	quitBtn->setDefault( TRUE );
	QToolTip::add(  quitBtn, i18n( "Closes this dialog" ) );

	controlPanelLayout->addWidget( quitBtn );

	mainFrmLayout->addWidget( controlPanel, 1, 1 );
	QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
	mainFrmLayout->addItem( spacer, 0, 1 );

	// signals and slots connections
	connect( dumpBtn, SIGNAL( clicked() ), outputView, SLOT( dumpSlot() ) );
	connect( helpBtn, SIGNAL( clicked() ), this, SLOT( helpSlot() ) );
	clearOutput();
	
	outputView->readOptions();
}
//---------------------------------------------------------------------------------------------
CdboDialogCntPnl::~CdboDialogCntPnl(){
	outputView->saveOptions();
}
//---------------------------------------------------------------------------------------------
void CdboDialogCntPnl::output(int type, QString message){
	outputView->output(type, message);
}
//---------------------------------------------------------------------------------------------
void CdboDialogCntPnl::outputDirty(){
	disconnect( outputView, SIGNAL( outputDirty() ), this, SLOT( outputDirty() ) );
	dumpBtn->setEnabled(true);
	QToolTip::add(dumpBtn, i18n( "Save process output to a log file. Use context menu of 'Process Output View' to choose new location for the log.<b></b>" ));
}
//---------------------------------------------------------------------------------------------
void CdboDialogCntPnl::clearOutput(){
	outputView->clearOutput();
	dumpBtn->setEnabled(false);
	QToolTip::remove(dumpBtn);
	connect( outputView, SIGNAL( outputDirty() ), this, SLOT( outputDirty() ) );
}
////////////////////////////////// TIMER    /////////////////////////////////////////////////////////
//---------------------------------------------------------------------------------------------
void CdboDialogCntPnl::startForwardTimer(){
	eTimeId = startTimer(1000);
	timer.start();
}
//---------------------------------------------------------------------------------------------
void CdboDialogCntPnl::timerEvent(QTimerEvent *event){
	if(event->timerId() == eTimeId){
		int i = timer.elapsed() / 1000;
		if(i < 59)
			eTimeLbl->setText(i18n("%1 sec").arg( QString::number( i )));
		else
			eTimeLbl->setText(i18n("%1 min %2 sec").arg(
				QString::number( i / 60 )).arg(QString::number( i % 60 )));
		return;
	}
	if(event->timerId() == staTimeId){
		QString msg = statusLbl->text();
		if(dotCount == 7){
			dotCount = 0;
			statusLbl->setText( msg.left(msg.length() - 7));
			return;
		}
		dotCount++;
		statusLbl->setText( msg + "." );
		return;
	}
	
}
//---------------------------------------------------------------------------------------------
void CdboDialogCntPnl::status(QString msg) {
	if(msg.find(i18n("Ready")) == -1){
		if(msg.find("...", msg.length() - 3) != -1)
			msg = msg.left(msg.length() - 3) + " ";
		dotCount = 0;
		staTimeId = startTimer(400);
		timer.start();
	}
	else
		killTimer(staTimeId);
	statusLbl->setText( msg );
}
//---------------------------------------------------------------------------------------------
void CdboDialogCntPnl::stopForwardTimer(){
	killTimer(eTimeId);
	//killTimer(procTimeId);
}
//===============================================
void CdboDialogCntPnl::helpSlot(){
	kapp->invokeHelp("", "cdbakeoven");
}

