/***************************************************************************
                          CdboDriveInfoPnl.cpp  -  description
                             -------------------
    begin                : Sat May 5 2001
    copyright            : (C) 2001 by Alexy Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

 /***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qbuttongroup.h>
#include <qcombobox.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qhbox.h>
#include <qwhatsthis.h>

#include <kdialog.h>
#include <kconfig.h>
#include <kiconloader.h>
#include <kprocess.h>

#include "CdboDriveInfoPnl.h"
#include "CdboDriveInfoPnl.moc"

CdboDriveInfoPnl::CdboDriveInfoPnl( QWidget* parent,  const char* name, WFlags fl)
    : QWidget( parent, name, fl ){

	if ( !name )
		setName( "CdboDriveInfoPnl" );

	type = 0;
	config = new KConfig("cdbakeovenrc");

	driveInfoPnlLayout = new QHBoxLayout( this, 0, 0, "driveInfoPnlLayout");

	driveInfoPnlFrm = new QButtonGroup( this, "driveInfoPnlFrm" );
	driveInfoPnlFrm->setTitle( i18n( "ButtonGroup1" ) );
	driveInfoPnlFrm->setColumnLayout(0, Qt::Vertical );
	driveInfoPnlFrm->layout()->setSpacing( KDialog::spacingHint() );
	driveInfoPnlFrm->layout()->setMargin( KDialog::marginHint() );
	driveInfoPnlFrmLayout = new QGridLayout( driveInfoPnlFrm->layout() );
	driveInfoPnlFrmLayout->setAlignment( Qt::AlignTop );

	QHBox *deviceHBox = new QHBox(driveInfoPnlFrm);
	deviceHBox->setSpacing(KDialog::spacingHint());

	devicesCmb = new QComboBox( FALSE, deviceHBox, "devicesCmb" );
	devicesCmb->setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Preferred));
	devicesCmb->setDuplicatesEnabled(false);
	deviceBtn = new QPushButton( deviceHBox, "deviceBtn" );

	driveInfoPnlFrmLayout->addMultiCellWidget( deviceHBox, 0, 0, 0, 1 );

	detectBtn = new QPushButton( driveInfoPnlFrm, "detectBtn" );
	detectBtn->setText( i18n( "&Detect" ) );
	QToolTip::add( detectBtn, i18n( "Detect devices" ) );
	QWhatsThis::add( detectBtn, i18n( "Invoke detection dialog." ) );

	driveInfoPnlFrmLayout->addWidget( detectBtn, 1, 1 );

	usrBtn = new QPushButton( driveInfoPnlFrm, "usrBtn" );
	usrBtn->setText( i18n( "&Preview" ) );

	driveInfoPnlFrmLayout->addWidget( usrBtn, 1, 0 );

	driveInfoPnlLayout->addWidget( driveInfoPnlFrm );
	// signals and slots connections
	connect( deviceBtn, SIGNAL( clicked() ), this, SLOT( deviceBtnSlot() ) );
	connect( devicesCmb, SIGNAL( activated(int) ), this, SLOT( emitSelectedChaged(int) ) );
}

/*  
 *  Destroys the object and frees any allocated resources
 */
CdboDriveInfoPnl::~CdboDriveInfoPnl(){
	delete config;
}
//---------------------------------------------------------------------------
void CdboDriveInfoPnl::setType(int t) {
	if(t < 1 || t > 2) return;//invalid type
	type = t;
	if(type == Target){
		setTitle( i18n( "Destination" ) );
		QWhatsThis::add(  driveInfoPnlFrm, i18n( "Target (writing device)." ) );
		QToolTip::add(  devicesCmb, i18n( "Available recording devices." ) );
		
		QPixmap image = BarIcon("player_eject");
		setDevImage(image);
		QToolTip::add(  deviceBtn, i18n( "Eject destination media." ) );
	}
	else{
		setTitle(i18n("Source"));
		QWhatsThis::add(  driveInfoPnlFrm, i18n( "Source (reading device)." ) );
		QToolTip::add(  devicesCmb, i18n( "Available CD-ROM drives." ) );
		QPixmap image = BarIcon("player_eject");
		setDevImage(image);
		QToolTip::add(  deviceBtn, i18n( "Eject source media." ) );
	}
}
//---------------------------------------------------------------------------------------------
void CdboDriveInfoPnl::readOptions(){
	if(!type) return;
	if(config)
		delete config;
	config = new KConfig("cdbakeovenrc");
	// Always read the target list, so we can use the right icons ;)
	config->setGroup("Target Devices");
	QStringList targetsList(config->readListEntry("Targets List"));

	// Clear the combo
	devicesCmb->clear();

	if(type == Target){
		for ( QStringList::Iterator it = targetsList.begin(); it != targetsList.end(); ++it ) {
			devicesCmb->insertItem(SmallIcon("cdwriter_unmount"), *it);
		}

       	
		int pos = config->readNumEntry("Last Target", 0);
		if(pos <= devicesCmb->count())
			devicesCmb->setCurrentItem(pos);
	}
	else
	{
		config->setGroup("Source Devices");
		QStringList sourceList(config->readListEntry("Source List"));

		for ( QStringList::Iterator it = sourceList.begin(); it != sourceList.end(); ++it ) {
			if (targetsList.contains(*it))
				devicesCmb->insertItem(SmallIcon("cdwriter_unmount"), *it);
			else
				devicesCmb->insertItem(SmallIcon("cdrom_unmount"), *it);
		}
       	
		int pos = config->readNumEntry("Last Source", 0);
		if(pos <= devicesCmb->count())
			devicesCmb->setCurrentItem(pos);
	}
}
//---------------------------------------------------------------------------------------------
void CdboDriveInfoPnl::saveOptions(){
	if(!type) return;

	if(type == Target){
		config->setGroup("Target Devices");
		config->writeEntry("Last Target", devicesCmb->currentItem());
	}
	else{
		config->setGroup("Source Devices");
		config->writeEntry("Last Source", devicesCmb->currentItem());
	}
	config->sync();
}
//---------------------------------------------------------------------------------------------
QString CdboDriveInfoPnl::getScsiDevice(){
	//if(type != Target) return "";
	config->setGroup("Target Devices");
	return config->readEntry(devicesCmb->currentText(), QString::null);
}
//---------------------------------------------------------------------------------------------
QString* CdboDriveInfoPnl::scsiDevice(){
	//if(type != Target) return "";
	config->setGroup("Target Devices");
	mScsiDevice = config->readEntry(devicesCmb->currentText(), QString::null);
	if(mScsiDevice.isNull())
		return 0;
	return  &mScsiDevice;
}
//---------------------------------------------------------------------------
void CdboDriveInfoPnl::deviceBtnSlot(){
    	deviceBtn->setEnabled(false);
    	if(!ejected) eject();
	else closeTray();
}
//---------------------------------------------------------------------------------------------
void CdboDriveInfoPnl::eject(){
	cdboProc = new KProcess();
	*cdboProc << "eject" << getDriveName();
	connect(cdboProc, SIGNAL(processExited(KProcess *)),
     			this, SLOT(processComplited(KProcess *)));
	ejected = true;
	emit trayEjected();
	if(!cdboProc->start(KProcess::NotifyOnExit)){
		deviceBtn->setEnabled(true);
		delete cdboProc;
	}
}
//---------------------------------------------------------------------------------------------
void CdboDriveInfoPnl::closeTray(){
	cdboProc = new KProcess();
	*cdboProc << "eject" << "-t" << getDriveName();
	ejected = false;
	connect(cdboProc, SIGNAL(processExited(KProcess *)),
      			this, SLOT(processComplited(KProcess *)));
	if(!cdboProc->start(KProcess::NotifyOnExit)){
	//if(!cdboProc->start(KProcess::Block)){
		deviceBtn->setEnabled(true);
		delete cdboProc;
	}
	
}
//---------------------------------------------------------------------------------------------
void CdboDriveInfoPnl::closeTrayAndWait(){
	//done = false;
	cdboProc = new KProcess();
	*cdboProc << "eject" << "-t" << getDriveName();
	ejected = false;
	//connect(cdboProc, SIGNAL(processExited(KProcess *)),
      //			this, SLOT(processComplited(KProcess *)));
	cdboProc->start(KProcess::Block);
	//if(!cdboProc->start(KProcess::NotifyOnExit))
	//	done = true;
	delete cdboProc;	
}
//---------------------------------------------------------------------------------------------
void CdboDriveInfoPnl::processComplited(KProcess *){
	if(!blocked)
		deviceBtn->setEnabled(true);
	delete cdboProc;
}
//---------------------------------------------------------------------------------------------
void CdboDriveInfoPnl::blockBtns(bool yes){
	blocked = yes;
	deviceBtn->setEnabled(!yes);
	devicesCmb->setEnabled(!yes);
	usrBtn->setEnabled(!yes);
}
//=======================================================
QString CdboDriveInfoPnl::getDriveName() {
	//
	if(type != Target){
		config->setGroup("Source Devices");
		return config->readEntry(devicesCmb->currentText());
	}
	else{	
		config->setGroup("Target Devices");
		return config->readEntry(devicesCmb->currentText() + " device");
	}

	return "";
}
//=======================================================
QString CdboDriveInfoPnl::getDeviceTitle(){
	return devicesCmb->currentText();
}
//---------------------------------------------------------------------------------------------
QString* CdboDriveInfoPnl::driveName(){
	if(type != Target){
		config->setGroup("Source Devices");
		mDeviceName = config->readEntry(devicesCmb->currentText());
	}
	else{
		config->setGroup("Target Devices");
		mDeviceName = config->readEntry(devicesCmb->currentText() + " device");
	}
	return &mDeviceName;
}
//---------------------------------------------------------------------------------------------
void CdboDriveInfoPnl::setCurrent(const QString &device){
	
	config->setGroup("Target Devices");
	QStringList targetsList(config->readListEntry("Targets List"));

	for (unsigned int i = 0; i < targetsList.count(); i++ ) {
		if( config->readEntry(targetsList[i] + " device") == device)
			devicesCmb->setCurrentText(targetsList[i]);
		
	}
}

