/***************************************************************************
                          CdboFileListView.cpp  -  description
                             -------------------
    begin                : Sat Oct 21 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sourceforge.net

	 Some code here copied from Kate of kdebase.

 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qdir.h>
#include <qcursor.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qtextstream.h>

#include <kapplication.h>
#include <klocale.h>
#include <kconfig.h>
#include <kurl.h>
#include <kfileview.h>
#include <kdebug.h>
#include <kiconloader.h>
#include <kpopupmenu.h>
#include <kpropertiesdialog.h>
#include <klineeditdlg.h>
#include <kmessagebox.h>


#include <kurlcombobox.h>
#include <kurlcompletion.h>
#include <kcombobox.h>
#include <ktoolbarbutton.h>

#include <kstandarddirs.h>
#include <kbookmarkimporter.h>




#include "CdboFileListView.h"
#include "CdboFileListView.moc"

#include "kfilednddetailview.h"
#include "kfiledndiconview.h"

CdboFileListView::CdboFileListView(QWidget *parent, const char *name) :
	QSplitter(parent, name){


	this->setOrientation(QSplitter::Horizontal);

//	showHidden = false;
	currBranch = 0;
	lastFilter = QString::null;
	topLimitUrl = QString::null;

	//BEGIN ////////////////////// left //////////////////////////////////////////////
	dirsView = new CdboFileTreeView(this, "dirsView");
	dirsView->addColumn(i18n("Name"));
	dirsView->setShowFolderOpenPixmap(false);
	dirsView->setDragEnabled (true);
	dirsView->setAutoOpen(true);
	dirsView->setAcceptDrops (true);

	connect(dirsView, SIGNAL(executed(QListViewItem *)),
		this, SLOT(dirSelected(QListViewItem *)));
	connect(dirsView, SIGNAL(returnPressed(QListViewItem *)),
		this, SLOT(dirSelected(QListViewItem *)));
	connect(dirsView, SIGNAL(dropped(KURL::List&,KURL&)),
		this, SLOT(slotDropped(KURL::List&,KURL&)));

	connect(dirsView, SIGNAL(mkdir(const KURL&, const QString&, bool)),
		this, SLOT(mkdir(const KURL&, const QString&, bool)));
	connect(dirsView, SIGNAL(del(const KFileItemList&)),
		this, SLOT(del(const KFileItemList&)));


	//END ////////////////////// left //////////////////////////////////////////////


	//BEGIN ////////////////////// right //////////////////////////////////////////////
	QVBox* rightBox = new QVBox(this, "rightFrame");

	//BEGIN ////////////////////// URL cmb //////////////////////////////////////////////
	locationBox = new QHBox(rightBox);

	btnClearPath = new QToolButton( locationBox, "btnClearPath" );
	btnClearPath->setIconSet( SmallIconSet("locationbar_erase" ) );
	connect( btnClearPath, SIGNAL( clicked() ), this, SLOT( clearPathSLot() ) );

	cmbPath = new KURLComboBox( KURLComboBox::Directories, true, locationBox, "path combo" );
	cmbPath->setSizePolicy( QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed ));
	KURLCompletion* cmpl = new KURLCompletion(KURLCompletion::DirCompletion);
	cmbPath->setCompletionObject( cmpl );

	connect( cmbPath, SIGNAL( urlActivated( const KURL&  )),
		this,  SLOT( cmbPathActivated( const KURL& ) ));
	connect( cmbPath, SIGNAL( returnPressed( const QString&  )),
		this,  SLOT( cmbPathReturnPressed( const QString& ) ));


	QWhatsThis::add( cmbPath,
		i18n("<p>Here you can enter a path for a directory to display."
				"<p>To go to a directory previously entered, press the arrow on the right end and choose one."
				"<p>The entry has directory completion. Right-click to choose how completion should behave.") );
	//END ////////////////////// URL cmb //////////////////////////////////////////////



	//BEGIN ////////////////////// file view //////////////////////////////////////////////
	filesView = new CdboDirOperator( QString::null, rightBox, "filesView" );
	//filesView->setView(KFile::Default);

	connect(filesView, SIGNAL(urlEntered (const KURL&)),
		this, SLOT(slotSelectUrl(const KURL&)));
	connect(filesView, SIGNAL(dropped(KURL::List&,KURL&)),
		this, SLOT(slotDropped(KURL::List&,KURL&)));
	connect(filesView, SIGNAL(finishedLoading()),
		this, SLOT(finishedLoadingSlot()));

	//END	////////////////////// file view //////////////////////////////////////////////

	//BEGIN ////////////////////// filter //////////////////////////////////////////////
	filterBox = new QHBox(rightBox);
	btnFilter = new QToolButton( filterBox, "btnFilter");
	btnFilter->setIconSet( SmallIconSet("filter" ) );
	btnFilter->setToggleButton( true );

	filter = new KHistoryCombo( true, filterBox, "filter");
	filter->setSizePolicy( QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed ));
	filterBox->setStretchFactor(filter, 2);
	rightBox->setStretchFactor(filesView, 2);

	connect( btnFilter, SIGNAL( clicked() ), this, SLOT( btnFilterClick() ) );
	connect( filter, SIGNAL( activated(const QString&) ), SLOT( slotFilterChange(const QString&) ) );
	connect( filter, SIGNAL( returnPressed(const QString&) ),filter, SLOT( addToHistory(const QString&) ) );

	QWhatsThis::add( filter,
		i18n("<p>Here you can enter a name filter to limit which files are displayed."
				"<p>To clear the filter, toggle off the filter button to the left."
				"<p>To reapply the last filter used, toggle on the filter button." ) );
	QWhatsThis::add( btnFilter,
		i18n("<p>This button clears the name filter when toggled off, or reapplies the "
				"last filter used when toggled on.") );

	//END ////////////////////// filter //////////////////////////////////////////////

	//END ////////////////////// right //////////////////////////////////////////////

	setupActions();
}
//===================================
CdboFileListView::~CdboFileListView(){
	delete dirsView;
	delete filesView;
}

//===================================
void CdboFileListView::readOptions(KConfig* config){
	bool defaultConfig = false;
	if(!config){
		config = new KConfig("cdbakeovenrc");
		defaultConfig = true;
	}

	config->setGroup("Settings of " + QString(name()));


	setSizes(config->readIntListEntry("Sizes of Parts"));

	//BEGIN /////////// location
	if(showLocationAction->isEnabled()){
		showLocationAction->setChecked(config->readBoolEntry("Show Location", true));
		slotShowLocationPnl();
	}
	cmbPath->setMaxItems( config->readNumEntry( "pathcombo history len", 9 ) );
	cmbPath->setURLs( config->readListEntry("dir history") );

	//END /////////// location

	//BEGIN /////////// filter

	showFilterAction->setChecked(config->readBoolEntry("Show Filter", true));
	slotShowFilterPnl();

	filter->setMaxCount( config->readNumEntry( "filter history len", 9 ) );
	QStringList list = config->readListEntry("filter history");
	if(list.count() == 0){
		list.append("*.bz2 *.tar *.gz *.tgz *.zip");
		list.append("*.mp3 *.wav *.au *.ogg *.aiff *.raw");
	}
	filter->setHistoryItems(list, true );

	lastFilter = config->readEntry( "last filter" );
	QString flt("");
	if ( config->readBoolEntry( "restore last filter", true ) || kapp->isRestored() )
		flt = config->readEntry("current filter");
	filter->lineEdit()->setText( flt );
	slotFilterChange( flt );

	//END /////////// filter

	filesView->readConfig(config, name() + QString(" filesView"));

	if(showLocationAction->isEnabled())
		cmbPath->setURL(filesView->url());

	getActionCollection()->action( "reload" )->activate();


	if(defaultConfig)
		delete config;
}
//===================================
void CdboFileListView::saveOptions(KConfig* config){
	bool defaultConfig = false;
	if(!config){
		config = new KConfig("cdbakeovenrc");
		defaultConfig = true;
	}

	config->setGroup("Settings of " + QString(name()));

	config->writeEntry("Sizes of Parts", sizes());

	//BEGIN /////////// location

	config->writeEntry("Show Filter", showFilterAction->isChecked());
	if(showLocationAction->isEnabled()){
		config->writeEntry("Show Location", showLocationAction->isChecked());
		config->writeEntry( "pathcombo history len", cmbPath->maxItems() );
		QStringList l;
		for (int i = 0; i < cmbPath->count(); i++) {
			l.append( cmbPath->text( i ) );
		}
		config->writeEntry("dir history", l );
	}

	//END /////////// location

	//BEGIN /////////// filter

	config->writeEntry( "filter history len", filter->maxCount() );
	config->writeEntry( "filter history", filter->historyItems() );
	config->writeEntry( "current filter", filter->currentText() );
	config->writeEntry( "last filter", lastFilter );

	//END /////////// filter

	filesView->writeConfig(config, name() + QString(" filesView"));

	config->sync();

	if(defaultConfig)
		delete config;
}
//===================================
void CdboFileListView::setupActions(){
	filesView->setupMenu(KDirOperator::FileActions|KDirOperator::SortActions|KDirOperator::ViewActions);
	filesView->initActioins();
	//BEGIN //////////////////////// extra actions ////////////////////////////////////////////////

	showFilterAction = new KToggleAction(i18n("Show Filter"), KShortcut::null(),
			this, SLOT(slotShowFilterPnl()), getActionCollection(), "show_filter_action");

	showLocationAction = new KToggleAction(i18n("Show Location Bar"), KShortcut::null(),
			this, SLOT(slotShowLocationPnl()), getActionCollection(), "show_location_action");

	filesView->plugViewMenuAction(showLocationAction, 5);
	filesView->plugViewMenuAction(showFilterAction, 6);


	KActionMenu *acmBookmarks = new KActionMenu( i18n("&Bookmarks"), /*"bookmark",*/
			getActionCollection(), "bookmarks" );
	acmBookmarks->setDelayed( false );
	bookmarkHandler = new KBookmarkHandler( this, acmBookmarks->popupMenu() );

	connect( bookmarkHandler, SIGNAL( openURL( const QString&  )),
		this,  SLOT( openURL( const QString& ) ));

	 acmBookmarks->setEnabled(false);


	//END ////////////////////// extra actions ////////////////////////////////////////////////

	//BEGIN //////////////////////// copy menu ////////////////////////////////////////////////

	copyActionMenu = new KActionMenu( i18n("DND Menu"), getActionCollection(), "dnd menu");

	KAction *a = new KAction( i18n( "&Copy Here" ), "editcopy", KShortcut::null(), this,
				SLOT( copySlot() ), getActionCollection(), "copy_action_flv" );
	copyActionMenu->insert(a);
	a = new KAction( i18n( "&Move Here" ), KShortcut::null(), this,
				SLOT( moveSlot() ), getActionCollection(), "move_action_flv" );
	copyActionMenu->insert(a);

	copyActionMenu->insert( getActionCollection()->action("separator") );

	a = new KAction( i18n( "C&ancel" ), "cancel", KShortcut::null(), this,
		SLOT( cleanUpMoved() ), getActionCollection(), "cancel_action_flv" );
	copyActionMenu->insert(a);

	//END //////////////////////// copy menu ////////////////////////////////////////////////

	dirsView->setupActions(getActionCollection());

}
//===================================
void CdboFileListView::addBranch(QString path, QString name, const QPixmap& pix, bool open, bool dirOnly){

	currBranch = dirsView->addBranch(KURL(path), name, pix, /*showHidden*/false);
	if(dirOnly)
		dirsView->setDirOnlyMode(currBranch, true);

	if(open){
		currBranch->setOpen(true);
		filesView->setURL(KURL(path), true);
	}
}
//===================================
KAction* CdboFileListView::getAction(const char *name){
	return filesView->actionCollection()->action( name );
}
//===================================
KActionCollection* CdboFileListView::getActionCollection(){
	return filesView->actionCollection();
}
//===================================
void CdboFileListView::plugMenuAction(KAction* action){
	filesView->plugMenuAction(action);
	dirsView->plugMenuAction(action);
}
//===================================
void CdboFileListView::enableBookmarkAction(){
	KAction *ba = getActionCollection()->action( "bookmarks" );
	ba->setEnabled(true);
	filesView->plugBookmarkAction(ba);
	dirsView->plugBookmarkAction(ba);
}
//===================================
void CdboFileListView::enableAddToCDAction(){
	KAction *a = getActionCollection()->action("separator");
	filesView->prependMenuAction(a);
	dirsView->prependMenuAction(a);

	a = getActionCollection()->action( "add_to_cd_ftv" );
	a->setEnabled(true);
	dirsView->prependMenuAction(a);
	a = getActionCollection()->action( "add_to_cd_dirop" );
	a->setEnabled(true);
	filesView->prependMenuAction(a);

	connect(dirsView, SIGNAL(addToCD(KURL::List&)),
		this, SLOT(emitAddToCD(KURL::List&)));
	connect(filesView, SIGNAL(addToCD(KURL::List&)),
		this, SLOT(emitAddToCD(KURL::List&)));
}
//===================================
void CdboFileListView::adoptCopyMenu(const QString& copyStr, const QString& moveStr){
	if(!copyStr.isNull())
		getActionCollection()->action("copy_action_flv")->setText(copyStr);
	if(!moveStr.isNull())
		getActionCollection()->action("move_action_flv")->setText(moveStr);
}
//===================================
void CdboFileListView::dirSelected(QListViewItem *){

	currBranch = dirsView->currentKFileTreeViewItem()->branch();
	filesView->setURL(dirsView->currentKFileTreeViewItem()->url(), true);
}
//===================================
void CdboFileListView::slotSelectUrl(const KURL& url){
	getActionCollection()->action( "stop_load_action" )->setEnabled(true);
	if(!currBranch)
		return;

	QString urlToFind = url.path().remove(0,currBranch->rootUrl().path().length());

	KFileTreeViewItem *item = dirsView->findItem(currBranch, urlToFind + "/");
	if(!item)
		item = currBranch->root();

	if(!item->isOpen())
		dirsView->setOpen(item, true);

	dirsView->ensureItemVisible(item);
	dirsView->setCurrentItem(item);
	if(showLocationAction->isEnabled())
		cmbPath->setURL(url);
}
//===================================
void CdboFileListView::finishedLoadingSlot(){
	getActionCollection()->action( "stop_load_action" )->setEnabled(false);

	(filesView->url().path() == topLimitUrl || filesView->url().path() == "/")?
		getActionCollection()->action( "up" )->setEnabled( false ):
		getActionCollection()->action( "up" )->setEnabled( true );
}
//===================================
void CdboFileListView::slotDropped(KURL::List& listToMove, KURL& _moveToUrl){

	if(listToMove.count() == 0)
		return;
	this->listToMove = listToMove;

	if(_moveToUrl.isEmpty())
		this->moveToUrl = currBranch->root()->url();
	else
		this->moveToUrl = _moveToUrl;

	copyActionMenu->popup( QCursor::pos());
}
//===================================
void CdboFileListView::mkdir(const KURL& location, const QString& dirname, bool enter){
	filesView->setURL(location, true);
	filesView->mkdir(dirname, enter);
}
//===================================
void CdboFileListView::del(const KFileItemList& items){
	filesView->del(items);
}
//===================================
void CdboFileListView::copySlot(){
	KIO::copy(listToMove, moveToUrl);
}
//===================================
void CdboFileListView::moveSlot(){
	int i = 0;

	while(i < (int)listToMove.count()){

		if(moveToUrl == listToMove[i].url()){
			KMessageBox::sorry(this,
				i18n("You cannot move the directory into itself."));

			cleanUpMoved();
			return;
		}
		if(listToMove[i].isParentOf(moveToUrl)){
			KMessageBox::sorry(this,
				i18n("You cannot move a parent directory into its child."));
			cleanUpMoved();
			return;
		}
		i++;
	}
	KIO::move(listToMove, moveToUrl);
}
//===================================
void CdboFileListView::cleanUpMoved(){
	//listToMove.clear();
	//moveToUrl.setPath("");
}
//===================================
void CdboFileListView::clearHistory() {
	filesView->clearHistory();
}
//===================================
void CdboFileListView::slotShowFilterPnl(){
	if(showFilterAction->isChecked())
		filterBox->show();
	else
		filterBox->hide();
}
//===================================
void CdboFileListView::disableLocationPnl(){
	showLocationAction->unplugAll();
	showLocationAction->setEnabled(false);
	locationBox->hide();
}
//===================================
void CdboFileListView::slotShowLocationPnl(){

	if(showLocationAction->isChecked())
		locationBox->show();
	else
		locationBox->hide();
}
//===================================
void CdboFileListView::slotFilterChange( const QString & nf ){
	QString f = nf.stripWhiteSpace();
	bool empty = f.isEmpty() || f == "*";
	if ( empty ) {
		filesView->clearFilter();
		filter->lineEdit()->setText( QString::null );
		QToolTip::add( btnFilter, i18n("Apply Last Filter (\"%1\")").arg( lastFilter ) );
	}
	else {
		filesView->setNameFilter( f );
		lastFilter = f;
		QToolTip::add( btnFilter, i18n("Clear Filter") );
	}
	btnFilter->setOn( !empty );
	//filesView->updateDir();
	getActionCollection()->action( "reload" )->activate();
	// this will be never true after the filter has been used;)
	btnFilter->setEnabled( !( empty && lastFilter.isEmpty() ) );
}
//===================================
void CdboFileListView::btnFilterClick(){
	if ( !btnFilter->isOn() ) {
		slotFilterChange( QString::null );
	}
	else {
		filter->lineEdit()->setText( lastFilter );
		slotFilterChange( lastFilter );
	}
}
//===================================
void CdboFileListView::clearPathSLot(){
	cmbPath->setCurrentText("");
}
//===================================
void CdboFileListView::cmbPathActivated( const KURL& u ){
	cmbPathReturnPressed( u.url() );
}
//===================================
void CdboFileListView::cmbPathReturnPressed(const QString& url){
	QStringList urls = cmbPath->urls();
	urls.remove(url);
	urls.prepend(url);
	cmbPath->setURLs( urls, KURLComboBox::RemoveBottom );
	//filesView->setFocus();
	filesView->setURL( KURL(url), true );
}
//===================================
void CdboFileListView::openURL(const QString& url){

	filesView->setURL( KURL(url), true );
}
//===================================
QString CdboFileListView::activeURL(){
	if(filesView->hasFocus())
		return filesView->url().url();
	else
		return dirsView->currentURL().url();
}

//===================================
//===================================
//===================================



CdboDirOperator::CdboDirOperator(const KURL& urlName, QWidget *parent, const char* name):
	KDirOperator(urlName, parent, name){

	firstMenuInsert = true;
	bookmarksUsed = false;
	menuTopOffset = 0;
	filesViewMenu = actionCollection()->action( "home" )->container(0);
}
//===================================
CdboDirOperator::~CdboDirOperator(){
}
//===================================
void CdboDirOperator::initActioins(){

	removeAction("short view");
	removeAction("detailed view");
	removeAction("separate dirs");

	KAction* a = new KAction(i18n("Stop"), "stop", KShortcut(Key_Escape),
		this, SLOT(stopLoading()), actionCollection(), "stop_load_action");
	a->setEnabled(false);

	a = new KAction(i18n("Add to CD"), "bottom", KShortcut(Key_Plus),
		this, SLOT(addToCD()), actionCollection(), "add_to_cd_dirop");
	a->setEnabled(false);
	detailesViewAction = new KRadioAction(i18n("Detailed List View"), "view_detailed", KShortcut::null(),
			this, SLOT(setDetailView()), actionCollection(), "cdbo_detailed_view");
	detailesViewAction->setExclusiveGroup(QString::fromLatin1("CdboViewMode"));
	plugViewMenuAction(detailesViewAction);

	iconViewAction =new KRadioAction(i18n("Icon View"), "view_icon", KShortcut::null(),
			this, SLOT(setIconView()), actionCollection(), "cdbo_icon_view");
	iconViewAction->setExclusiveGroup(QString::fromLatin1("CdboViewMode"));
	plugViewMenuAction(iconViewAction);

	/*new KAction(i18n("Thumbnail View"), "view_icon", KShortcut::null(),
			filesView, SLOT(setDatailView()), getActionCollection(), "cdbo_thumbnail_view");*/

	connect(filesViewMenu, SIGNAL(aboutToShow()),
		this, SLOT(resetActions()));

}
//===================================
void CdboDirOperator::resetActions(){
	const KFileItemList *selected = selectedItems();
	if(!selected || selected->isEmpty()){
		actionCollection()->action("add_to_cd_dirop")->setEnabled(false);
	}else{
		actionCollection()->action("add_to_cd_dirop")->setEnabled(true);
	}
}
//===================================
void CdboDirOperator::removeAction(const char *name){
	KAction* a = actionCollection()->action(name);
	if(a){
		a->unplugAll();
		a->setEnabled(false);
	}
}
//===================================
void CdboDirOperator::plugViewMenuAction(KAction* action, int pos){
	KActionMenu *viewMenuAction = static_cast<KActionMenu*>(actionCollection()->action( "view menu" ));
	viewMenuAction->insert(action, pos);
}
//===================================
void CdboDirOperator::plugBookmarkAction(KAction* bookmarksAction){
	bookmarksUsed = true;
	bookmarksAction->plug( filesViewMenu, 2+menuTopOffset);
}
//===================================
void CdboDirOperator::plugMenuAction(KAction* action){
	if(firstMenuInsert){
		actionCollection()->action( "separator" )->plug( filesViewMenu, bookmarksUsed? 8+menuTopOffset: 7+menuTopOffset);
		firstMenuInsert = false;
	}
	action->plug( filesViewMenu, bookmarksUsed? 8+menuTopOffset: 7+menuTopOffset);
}
//===================================
void CdboDirOperator::prependMenuAction(KAction* action){
	action->plug( filesViewMenu,0);
	menuTopOffset++;
}
//===================================
void CdboDirOperator::readConfig( KConfig * config, const QString& group){
	KDirOperator::readConfig(config, group);
	if(!group.isNull())
		config->setGroup(group);

	if(config->readBoolEntry("Last Cdbo View was Details", true)){
		detailesViewAction->setChecked(true);
		setDetailView();
	}
	else{
		iconViewAction->setChecked(true);
		setIconView();
	}
}
//===================================
void CdboDirOperator::writeConfig( KConfig *config, const QString& group){
	KDirOperator::writeConfig(config, group);
	if(!group.isNull())
		config->setGroup(group);

	config->writeEntry("Last Cdbo View was Details", detailesViewAction->isChecked());
	config->sync();
}
//===================================
void CdboDirOperator::setDetailView(){
	dirLister()->stop();
	KFileDnDDetailView* new_view = new KFileDnDDetailView( this, "cdbo detail view");
	new_view->setViewName( i18n("Cdbo Detailed View") );

	connect(new_view, SIGNAL(dropped( KFileView*, KURL::List&)),
		this, SLOT(slotDropped(KFileView*,KURL::List&)));

	setView(new_view);
	new_view->setSelectionMode(KFile::Extended);
}
//===================================
void CdboDirOperator::setIconView(){
	dirLister()->stop();
	KFileDnDIconView* new_view = new KFileDnDIconView( this, "cdbo simple view");
		new_view->setViewName( i18n("Cdbo Icon View") );

	connect(new_view, SIGNAL(dropped( KFileView*, KURL::List&)),
		this, SLOT(slotDropped(KFileView*,KURL::List&)));

	setView(new_view);
	new_view->setSelectionMode(KFile::Extended);
}
//===================================
void CdboDirOperator::slotDropped( KFileView* /*view*/, KURL::List& list){
	KURL thisUrl = url();
	emit dropped(list, thisUrl);
}
//===================================
void CdboDirOperator::stopLoading(){
	dirLister()->stop();
	back();
}
//===================================
void CdboDirOperator::addToCD(){
	const KFileItemList *selected = selectedItems();
	if(!selected || selected->isEmpty())
		return;
	KURL::List list;
	KFileItem *item;
	KFileItemListIterator it( *selected );
	for ( ; it.current(); ++it ){
		item = it.current();
		list.append(item->url());
	}
	emit addToCD( list );
}

//===================================
//===================================
//===================================

CdboFileTreeView::CdboFileTreeView(QWidget *parent, const char *name) :
	KFileTreeView(parent, name){
	setDragEnabled (true);
	setAutoOpen(true);
	setAcceptDrops (true);
	firstMenuInsert = true;
	bookmarksUsed = false;
	menuTopOffset = 0;

	connect( this, SIGNAL(rightButtonPressed(QListViewItem*, const QPoint&, int)),
		this, SLOT(popupMenuSlot(QListViewItem*, const QPoint&, int)));

}
//===================================
CdboFileTreeView::~CdboFileTreeView(){
}
//===================================
void CdboFileTreeView::setupActions(KActionCollection *ac){
	actionCollection = ac;
	actionMenu = new KActionMenu( i18n("CdboFileTreeView Menu"), actionCollection, "CdboFileTreeView menu");

	KAction *a = new KAction(i18n("Add to CD"), "bottom", KShortcut::null(),
		this, SLOT(addToCD()), actionCollection, "add_to_cd_ftv");
	a->setEnabled(false);

	a = new KAction( i18n("New Directory..."), KShortcut::null(),
		this, SLOT( mkdir() ), actionCollection, "mkdir_ftv" );
	a->setIcon( QString::fromLatin1("folder_new") );

	actionMenu->insert(a);

	a = new KAction( i18n( "Delete" ), "editdelete", KShortcut::null(), this,
                  SLOT( deleteSlot() ), actionCollection, "delete_ftv" );

	actionMenu->insert(a);

	actionMenu->insert( actionCollection->action( "separator" ) );


	a = new KAction( i18n("Properties..."), KShortcut::null(), this,
                 SLOT(slotProperties()), actionCollection, "properties_ftv" );
	actionMenu->insert(a);

}
//===================================
void CdboFileTreeView::popupMenuSlot(QListViewItem*, const QPoint& point, int){
	if(!currentKFileTreeViewItem() && firstMenuInsert)
		return;

	if(!currentKFileTreeViewItem()){
		actionCollection->action("delete_ftv")->setEnabled(false);
		actionCollection->action("mkdir_ftv")->setEnabled(false);
		actionCollection->action("properties_ftv")->setEnabled(false);
		actionCollection->action("add_to_cd_ftv")->setEnabled(false);
	}
	else{
		actionCollection->action("delete_ftv")->setEnabled(true);
		actionCollection->action("mkdir_ftv")->setEnabled(true);
		actionCollection->action("properties_ftv")->setEnabled(true);
		actionCollection->action("add_to_cd_ftv")->setEnabled(true);
	}

	actionMenu->popup( point);
}
//===================================
void CdboFileTreeView::plugBookmarkAction(KAction* bookmarksAction){
	bookmarksUsed = true;
	actionMenu->insert( bookmarksAction, 2+menuTopOffset);
}
//===================================
void CdboFileTreeView::plugMenuAction(KAction* action){
	if(firstMenuInsert){
		actionMenu->insert( actionCollection->action( "separator" ),  bookmarksUsed? 3+menuTopOffset: 2+menuTopOffset);
		firstMenuInsert = false;
	}

	actionMenu->insert(action, bookmarksUsed? 4+menuTopOffset: 3+menuTopOffset);
}
//===================================
void CdboFileTreeView::prependMenuAction(KAction* action){
	actionMenu->insert( action,0);
	menuTopOffset++;
}
//===================================
void CdboFileTreeView::mkdir(){

	bool done = false, ok;
	QString name = i18n("New Directory");
	do{
		name = KLineEditDlg::getText(i18n("Create new directory in:") +
		QString::fromLatin1( "\n" ) +
			currentURL().prettyURL(), name, &ok, this);
		if(!ok)
			return;
		name = name.simplifyWhiteSpace();
		if(name.isEmpty()){
			int result = KMessageBox::warningYesNo(this,
					i18n("New folder name cannot be empty. "\
						"Would you like to choose another name?"),
					i18n("No Name"),
					KStdGuiItem::yes(),
					KStdGuiItem::cancel());

			switch(result){
				case KMessageBox::Yes:
					done = false;
					break;
				case KMessageBox::No:
					return;
			}
		}
		else
			done = true;
	}
	while( !done );
	emit mkdir(currentURL(), name, true );

}
//===================================
void CdboFileTreeView::deleteSlot(){
	KFileItemList list;
	list.append(currentKFileTreeViewItem()->fileItem());
	emit del( list );
}
//===================================
void CdboFileTreeView::slotProperties(){
	(void) new KPropertiesDialog(currentURL(), this, "props dlg", true);

}
//===================================
void CdboFileTreeView::addToCD(){
	KURL::List list;
	list.append(currentKFileTreeViewItem()->fileItem()->url());
	emit addToCD( list );
}
//===================================
//===================================
//===================================


//===================================
KBookmarkHandler::KBookmarkHandler( CdboFileListView *parent, KPopupMenu* kpopupmenu )
	: QObject( parent, "KBookmarkHandler" ),
		KBookmarkOwner(),
		mParent( parent ),
		m_menu( kpopupmenu ),
		m_importStream( 0L ){

	if (!m_menu)
		m_menu = new KPopupMenu( parent, "bookmark menu" );

	QString file = locate( "data", "cdbakeoven/bookmarks.xml" );
	if ( file.isEmpty() )
		file = locateLocal( "data", "cdbakeoven/bookmarks.xml" );

	KBookmarkManager *manager = KBookmarkManager::managerForFile( file, false);
	manager->setUpdate( true );
	manager->setShowNSBookmarks( false );

	m_bookmarkMenu = new KBookmarkMenu( manager, this, m_menu, 0, true );
}
//===================================
KBookmarkHandler::~KBookmarkHandler(){
    //     delete m_bookmarkMenu; ###
}
//===================================
QString KBookmarkHandler::currentURL() const{
    return mParent->activeURL();
}
//===================================
void KBookmarkHandler::slotNewBookmark( const QString& /*text*/,
                                            const QCString& url,
                                            const QString& additionalInfo ){
	*m_importStream << "<bookmark icon=\"" << KMimeType::iconForURL( url );
	*m_importStream << "\" href=\"" << QString::fromUtf8(url) << "\">\n";
	*m_importStream << "<title>" << (additionalInfo.isEmpty() ? QString::fromUtf8(url) : additionalInfo) << "</title>\n</bookmark>\n";
}
//===================================
void KBookmarkHandler::slotNewFolder( const QString& text, bool /*open*/,
                                          const QString& /*additionalInfo*/ ){
	*m_importStream << "<folder icon=\"bookmark_folder\">\n<title=\"";
	*m_importStream << text << "\">\n";
}
//===================================
void KBookmarkHandler::newSeparator(){
	*m_importStream << "<separator/>\n";
}
//===================================
void KBookmarkHandler::endFolder(){
	*m_importStream << "</folder>\n";
}
//===================================
void KBookmarkHandler::virtual_hook( int id, void* data ){
	KBookmarkOwner::virtual_hook( id, data );
}





