/***************************************************************************
                          CdboSelectDriveDialog.cpp  -  description
                             -------------------
    begin                : Sun Dec 30 2001
    copyright            : (C) 2001 by Alex Podolsky
    email                : alexpod@users.sf.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>

#include "CdboSelectDriveDialog.h"
#include "CdboSelectDriveDialog.moc"

//=================================================
CdboSelectDriveDialog::CdboSelectDriveDialog( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : KDialog( parent, name, modal, fl )
{
    if ( !name )
	setName( "CdboSelectDriveDialog" );
	setCaption( i18n( "Select Drive" ) );
	
	CdboSelectDriveDialogLayout = new QVBoxLayout( this );
	CdboSelectDriveDialogLayout->setSpacing( 5 );
	CdboSelectDriveDialogLayout->setMargin( 9 );

	devicePnl = new CdboDriveInfoPnl( this, "devicePnl" );
	devicePnl->setType(CdboDriveInfoPnl::Target);
	devicePnl->readOptions();
	devicePnl->closeTray();

	devicePnl->setUsrBtnTitle( i18n("&Load") );
	devicePnl->setTitle( i18n( "Please Select Source for the Session") );
	devicePnl->getUsrBtn()->setDefault( true );
	//devicePnl->getUsrBtn()->setAutoMask( true );

	devicePnl->getDetectBtn()->setText( i18n( "&Cancel" ) );
	QToolTip::add(  devicePnl->getDetectBtn(), i18n( "Cancel" ) );
	QWhatsThis::add(  devicePnl->getDetectBtn(), i18n( "Use to stop loading CD session." ) );
	
	CdboSelectDriveDialogLayout->addWidget( devicePnl );
    // connecting to the Drive Info Panel
	connect( devicePnl->getDetectBtn(), SIGNAL( clicked() ), this, SLOT( accept() ) );
	connect( devicePnl->getDetectBtn(), SIGNAL( clicked() ), this, SLOT( emitEmpty() ) );
	connect( devicePnl->getUsrBtn(), SIGNAL( clicked() ), this, SLOT( loadSlot() ) );
	connect( devicePnl->getUsrBtn(), SIGNAL( clicked() ), this, SLOT( accept() ) );
	
	if(!devicePnl->isReady())
		devicePnl->blockBtns(true);
	//exec();
}
//==========================================
CdboSelectDriveDialog::~CdboSelectDriveDialog(){
}
//==========================================
void CdboSelectDriveDialog::loadSlot(){
	devicePnl->saveOptions();
	emit load(devicePnl->getDriveName());
	emit loadScsi(devicePnl->getScsiDevice());
}

//==========================================
void CdboSelectDriveDialog::emitEmpty(){
	emit load(QString::null);
	emit loadScsi(QString::null);
}
