/***************************************************************************
                          CdboSpeedPnl.cpp  -  description
                             -------------------
    begin                : Tue May 22 2001
    copyright            : (C) 2001 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include "CdboSpeedPnl.h"
#include "CdboSpeedPnl.moc"

#include <qframe.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qlcdnumber.h>
#include <qpushbutton.h>
#include <qslider.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>

#include <kconfig.h>
#include <klocale.h>

CdboSpeedPnl::CdboSpeedPnl( QWidget* parent,  const char* name, WFlags fl )
    : QWidget( parent, name, fl ){

    	if ( !name )
		setName( "CdboSpeedPnl" );

	config = new KConfig("cdbakeovenrc");


    	resize( 359, 81 );
    	cdboSpeedPnlLayout = new QHBoxLayout( this );
    	cdboSpeedPnlLayout->setSpacing( 0 );
    	cdboSpeedPnlLayout->setMargin( 0 );

    	cdboSpeedPnlFrm = new QGroupBox( this, "cdboSpeedPnlFrm" );
    	cdboSpeedPnlFrm->setLineWidth( 1 );
    	cdboSpeedPnlFrm->setTitle( i18n( "Speed" ) );
    	QWhatsThis::add(  cdboSpeedPnlFrm, i18n( "You can increase or decrease 'Maximum Speed' from the preferences menu." ) );
    	cdboSpeedPnlFrm->setColumnLayout(0, Qt::Vertical );
    	cdboSpeedPnlFrm->layout()->setSpacing( 0 );
    	cdboSpeedPnlFrm->layout()->setMargin( 0 );
    	cdboSpeedPnlFrmLayout = new QHBoxLayout( cdboSpeedPnlFrm->layout() );
    	cdboSpeedPnlFrmLayout->setAlignment( Qt::AlignTop );
    	cdboSpeedPnlFrmLayout->setSpacing( 6 );
    	cdboSpeedPnlFrmLayout->setMargin( 11 );

    	speedLcd = new QLCDNumber( cdboSpeedPnlFrm, "speedLcd" );
    	speedLcd->setMinimumSize( QSize( 40, 37 ) );
    	QFont speedLcd_font(  speedLcd->font() );
    	speedLcd_font.setBold( TRUE );
    	speedLcd->setFont( speedLcd_font );
     speedLcd->setFrameShape( QLCDNumber::PopupPanel );
     speedLcd->setFrameShadow( QLCDNumber::Plain );
     speedLcd->setNumDigits( 2 );
     speedLcd->setSegmentStyle( QLCDNumber::Filled );
     speedLcd->setProperty( "intValue", 4 );
     QToolTip::add(  speedLcd, i18n( "Recording speed." ) );
     cdboSpeedPnlFrmLayout->addWidget( speedLcd );

    	spaser = new QFrame( cdboSpeedPnlFrm, "spaser" );
    	spaser->setMinimumSize( QSize( 1, 0 ) );
    	spaser->setMaximumSize( QSize( 20, 32767 ) );
    	spaser->setBackgroundOrigin( QFrame::ParentOrigin );
    	spaser->setFrameShape( QFrame::NoFrame );
    	cdboSpeedPnlFrmLayout->addWidget( spaser );

    	minLbl = new QLabel( cdboSpeedPnlFrm, "minLbl" );
    	minLbl->setMinimumSize( QSize( 0, 30 ) );
    	minLbl->setMaximumSize( QSize( 32767, 30 ) );
    	minLbl->setText( i18n( "Min" ) );
    	minLbl->setAlignment( int( QLabel::AlignBottom | QLabel::AlignRight ) );
    	cdboSpeedPnlFrmLayout->addWidget( minLbl );

    	speedSld = new QSlider( cdboSpeedPnlFrm, "speedSld" );
    	speedSld->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)0, speedSld->sizePolicy().hasHeightForWidth() ) );
    	speedSld->setMinimumSize( QSize( 170, 23 ) );
    	speedSld->setLineStep( 1 );
    	speedSld->setPageStep( 1 );
    	speedSld->setOrientation( QSlider::Horizontal );
    	speedSld->setTickmarks( QSlider::Right );
    	speedSld->setTickInterval( 1 );
    	cdboSpeedPnlFrmLayout->addWidget( speedSld );

    	maxLbl = new QLabel( cdboSpeedPnlFrm, "maxLbl" );
    	maxLbl->setMinimumSize( QSize( 0, 30 ) );
    	maxLbl->setMaximumSize( QSize( 32767, 30 ) );
    	maxLbl->setText( i18n( "Max" ) );
    	maxLbl->setAlignment( int( QLabel::AlignBottom | QLabel::AlignLeft ) );
    	cdboSpeedPnlFrmLayout->addWidget( maxLbl );
    	cdboSpeedPnlLayout->addWidget( cdboSpeedPnlFrm );

    	//readOptions();
    // signals and slots connections
    	connect( speedSld, SIGNAL( valueChanged(int) ), this, SLOT( speedChangeSlot() ) );
}

CdboSpeedPnl::~CdboSpeedPnl(){
}
//---------------------------------------------------------------------------------------------------------
void CdboSpeedPnl::saveOptions(){
     config->setGroup("Target Devices");
     config->writeEntry("Target Speed", speedSld->value());
	config->sync();
}
//---------------------------------------------------------------------------------------------
void CdboSpeedPnl::readOptions(){
	if(config)
		delete config;
	config = new KConfig("cdbakeovenrc");
	/////////////////////////////////////////////////////////////////////
     config->setGroup("Default Settings");
	int maxSpeed = config->readNumEntry("Max Write Speed", 20);
	if(maxSpeed <  8) maxSpeed = 8;
	speedSld->setMaxValue(maxSpeed);

	int steps = maxSpeed / 20;
	if(steps == 0)
		steps = 1;
	else if(steps > 2)
		steps += steps % 2;

	speedSld->setTickInterval(steps);
	speedSld->setSteps(steps, steps);

	///////////////////////////////////////////////////////////
	config->setGroup("Target Devices");
	int speed = config->readNumEntry("Target Speed", 0);
	speedSld->setValue(speed);
	speedChangeSlot();
}
/*
 *  Main event handler. Reimplemented to handle application
 *  font changes
 */
bool CdboSpeedPnl::event( QEvent* ev ){
    bool ret = QWidget::event( ev );
    if ( ev->type() == QEvent::ApplicationFontChange ) {
	QFont speedLcd_font(  speedLcd->font() );
	speedLcd_font.setBold( TRUE );
	speedLcd->setFont( speedLcd_font );
    }
    return ret;
}
//-----------------------------------------------------------------------------------
void CdboSpeedPnl::speedChangeSlot(){
    	int speed = speedSld->value();
	speedLcd->display(speed);

    QString speedComment = i18n("Speed = %1 x (%2 KB/sec).<b></b> Actual recording speed may depend on capabilities of your drive and media.").
                           arg(speed).arg(speed * 172);
	QToolTip::add(speedSld, speedComment);
}
//======================
QString* CdboSpeedPnl::speed(){
	mSpeed = QString::number(speedSld->value());
	return &mSpeed;
}
